<?php
/**
 * BEAR
 *
 * PHP versions 5
 *
 * @category  BEAR
 * @package   BEAR_Emoji
 * @author    Akihito Koriyama <koriyama@bear-project.net>
 * @copyright 2008-2011 Akihito Koriyama  All rights reserved.
 * @license   http://opensource.org/licenses/bsd-license.php BSD
 * @version   SVN: Release: @package_version@ $Id: Map.php 2486 2011-06-06 07:44:05Z koriyama@bear-project.net $ Map.php 2479 2011-06-04 03:10:01Z koriyama@bear-project.net $
 * @link      http://www.bear-project.net/
 */

/**
 * 絵文字マップ
 *
 * 連想配列でキャリアの相互の絵文字変換情報を格納します。
 * BEAR_Emojiクラスから呼び出されます。
 *
 * @category  BEAR
 * @package   BEAR_Emoji
 * @author    Akihito Koriyama <koriyama@bear-project.net>
 * @copyright 2008-2011 Akihito Koriyama  All rights reserved.
 * @license   http://opensource.org/licenses/bsd-license.php BSD
 * @version   SVN: Release: @package_version@ $Id: Map.php 2486 2011-06-06 07:44:05Z koriyama@bear-project.net $ Map.php 2479 2011-06-04 03:10:01Z koriyama@bear-project.net $
 * @link      http://www.bear-project.net/
 *
 * @Singleton
 */
class BEAR_Emoji_Map
{
    /**
     * 絵文字変換テーブル
     *
     * 変換マップ連想配列を返します。
     *
     * @return array
     * @static
     */
    public static function getEmojiConvertArray()
    {
        $table = array(
            '&#63647;' => array(
                'i' => '&#63647;',
                'e' => '&#63072;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6a, 0x0f)
            ),
            '&#63648;' => array(
                'i' => '&#63648;',
                'e' => '&#63077;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x69, 0x0f)
            ),
            '&#63649;' => array(
                'i' => '&#63649;',
                'e' => '&#63076;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6b, 0x0f)
            ),
            '&#63650;' => array(
                'i' => '&#63650;',
                'e' => '&#63069;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x68, 0x0f)
            ),
            '&#63651;' => array(
                'i' => '&#63651;',
                'e' => '&#63071;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5d, 0x0f)
            ),
            '&#63652;' => array(
                'i' => '&#63652;',
                'e' => '&#63041;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x63, 0x0f)
            ),
            '&#63653;' => array(
                'i' => '&#63653;',
                'e' => '&#63413;',
                's' => pack('C*', 0x5b, 0x96, 0xb6, 0x5d)
            ),
            '&#63654;' => array(
                'i' => '&#63654;',
                'e' => '&#62396;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x5c, 0x0f)
            ),
            '&#63655;' => array(
                'i' => '&#63655;',
                'e' => '&#63079;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x5f, 0x0f)
            ),
            '&#63656;' => array(
                'i' => '&#63656;',
                'e' => '&#63080;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x60, 0x0f)
            ),
            '&#63657;' => array(
                'i' => '&#63657;',
                'e' => '&#63081;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x61, 0x0f)
            ),
            '&#63658;' => array(
                'i' => '&#63658;',
                'e' => '&#63082;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x62, 0x0f)
            ),
            '&#63659;' => array(
                'i' => '&#63659;',
                'e' => '&#63083;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x63, 0x0f)
            ),
            '&#63660;' => array(
                'i' => '&#63660;',
                'e' => '&#63084;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x64, 0x0f)
            ),
            '&#63661;' => array(
                'i' => '&#63661;',
                'e' => '&#63085;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x65, 0x0f)
            ),
            '&#63662;' => array(
                'i' => '&#63662;',
                'e' => '&#63086;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x66, 0x0f)
            ),
            '&#63663;' => array(
                'i' => '&#63663;',
                'e' => '&#63087;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x67, 0x0f)
            ),
            '&#63664;' => array(
                'i' => '&#63664;',
                'e' => '&#63088;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x68, 0x0f)
            ),
            '&#63665;' => array(
                'i' => '&#63665;',
                'e' => '&#63089;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x69, 0x0f)
            ),
            '&#63666;' => array(
                'i' => '&#63666;',
                'e' => '&#63090;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6a, 0x0f)
            ),
            '&#63667;' => array(
                'i' => '&#63667;',
                'e' => pack('C*', 0x81, 0xac),
                's' => pack('C*', 0x81, 0xac)
            ),
            '&#63668;' => array(
                'i' => '&#63668;',
                'e' => '&#63123;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x36, 0x0f)
            ),
            '&#63669;' => array(
                'i' => '&#63669;',
                'e' => '&#63414;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x34, 0x0f)
            ),
            '&#63670;' => array(
                'i' => '&#63670;',
                'e' => '&#63120;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x35, 0x0f)
            ),
            '&#63671;' => array(
                'i' => '&#63671;',
                'e' => '&#63119;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x38, 0x0f)
            ),
            '&#63672;' => array(
                'i' => '&#63672;',
                'e' => '&#62336;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x33, 0x0f)
            ),
            '&#63673;' => array(
                'i' => '&#63673;',
                'e' => '&#63415;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x4a, 0x0f)
            ),
            '&#63674;' => array(
                'i' => '&#63674;',
                'e' => '&#63122;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x52, 0x0f)
            ),
            '&#63675;' => array(
                'i' => '&#63675;',
                'e' => '&#63416;',
                's' => pack('C*', 0x5b, 0x83, 0x7c, 0x83, 0x50, 0x83, 0x62, 0x83, 0x67, 0x83, 0x78, 0x83, 0x8b, 0x5d)
            ),
            '&#63676;' => array(
                'i' => '&#63676;',
                'e' => '&#63118;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3e, 0x0f)
            ),
            '&#63677;' => array(
                'i' => '&#63677;',
                'e' => '&#63468;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x54, 0x0f)
            ),
            '&#63678;' => array(
                'i' => '&#63678;',
                'e' => '&#63113;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x55, 0x0f)
            ),
            '&#63679;' => array(
                'i' => '&#63679;',
                'e' => '&#63114;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3b, 0x0f)
            ),
            '&#63680;' => array(
                'i' => '&#63680;',
                'e' => '&#63114;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x4e, 0x0f)
            ),
            '&#63681;' => array(
                'i' => '&#63681;',
                'e' => '&#63112;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x79, 0x0f)
            ),
            '&#63682;' => array(
                'i' => '&#63682;',
                'e' => '&#62293;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x22, 0x0f)
            ),
            '&#63683;' => array(
                'i' => '&#63683;',
                'e' => '&#63116;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3d, 0x0f)
            ),
            '&#63684;' => array(
                'i' => '&#63684;',
                'e' => '&#63108;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x56, 0x0f)
            ),
            '&#63685;' => array(
                'i' => '&#63685;',
                'e' => '&#63110;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x58, 0x0f)
            ),
            '&#63686;' => array(
                'i' => '&#63686;',
                'e' => '&#62289;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x73, 0x0f)
            ),
            '&#63687;' => array(
                'i' => '&#63687;',
                'e' => '&#62290;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x75, 0x0f)
            ),
            '&#63688;' => array(
                'i' => '&#63688;',
                'e' => '&#63107;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6d, 0x0f)
            ),
            '&#63689;' => array(
                'i' => '&#63689;',
                'e' => '&#63099;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x74, 0x0f)
            ),
            '&#63690;' => array(
                'i' => '&#63690;',
                'e' => '&#62292;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x78, 0x0f)
            ),
            '&#63691;' => array(
                'i' => '&#63691;',
                'e' => '&#63100;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x76, 0x0f)
            ),
            '&#63692;' => array(
                'i' => '&#63692;',
                'e' => '&#63374;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5a, 0x0f)
            ),
            '&#63693;' => array(
                'i' => '&#63693;',
                'e' => '&#63102;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6f, 0x0f)
            ),
            '&#63694;' => array(
                'i' => '&#63694;',
                'e' => '&#63042;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6e, 0x0f)
            ),
            '&#63695;' => array(
                'i' => '&#63695;',
                'e' => '&#63101;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x71, 0x0f)
            ),
            '&#63696;' => array(
                'i' => '&#63696;',
                'e' => '&#63109;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x63, 0x0f)
            ),
            '&#63697;' => array(
                'i' => '&#63697;',
                'e' => '&#63412;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x65, 0x0f)
            ),
            '&#63698;' => array(
                'i' => '&#63698;',
                'e' => '&#63131;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x64, 0x0f)
            ),
            '&#63699;' => array(
                'i' => '&#63699;',
                'e' => '&#63132;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x67, 0x0f)
            ),
            '&#63700;' => array(
                'i' => '&#63700;',
                'e' => '&#63151;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x40, 0x0f)
            ),
            '&#63701;' => array(
                'i' => '&#63701;',
                'e' => '&#63219;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5e, 0x0f)
            ),
            '&#63702;' => array(
                'i' => '&#63702;',
                'e' => '&#63215;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x33, 0x0f)
            ),
            '&#63703;' => array(
                'i' => '&#63703;',
                'e' => '&#63196;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5c, 0x0f)
            ),
            '&#63704;' => array(
                'i' => '&#63704;',
                'e' => '&#63216;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5d, 0x0f)
            ),
            '&#63705;' => array(
                'i' => '&#63705;',
                'e' => '&#63345;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x56, 0x0f)
            ),
            '&#63706;' => array(
                'i' => '&#63706;',
                'e' => pack('C*', 0x81, 0xac),
                's' => pack('C*', 0x81, 0xac)
            ),
            '&#63707;' => array(
                'i' => '&#63707;',
                'e' => '&#63201;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2a, 0x0f)
            ),
            '&#63708;' => array(
                'i' => '&#63708;',
                'e' => '&#63417;',
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x22, 0x0f)
            ),
            '&#63709;' => array(
                'i' => '&#63709;',
                'e' => '&#62409;',
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x23, 0x0f)
            ),
            '&#63710;' => array(
                'i' => '&#63710;',
                'e' => '&#63419;',
                's' => pack('C*', 0x81, 0xac)
            ),
            '&#63711;' => array(
                'i' => '&#63711;',
                'e' => '&#63094;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x45, 0x0f)
            ),
            '&#63712;' => array(
                'i' => '&#63712;',
                'e' => '&#63061;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2e, 0x0f)
            ),
            '&#63713;' => array(
                'i' => '&#63713;',
                'e' => '&#63062;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x28, 0x0f)
            ),
            '&#63714;' => array(
                'i' => '&#63714;',
                'e' => '&#63214;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x28, 0x0f)
            ),
            '&#63715;' => array(
                'i' => '&#63715;',
                'e' => '&#63092;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x43, 0x0f)
            ),
            '&#63716;' => array(
                'i' => '&#63716;',
                'e' => '&#63095;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x68, 0x0f)
            ),
            '&#63717;' => array(
                'i' => '&#63717;',
                'e' => '&#63420;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x34, 0x0f)
            ),
            '&#63718;' => array(
                'i' => '&#63718;',
                'e' => '&#63144;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x32, 0x0f)
            ),
            '&#63719;' => array(
                'i' => '&#63719;',
                'e' => '&#63421;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x6b, 0x0f)
            ),
            '&#63720;' => array(
                'i' => '&#63720;',
                'e' => '&#63411;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x29, 0x0f)
            ),
            '&#63721;' => array(
                'i' => '&#63721;',
                'e' => '&#63397;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2a, 0x0f)
            ),
            '&#63722;' => array(
                'i' => '&#63722;',
                'e' => '&#62309;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x21, 0x0f)
            ),
            '&#63723;' => array(
                'i' => '&#63723;',
                'e' => '&#63195;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x4a, 0x0f)
            ),
            '&#63724;' => array(
                'i' => '&#63724;',
                'e' => '&#63135;',
                's' => pack('C*', 0x5b, 0x83, 0x51, 0x81, 0x5b, 0x83, 0x80, 0x5d)
            ),
            '&#63725;' => array(
                'i' => '&#63725;',
                'e' => '&#63205;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x46, 0x0f)
            ),
            '&#63726;' => array(
                'i' => '&#63726;',
                'e' => '&#62328;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2c, 0x0f)
            ),
            '&#63727;' => array(
                'i' => '&#63727;',
                'e' => '&#63422;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2e, 0x0f)
            ),
            '&#63728;' => array(
                'i' => '&#63728;',
                'e' => '&#63423;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2d, 0x0f)
            ),
            '&#63729;' => array(
                'i' => '&#63729;',
                'e' => '&#63424;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2f, 0x0f)
            ),
            '&#63730;' => array(
                'i' => '&#63730;',
                'e' => '&#63425;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x39, 0x0f)
            ),
            '&#63731;' => array(
                'i' => '&#63731;',
                'e' => '&#63426;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x3b, 0x0f)
            ),
            '&#63732;' => array(
                'i' => '&#63732;',
                'e' => '&#62600;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x30, 0x0f)
            ),
            '&#63733;' => array(
                'i' => '&#63733;',
                'e' => '&#63427;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x31, 0x0f)
            ),
            '&#63734;' => array(
                'i' => '&#63734;',
                'e' => '&#63428;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x32, 0x0f)
            ),
            '&#63735;' => array(
                'i' => '&#63735;',
                'e' => '&#63337;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x58, 0x0f)
            ),
            '&#63736;' => array(
                'i' => '&#63736;',
                'e' => '&#63336;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x57, 0x0f)
            ),
            '&#63737;' => array(
                'i' => '&#63737;',
                'e' => '&#62443;',
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x56, 0x0f)
            ),
            '&#63738;' => array(
                'i' => '&#63738;',
                'e' => '&#62444;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x27, 0x0f)
            ),
            '&#63739;' => array(
                'i' => '&#63739;',
                'e' => '&#63191;',
                's' => pack('C*', 0x5b, 0x83, 0x81, 0x83, 0x4b, 0x83, 0x6c, 0x5d)
            ),
            '&#63740;' => array(
                'i' => '&#63740;',
                'e' => '&#63063;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2a, 0x0f)
            ),
            '&#63808;' => array(
                'i' => '&#63808;',
                'e' => '&#63429;',
                's' => pack('C*', 0x81, 0x9c)
            ),
            '&#63809;' => array(
                'i' => '&#63809;',
                'e' => '&#63430;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6c, 0x0f)
            ),
            '&#63810;' => array(
                'i' => '&#63810;',
                'e' => '&#63431;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6c, 0x0f)
            ),
            '&#63811;' => array(
                'i' => '&#63811;',
                'e' => '&#63070;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6c, 0x0f)
            ),
            '&#63812;' => array(
                'i' => '&#63812;',
                'e' => pack('C*', 0x81, 0x9b),
                's' => pack('C*', 0x81, 0x9b)
            ),
            '&#63813;' => array(
                'i' => '&#63813;',
                'e' => '&#63162;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x72, 0x0f)
            ),
            '&#63814;' => array(
                'i' => '&#63814;',
                'e' => '&#63156;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6f, 0x0f)
            ),
            '&#63815;' => array(
                'i' => '&#63815;',
                'e' => '&#63117;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3c, 0x0f)
            ),
            '&#63816;' => array(
                'i' => '&#63816;',
                'e' => '&#63138;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x53, 0x0f)
            ),
            '&#63817;' => array(
                'i' => '&#63817;',
                'e' => '&#63346;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x59, 0x0f)
            ),
            '&#63858;' => array(
                'i' => '&#63858;',
                'e' => '&#63455;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x24, 0x0f)
            ),
            '&#63859;' => array(
                'i' => '&#63859;',
                'e' => '&#62566;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x23, 0x0f)
            ),
            '&#63860;' => array(
                'i' => '&#63860;',
                'e' => '&#63225;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2b, 0x0f)
            ),
            '&#63861;' => array(
                'i' => '&#63861;',
                'e' => pack('C*', 0x5b, 0x69, 0x83, 0x82, 0x81, 0x5b, 0x83, 0x68, 0x5d),
                's' => pack('C*', 0x5b, 0x69, 0x83, 0x82, 0x81, 0x5b, 0x83, 0x68, 0x5d)
            ),
            '&#63862;' => array(
                'i' => '&#63862;',
                'e' => pack('C*', 0x5b, 0x69, 0x83, 0x82, 0x81, 0x5b, 0x83, 0x68, 0x5d),
                's' => pack('C*', 0x5b, 0x69, 0x83, 0x82, 0x81, 0x5b, 0x83, 0x68, 0x5d)
            ),
            '&#63863;' => array(
                'i' => '&#63863;',
                'e' => '&#63226;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x23, 0x0f)
            ),
            '&#63864;' => array(
                'i' => '&#63864;',
                'e' => pack('C*', 0x5b, 0x83, 0x68, 0x83, 0x52, 0x83, 0x82, 0x5d),
                's' => pack('C*', 0x5b, 0x83, 0x68, 0x83, 0x52, 0x83, 0x82, 0x5d)
            ),
            '&#63865;' => array(
                'i' => '&#63865;',
                'e' => pack(
                    'C*',
                    0x5b,
                    0x83,
                    0x68,
                    0x83,
                    0x52,
                    0x83,
                    0x82,
                    0x83,
                    0x7c,
                    0x83,
                    0x43,
                    0x83,
                    0x93,
                    0x83,
                    0x67,
                    0x5d
                ),
                's' => pack(
                    'C*',
                    0x5b,
                    0x83,
                    0x68,
                    0x83,
                    0x52,
                    0x83,
                    0x82,
                    0x83,
                    0x7c,
                    0x83,
                    0x43,
                    0x83,
                    0x93,
                    0x83,
                    0x67,
                    0x5d
                )
            ),
            '&#63866;' => array(
                'i' => '&#63866;',
                'e' => '&#63386;',
                's' => pack('C*', 0x81, 0x8f)
            ),
            '&#63867;' => array(
                'i' => '&#63867;',
                'e' => '&#63381;',
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x65, 0x82, 0x71, 0x82, 0x64, 0x82, 0x64, 0x81, 0x6e)
            ),
            '&#63868;' => array(
                'i' => '&#63868;',
                'e' => '&#62299;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x49, 0x0f)
            ),
            '&#63869;' => array(
                'i' => '&#63869;',
                'e' => '&#63218;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5f, 0x0f)
            ),
            '&#63870;' => array(
                'i' => '&#63870;',
                'e' => '&#63353;',
                's' => pack('C*', 0x81, 0xac)
            ),
            '&#63872;' => array(
                'i' => '&#63872;',
                'e' => '&#63432;',
                's' => pack('C*', 0x5b, 0x43, 0x4c, 0x5d)
            ),
            '&#63873;' => array(
                'i' => '&#63873;',
                'e' => '&#63217;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x34, 0x0f)
            ),
            '&#63874;' => array(
                'i' => '&#63874;',
                'e' => '&#63461;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x32, 0x0f)
            ),
            '&#63875;' => array(
                'i' => '&#63875;',
                'e' => '&#62445;',
                's' => pack('C*', 0x81, 0xac)
            ),
            '&#63876;' => array(
                'i' => '&#63876;',
                'e' => pack(
                    'C*',
                    0x81,
                    0x75,
                    0x83,
                    0x74,
                    0x83,
                    0x8a,
                    0x81,
                    0x5b,
                    0x83,
                    0x5f,
                    0x83,
                    0x43,
                    0x83,
                    0x84,
                    0x83,
                    0x8b,
                    0x5d
                ),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x31, 0x0f)
            ),
            '&#63877;' => array(
                'i' => '&#63877;',
                'e' => '&#62601;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x30, 0x0f)
            ),
            '&#63878;' => array(
                'i' => '&#63878;',
                'e' => '&#63304;',
                's' => pack('C*', 0x5b, 0x51, 0x5d)
            ),
            '&#63879;' => array(
                'i' => '&#63879;',
                'e' => '&#63227;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3c, 0x0f)
            ),
            '&#63880;' => array(
                'i' => '&#63880;',
                'e' => '&#63228;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3d, 0x0f)
            ),
            '&#63881;' => array(
                'i' => '&#63881;',
                'e' => '&#63296;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3e, 0x0f)
            ),
            '&#63882;' => array(
                'i' => '&#63882;',
                'e' => '&#63297;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3f, 0x0f)
            ),
            '&#63883;' => array(
                'i' => '&#63883;',
                'e' => '&#63298;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x40, 0x0f)
            ),
            '&#63884;' => array(
                'i' => '&#63884;',
                'e' => '&#63299;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x41, 0x0f)
            ),
            '&#63885;' => array(
                'i' => '&#63885;',
                'e' => '&#63300;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x42, 0x0f)
            ),
            '&#63886;' => array(
                'i' => '&#63886;',
                'e' => '&#63301;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x43, 0x0f)
            ),
            '&#63887;' => array(
                'i' => '&#63887;',
                'e' => '&#63302;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x44, 0x0f)
            ),
            '&#63888;' => array(
                'i' => '&#63888;',
                'e' => '&#63433;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x45, 0x0f)
            ),
            '&#63920;' => array(
                'i' => '&#63920;',
                'e' => '&#63434;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6d, 0x0f)
            ),
            '&#63889;' => array(
                'i' => '&#63889;',
                'e' => '&#63410;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x42, 0x0f)
            ),
            '&#63890;' => array(
                'i' => '&#63890;',
                'e' => '&#62585;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x47, 0x0f)
            ),
            '&#63891;' => array(
                'i' => '&#63891;',
                'e' => '&#63055;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x43, 0x0f)
            ),
            '&#63892;' => array(
                'i' => '&#63892;',
                'e' => '&#63056;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x47, 0x0f)
            ),
            '&#63893;' => array(
                'i' => '&#63893;',
                'e' => '&#63049;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x77, 0x0f)
            ),
            '&#63894;' => array(
                'i' => '&#63894;',
                'e' => '&#63050;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x79, 0x0f)
            ),
            '&#63895;' => array(
                'i' => '&#63895;',
                'e' => '&#62356;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x78, 0x0f)
            ),
            '&#63896;' => array(
                'i' => '&#63896;',
                'e' => '&#62359;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x27, 0x0f)
            ),
            '&#63897;' => array(
                'i' => '&#63897;',
                'e' => '&#63435;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x26, 0x0f)
            ),
            '&#63898;' => array(
                'i' => '&#63898;',
                'e' => '&#62446;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x56, 0x0f)
            ),
            '&#63899;' => array(
                'i' => '&#63899;',
                'e' => '&#63470;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5e, 0x0f)
            ),
            '&#63900;' => array(
                'i' => '&#63900;',
                'e' => '&#63125;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x43, 0x0f)
            ),
            '&#63901;' => array(
                'i' => '&#63901;',
                'e' => pack('C*', 0x81, 0xac),
                's' => pack('C*', 0x81, 0xac)
            ),
            '&#63902;' => array(
                'i' => '&#63902;',
                'e' => '&#63172;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x23, 0x0f)
            ),
            '&#63903;' => array(
                'i' => '&#63903;',
                'e' => '&#62334;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4e, 0x0f)
            ),
            '&#63904;' => array(
                'i' => '&#63904;',
                'e' => '&#63054;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2f, 0x0f)
            ),
            '&#63905;' => array(
                'i' => '&#63905;',
                'e' => '&#63166;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x54, 0x0f)
            ),
            '&#63906;' => array(
                'i' => '&#63906;',
                'e' => '&#63180;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2d, 0x0f)
            ),
            '&#63907;' => array(
                'i' => '&#63907;',
                'e' => '&#63058;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x31, 0x0f)
            ),
            '&#63908;' => array(
                'i' => '&#63908;',
                'e' => '&#63198;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x46, 0x0f)
            ),
            '&#63909;' => array(
                'i' => '&#63909;',
                'e' => '&#62447;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x58, 0x0f)
            ),
            '&#63910;' => array(
                'i' => '&#63910;',
                'e' => '&#63053;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5c, 0x0f)
            ),
            '&#63911;' => array(
                'i' => '&#63911;',
                'e' => '&#63066;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x41, 0x0f)
            ),
            '&#63912;' => array(
                'i' => '&#63912;',
                'e' => '&#62448;',
                's' => pack('C*', 0x81, 0x49, 0x81, 0x48)
            ),
            '&#63913;' => array(
                'i' => '&#63913;',
                'e' => '&#62449;',
                's' => pack('C*', 0x81, 0x49, 0x81, 0x49)
            ),
            '&#63914;' => array(
                'i' => '&#63914;',
                'e' => '&#63437;',
                's' => pack('C*', 0x81, 0xac)
            ),
            '&#63915;' => array(
                'i' => '&#63915;',
                'e' => '&#63438;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x51, 0x0f)
            ),
            '&#63916;' => array(
                'i' => '&#63916;',
                'e' => '&#63167;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x51, 0x0f)
            ),
            '&#63917;' => array(
                'i' => '&#63917;',
                'e' => '&#63181;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x50, 0x0f)
            ),
            '&#63918;' => array(
                'i' => '&#63918;',
                'e' => pack('C*', 0x81, 0xac),
                's' => pack('C*', 0x81, 0xac)
            ),
            '&#63919;' => array(
                'i' => '&#63919;',
                'e' => '&#62450;',
                's' => pack('C*', 0x81, 0xac)
            ),
            '&#63824;' => array(
                'i' => '&#63824;',
                'e' => '&#63127;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x44, 0x0f)
            ),
            '&#63825;' => array(
                'i' => '&#63825;',
                'e' => pack('C*', 0x5b, 0x82, 0xd3, 0x82, 0xad, 0x82, 0xeb, 0x5d),
                's' => pack('C*', 0x5b, 0x82, 0xd3, 0x82, 0xad, 0x82, 0xeb, 0x5d)
            ),
            '&#63826;' => array(
                'i' => '&#63826;',
                'e' => '&#63450;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x79, 0x83, 0x93, 0x81, 0x6e)
            ),
            '&#63829;' => array(
                'i' => '&#63829;',
                'e' => pack('C*', 0x81, 0xac),
                's' => pack('C*', 0x81, 0xac)
            ),
            '&#63830;' => array(
                'i' => '&#63830;',
                'e' => pack('C*', 0x5b, 0x82, 0xa2, 0x82, 0xb7, 0x5d),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x3f, 0x0f)
            ),
            '&#63831;' => array(
                'i' => '&#63831;',
                'e' => '&#62405;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x6b, 0x0f)
            ),
            '&#63835;' => array(
                'i' => '&#63835;',
                'e' => pack('C*', 0x5b, 0x53, 0x4f, 0x4f, 0x4e, 0x5d),
                's' => pack('C*', 0x5b, 0x53, 0x4f, 0x4f, 0x4e, 0x5d)
            ),
            '&#63836;' => array(
                'i' => '&#63836;',
                'e' => pack('C*', 0x5b, 0x4f, 0x4e, 0x5d),
                's' => pack('C*', 0x5b, 0x4f, 0x4e, 0x5d)
            ),
            '&#63837;' => array(
                'i' => '&#63837;',
                'e' => pack('C*', 0x5b, 0x65, 0x6e, 0x64, 0x5d),
                's' => pack('C*', 0x5b, 0x65, 0x6e, 0x64, 0x5d)
            ),
            '&#63838;' => array(
                'i' => '&#63838;',
                'e' => '&#63409;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x4d, 0x0f)
            ),
            '&#63921;' => array(
                'i' => '&#63921;',
                'e' => pack('C*', 0x5b, 0x82, 0x89, 0x83, 0x41, 0x83, 0x76, 0x83, 0x8a, 0x5d),
                's' => pack('C*', 0x5b, 0x82, 0x89, 0x83, 0x41, 0x83, 0x76, 0x83, 0x8a, 0x5d)
            ),
            '&#63922;' => array(
                'i' => '&#63922;',
                'e' => pack('C*', 0x5b, 0x82, 0x89, 0x83, 0x41, 0x83, 0x76, 0x83, 0x8a, 0x5d),
                's' => pack('C*', 0x5b, 0x82, 0x89, 0x83, 0x41, 0x83, 0x76, 0x83, 0x8a, 0x5d)
            ),
            '&#63923;' => array(
                'i' => '&#63923;',
                'e' => '&#63462;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x26, 0x0f)
            ),
            '&#63924;' => array(
                'i' => '&#63924;',
                'e' => '&#63197;',
                's' => pack('C*', 0x5b, 0x8d, 0xe0, 0x95, 0x7a, 0x5d)
            ),
            '&#63925;' => array(
                'i' => '&#63925;',
                'e' => '&#63202;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x3c, 0x0f)
            ),
            '&#63926;' => array(
                'i' => '&#63926;',
                'e' => '&#62587;',
                's' => pack('C*', 0x5b, 0x83, 0x57, 0x81, 0x5b, 0x83, 0x93, 0x83, 0x59, 0x5d)
            ),
            '&#63927;' => array(
                'i' => '&#63927;',
                'e' => '&#63121;',
                's' => pack('C*', 0x5b, 0x83, 0x58, 0x83, 0x6d, 0x83, 0x7b, 0x5d)
            ),
            '&#63928;' => array(
                'i' => '&#63928;',
                'e' => '&#63211;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x45, 0x0f)
            ),
            '&#63929;' => array(
                'i' => '&#63929;',
                'e' => pack('C*', 0x5b, 0x83, 0x68, 0x83, 0x41, 0x5d),
                's' => pack('C*', 0x5b, 0x83, 0x68, 0x83, 0x41, 0x5d)
            ),
            '&#63930;' => array(
                'i' => '&#63930;',
                'e' => '&#63136;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x4f, 0x0f)
            ),
            '&#63931;' => array(
                'i' => '&#63931;',
                'e' => '&#63464;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2c, 0x0f)
            ),
            '&#63932;' => array(
                'i' => '&#63932;',
                'e' => '&#62588;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x23, 0x0f, 0x1b, 0x24, 0x4f, 0x48, 0x0f)
            ),
            '&#63933;' => array(
                'i' => '&#63933;',
                'e' => '&#63396;',
                's' => pack('C*', 0x5b, 0x83, 0x8c, 0x83, 0x93, 0x83, 0x60, 0x5d)
            ),
            '&#63934;' => array(
                'i' => '&#63934;',
                'e' => '&#63097;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x21, 0x0f)
            ),
            '&#63935;' => array(
                'i' => '&#63935;',
                'e' => '&#63481;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2e, 0x0f)
            ),
            '&#63936;' => array(
                'i' => '&#63936;',
                'e' => '&#63213;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x54, 0x0f)
            ),
            '&#63937;' => array(
                'i' => '&#63937;',
                'e' => '&#63060;',
                's' => pack('C*', 0x5b, 0x8d, 0xbb, 0x8e, 0x9e, 0x8c, 0x76, 0x5d)
            ),
            '&#63938;' => array(
                'i' => '&#63938;',
                'e' => '&#63111;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x56, 0x0f)
            ),
            '&#63939;' => array(
                'i' => '&#63939;',
                'e' => '&#62338;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x58, 0x0f)
            ),
            '&#63940;' => array(
                'i' => '&#63940;',
                'e' => '&#63383;',
                's' => pack('C*', 0x5b, 0x98, 0x72, 0x8e, 0x9e, 0x8c, 0x76, 0x5d)
            ),
            '&#63941;' => array(
                'i' => '&#63941;',
                'e' => '&#62356;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x23, 0x0f)
            ),
            '&#63942;' => array(
                'i' => '&#63942;',
                'e' => '&#62361;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2a, 0x0f)
            ),
            '&#63943;' => array(
                'i' => '&#63943;',
                'e' => '&#63049;&#63438;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x35, 0x0f, 0x1b, 0x24, 0x4f, 0x51, 0x0f)
            ),
            '&#63944;' => array(
                'i' => '&#63944;',
                'e' => '&#63478;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x28, 0x0f)
            ),
            '&#63945;' => array(
                'i' => '&#63945;',
                'e' => '&#62561;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x36, 0x0f)
            ),
            '&#63946;' => array(
                'i' => '&#63946;',
                'e' => '&#62365;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2e, 0x0f)
            ),
            '&#63947;' => array(
                'i' => '&#63947;',
                'e' => '&#63476;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x26, 0x0f)
            ),
            '&#63948;' => array(
                'i' => '&#63948;',
                'e' => '&#63186;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2e, 0x0f)
            ),
            '&#63949;' => array(
                'i' => '&#63949;',
                'e' => '&#63168;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x25, 0x0f)
            ),
            '&#63950;' => array(
                'i' => '&#63950;',
                'e' => '&#63475;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x25, 0x0f)
            ),
            '&#63951;' => array(
                'i' => '&#63951;',
                'e' => '&#62361;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2a, 0x0f)
            ),
            '&#63952;' => array(
                'i' => '&#63952;',
                'e' => '&#62358;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x26, 0x0f)
            ),
            '&#63953;' => array(
                'i' => '&#63953;',
                'e' => '&#62355;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x22, 0x0f)
            ),
            '&#63954;' => array(
                'i' => '&#63954;',
                'e' => '&#63051;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x31, 0x0f)
            ),
            '&#63955;' => array(
                'i' => '&#63955;',
                'e' => '&#62573;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x33, 0x0f)
            ),
            '&#63956;' => array(
                'i' => '&#63956;',
                'e' => pack('C*', 0x5b, 0x82, 0x6d, 0x82, 0x66, 0x5d),
                's' => pack('C*', 0x5b, 0x82, 0x6d, 0x82, 0x66, 0x5d)
            ),
            '&#63957;' => array(
                'i' => '&#63957;',
                'e' => '&#63096;',
                's' => pack('C*', 0x5b, 0x83, 0x4e, 0x83, 0x8a, 0x83, 0x62, 0x83, 0x76, 0x5d)
            ),
            '&#63958;' => array(
                'i' => '&#63958;',
                'e' => '&#63348;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6e, 0x0f)
            ),
            '&#63959;' => array(
                'i' => '&#63959;',
                'e' => '&#63338;',
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x57, 0x0f)
            ),
            '&#63960;' => array(
                'i' => '&#63960;',
                'e' => '&#63043;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x35, 0x0f)
            ),
            '&#63961;' => array(
                'i' => '&#63961;',
                'e' => '&#63178;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x35, 0x0f)
            ),
            '&#63962;' => array(
                'i' => '&#63962;',
                'e' => '&#62589;',
                's' => pack('C*', 0x81, 0xac)
            ),
            '&#63963;' => array(
                'i' => '&#63963;',
                'e' => '&#63349;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6f, 0x0f)
            ),
            '&#63964;' => array(
                'i' => '&#63964;',
                'e' => '&#63065;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x72, 0x0f)
            ),
            '&#63965;' => array(
                'i' => '&#63965;',
                'e' => pack('C*', 0x5b, 0x8b, 0xd6, 0x5d),
                's' => pack('C*', 0x5b, 0x8b, 0xd6, 0x5d)
            ),
            '&#63966;' => array(
                'i' => '&#63966;',
                'e' => '&#62301;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4b, 0x0f)
            ),
            '&#63967;' => array(
                'i' => '&#63967;',
                'e' => pack('C*', 0x5b, 0x8d, 0x87, 0x5d),
                's' => pack('C*', 0x5b, 0x8d, 0x87, 0x5d)
            ),
            '&#63968;' => array(
                'i' => '&#63968;',
                'e' => '&#62300;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4a, 0x0f)
            ),
            '&#63969;' => array(
                'i' => '&#63969;',
                'e' => '&#62590;',
                's' => pack('C*', 0x81, 0xcc)
            ),
            '&#63970;' => array(
                'i' => '&#63970;',
                'e' => '&#62592;',
                's' => pack('C*', 0x81, 0xac)
            ),
            '&#63971;' => array(
                'i' => '&#63971;',
                'e' => '&#62291;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x77, 0x0f)
            ),
            '&#63972;' => array(
                'i' => '&#63972;',
                'e' => '&#62593;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x5e, 0x0f)
            ),
            '&#63973;' => array(
                'i' => '&#63973;',
                'e' => '&#63469;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5b, 0x0f)
            ),
            '&#63974;' => array(
                'i' => '&#63974;',
                'e' => '&#63212;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x30, 0x0f)
            ),
            '&#63975;' => array(
                'i' => '&#63975;',
                'e' => '&#63147;',
                's' => pack('C*', 0x5b, 0x83, 0x60, 0x83, 0x46, 0x83, 0x8a, 0x81, 0x5b, 0x5d)
            ),
            '&#63976;' => array(
                'i' => '&#63976;',
                'e' => '&#63165;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x24, 0x0f)
            ),
            '&#63977;' => array(
                'i' => '&#63977;',
                'e' => '&#62454;',
                's' => pack('C*', 0x5b, 0x83, 0x6f, 0x83, 0x69, 0x83, 0x69, 0x5d)
            ),
            '&#63978;' => array(
                'i' => '&#63978;',
                'e' => '&#62349;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x65, 0x0f)
            ),
            '&#63979;' => array(
                'i' => '&#63979;',
                'e' => '&#62594;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x30, 0x0f)
            ),
            '&#63980;' => array(
                'i' => '&#63980;',
                'e' => '&#63143;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x38, 0x0f)
            ),
            '&#63981;' => array(
                'i' => '&#63981;',
                'e' => '&#63139;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x50, 0x0f)
            ),
            '&#63982;' => array(
                'i' => '&#63982;',
                'e' => '&#63150;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x62, 0x0f)
            ),
            '&#63983;' => array(
                'i' => '&#63983;',
                'e' => '&#63145;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x66, 0x0f)
            ),
            '&#63984;' => array(
                'i' => '&#63984;',
                'e' => '&#62314;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2b, 0x0f)
            ),
            '&#63985;' => array(
                'i' => '&#63985;',
                'e' => '&#63441;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x60, 0x0f)
            ),
            '&#63986;' => array(
                'i' => '&#63986;',
                'e' => '&#62339;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x59, 0x0f)
            ),
            '&#63987;' => array(
                'i' => '&#63987;',
                'e' => '&#62595;',
                's' => pack('C*', 0x5b, 0x83, 0x4a, 0x83, 0x5e, 0x83, 0x63, 0x83, 0x80, 0x83, 0x8a, 0x5d)
            ),
            '&#63988;' => array(
                'i' => '&#63988;',
                'e' => '&#63161;',
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x43, 0x0f)
            ),
            '&#63989;' => array(
                'i' => '&#63989;',
                'e' => '&#63157;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x75, 0x0f)
            ),
            '&#63990;' => array(
                'i' => '&#63990;',
                'e' => '&#63090;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x39, 0x0f)
            ),
            '&#63991;' => array(
                'i' => '&#63991;',
                'e' => '&#62369;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x76, 0x0f)
            ),
            '&#63992;' => array(
                'i' => '&#63992;',
                'e' => '&#62597;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x24, 0x0f)
            ),
            '&#63993;' => array(
                'i' => '&#63993;',
                'e' => '&#63153;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3a, 0x0f)
            ),
            '&#63994;' => array(
                'i' => '&#63994;',
                'e' => '&#63159;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2b, 0x0f)
            ),
            '&#63995;' => array(
                'i' => '&#63995;',
                'e' => '&#63130;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x64, 0x0f)
            ),
            '&#63996;' => array(
                'i' => '&#63996;',
                'e' => '&#63477;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x27, 0x0f)
            ),
            '&#63065;' => array(
                'e' => '&#63065;',
                'i' => '&#63964;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x72, 0x0f)
            ),
            '&#63066;' => array(
                'e' => '&#63066;',
                'i' => '&#63911;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x41, 0x0f)
            ),
            '&#63067;' => array(
                'e' => '&#63067;',
                'i' => pack('C*', 0x81, 0x48),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x40, 0x0f)
            ),
            '&#63304;' => array(
                'e' => '&#63304;',
                'i' => '&#63878;',
                's' => pack('C*', 0x5b, 0x51, 0x5d)
            ),
            '&#63305;' => array(
                'e' => '&#63305;',
                'i' => pack('C*', 0x81, 0x83),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x5b, 0x0f)
            ),
            '&#63306;' => array(
                'e' => '&#63306;',
                'i' => pack('C*', 0x81, 0x84),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x5a, 0x0f)
            ),
            '&#63307;' => array(
                'e' => '&#63307;',
                'i' => pack('C*', 0x81, 0x83, 0x81, 0x83),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x5d, 0x0f)
            ),
            '&#63308;' => array(
                'e' => '&#63308;',
                'i' => pack('C*', 0x81, 0x84, 0x81, 0x84),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x5c, 0x0f)
            ),
            '&#63309;' => array(
                'e' => '&#63309;',
                'i' => pack('C*', 0x81, 0xa1),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3b, 0x0f)
            ),
            '&#63310;' => array(
                'e' => '&#63310;',
                'i' => pack('C*', 0x81, 0xa1),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3a, 0x0f)
            ),
            '&#63311;' => array(
                'e' => '&#63311;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x89, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x89, 0x81, 0x6e)
            ),
            '&#63130;' => array(
                'e' => '&#63130;',
                'i' => '&#63995;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x64, 0x0f)
            ),
            '&#63210;' => array(
                'e' => '&#63210;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x58, 0x83, 0x73, 0x81, 0x5b, 0x83, 0x4a, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x61, 0x0f)
            ),
            '&#63382;' => array(
                'e' => '&#63382;',
                'i' => '&#63930;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x55, 0x0f)
            ),
            '&#63070;' => array(
                'e' => '&#63070;',
                'i' => '&#63811;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6c, 0x0f)
            ),
            '&#63071;' => array(
                'e' => '&#63071;',
                'i' => '&#63651;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5d, 0x0f)
            ),
            '&#63312;' => array(
                'e' => '&#63312;',
                'i' => pack('C*', 0x81, 0xa1),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3b, 0x0f)
            ),
            '&#63313;' => array(
                'e' => '&#63313;',
                'i' => pack('C*', 0x81, 0xa1),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3a, 0x0f)
            ),
            '&#63314;' => array(
                'e' => '&#63314;',
                'i' => pack('C*', 0x81, 0x9f),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3b, 0x0f)
            ),
            '&#63315;' => array(
                'e' => '&#63315;',
                'i' => pack('C*', 0x81, 0x9f),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3b, 0x0f)
            ),
            '&#63316;' => array(
                'e' => '&#63316;',
                'i' => pack('C*', 0x81, 0xa1),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3b, 0x0f)
            ),
            '&#63317;' => array(
                'e' => '&#63317;',
                'i' => pack('C*', 0x81, 0xa1),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3a, 0x0f)
            ),
            '&#63318;' => array(
                'e' => '&#63318;',
                'i' => pack('C*', 0x81, 0x9c),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x39, 0x0f)
            ),
            '&#63319;' => array(
                'e' => '&#63319;',
                'i' => pack('C*', 0x81, 0x9c),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x39, 0x0f)
            ),
            '&#63383;' => array(
                'e' => '&#63383;',
                'i' => '&#63940;',
                's' => pack('C*', 0x81, 0x6d, 0x98, 0x72, 0x8e, 0x9e, 0x8c, 0x76, 0x81, 0x6e)
            ),
            '&#63320;' => array(
                'e' => '&#63320;',
                'i' => pack('C*', 0x81, 0x7b),
                's' => pack('C*', 0x81, 0x7b)
            ),
            '&#63321;' => array(
                'e' => '&#63321;',
                'i' => pack('C*', 0x81, 0x7c),
                's' => pack('C*', 0x81, 0x7c)
            ),
            '&#63322;' => array(
                'e' => '&#63322;',
                'i' => pack('C*', 0x81, 0x96),
                's' => pack('C*', 0x81, 0x96)
            ),
            '&#63323;' => array(
                'e' => '&#63323;',
                'i' => pack('C*', 0x81, 0xaa),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x52, 0x0f)
            ),
            '&#63324;' => array(
                'e' => '&#63324;',
                'i' => pack('C*', 0x81, 0xab),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x53, 0x0f)
            ),
            '&#63325;' => array(
                'e' => '&#63325;',
                'i' => '&#63965;',
                's' => pack('C*', 0x81, 0x6d, 0x8b, 0xd6, 0x8e, 0x7e, 0x81, 0x6e)
            ),
            '&#63326;' => array(
                'e' => '&#63326;',
                'i' => pack('C*', 0x81, 0xa5),
                's' => pack('C*', 0x81, 0xa5)
            ),
            '&#63327;' => array(
                'e' => '&#63327;',
                'i' => pack('C*', 0x81, 0xa3),
                's' => pack('C*', 0x81, 0xa3)
            ),
            '&#63328;' => array(
                'e' => '&#63328;',
                'i' => pack('C*', 0x81, 0xa5),
                's' => pack('C*', 0x81, 0xa5)
            ),
            '&#63329;' => array(
                'e' => '&#63329;',
                'i' => pack('C*', 0x81, 0xa3),
                's' => pack('C*', 0x81, 0xa3)
            ),
            '&#63330;' => array(
                'e' => '&#63330;',
                'i' => pack('C*', 0x81, 0x9f),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3b, 0x0f)
            ),
            '&#63331;' => array(
                'e' => '&#63331;',
                'i' => pack('C*', 0x81, 0x9f),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3b, 0x0f)
            ),
            '&#63332;' => array(
                'e' => '&#63332;',
                'i' => pack('C*', 0x81, 0xa1),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3b, 0x0f)
            ),
            '&#63333;' => array(
                'e' => '&#63333;',
                'i' => pack('C*', 0x81, 0xa1),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3a, 0x0f)
            ),
            '&#63334;' => array(
                'e' => '&#63334;',
                'i' => pack('C*', 0x81, 0x9c),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x39, 0x0f)
            ),
            '&#63335;' => array(
                'e' => '&#63335;',
                'i' => pack('C*', 0x81, 0x9c),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x39, 0x0f)
            ),
            '&#63336;' => array(
                'e' => '&#63336;',
                'i' => '&#63736;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x57, 0x0f)
            ),
            '&#63337;' => array(
                'e' => '&#63337;',
                'i' => '&#63735;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x58, 0x0f)
            ),
            '&#63072;' => array(
                'e' => '&#63072;',
                'i' => '&#63647;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6a, 0x0f)
            ),
            '&#63123;' => array(
                'e' => '&#63123;',
                'i' => '&#63668;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x36, 0x0f)
            ),
            '&#63409;' => array(
                'e' => '&#63409;',
                'i' => '&#63838;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x4e, 0x0f)
            ),
            '&#63073;' => array(
                'e' => '&#63073;',
                'i' => '&#63811;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6c, 0x0f)
            ),
            '&#63211;' => array(
                'e' => '&#63211;',
                'i' => '&#63928;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x45, 0x0f)
            ),
            '&#63356;' => array(
                'e' => '&#63356;',
                'i' => pack('C*', 0x83, 0xd3),
                's' => pack('C*', 0x83, 0xd3)
            ),
            '&#63187;' => array(
                'e' => '&#63187;',
                'i' => '&#63893;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x22, 0x0f)
            ),
            '&#63410;' => array(
                'e' => '&#63410;',
                'i' => '&#63889;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x42, 0x0f)
            ),
            '&#63131;' => array(
                'e' => '&#63131;',
                'i' => '&#63698;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x64, 0x0f)
            ),
            '&#63212;' => array(
                'e' => '&#63212;',
                'i' => '&#63974;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x30, 0x0f)
            ),
            '&#63338;' => array(
                'e' => '&#63338;',
                'i' => '&#63959;',
                's' => pack('C*', 0x35, 0x37, 0x38)
            ),
            '&#63339;' => array(
                'e' => '&#63339;',
                'i' => pack('C*', 0x81, 0x7e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x53, 0x0f)
            ),
            '&#63357;' => array(
                'e' => '&#63357;',
                'i' => '&#63722;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x21, 0x0f)
            ),
            '&#63384;' => array(
                'e' => '&#63384;',
                'i' => '&#63937;',
                's' => pack('C*', 0x81, 0x6d, 0x8d, 0xbb, 0x8e, 0x9e, 0x8c, 0x76, 0x81, 0x6e)
            ),
            '&#63060;' => array(
                'e' => '&#63060;',
                'i' => '&#63937;',
                's' => pack('C*', 0x81, 0x6d, 0x8d, 0xbb, 0x8e, 0x9e, 0x8c, 0x76, 0x81, 0x6e)
            ),
            '&#63358;' => array(
                'e' => '&#63358;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x74, 0x83, 0x8d, 0x83, 0x62, 0x83, 0x73, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x36, 0x0f)
            ),
            '&#63074;' => array(
                'e' => '&#63074;',
                'i' => pack('C*', 0x81, 0x6d, 0x90, 0xe1, 0x8c, 0x8b, 0x8f, 0xbb, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x90, 0xe1, 0x8c, 0x8b, 0x8f, 0xbb, 0x81, 0x6e)
            ),
            '&#63340;' => array(
                'e' => '&#63340;',
                'i' => pack('C*', 0x81, 0x7e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x53, 0x0f)
            ),
            '&#63341;' => array(
                'e' => '&#63341;',
                'i' => pack('C*', 0x81, 0x7e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x53, 0x0f)
            ),
            '&#63342;' => array(
                'e' => '&#63342;',
                'i' => pack('C*', 0x81, 0xa8),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x54, 0x0f)
            ),
            '&#63343;' => array(
                'e' => '&#63343;',
                'i' => pack('C*', 0x81, 0xa9),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x55, 0x0f)
            ),
            '&#63132;' => array(
                'e' => '&#63132;',
                'i' => '&#63699;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x67, 0x0f)
            ),
            '&#63344;' => array(
                'e' => '&#63344;',
                'i' => pack('C*', 0x81, 0x80),
                's' => pack('C*', 0x81, 0x80)
            ),
            '&#63360;' => array(
                'e' => '&#63360;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4a, 0x83, 0x8c, 0x83, 0x93, 0x83, 0x5f, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x4a, 0x83, 0x8c, 0x83, 0x93, 0x83, 0x5f, 0x81, 0x5b, 0x81, 0x6e)
            ),
            '&#63188;' => array(
                'e' => '&#63188;',
                'i' => '&#63893;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x76, 0x0f)
            ),
            '&#63075;' => array(
                'e' => '&#63075;',
                'i' => pack('C*', 0x81, 0x99),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4f, 0x0f)
            ),
            '&#63345;' => array(
                'e' => '&#63345;',
                'i' => '&#63705;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x56, 0x0f)
            ),
            '&#63346;' => array(
                'e' => '&#63346;',
                'i' => '&#63817;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x59, 0x0f)
            ),
            '&#63213;' => array(
                'e' => '&#63213;',
                'i' => '&#63936;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x54, 0x0f)
            ),
            '&#63347;' => array(
                'e' => '&#63347;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x60,
                    0x83,
                    0x46,
                    0x83,
                    0x62,
                    0x83,
                    0x4e,
                    0x83,
                    0x7d,
                    0x81,
                    0x5b,
                    0x83,
                    0x4e,
                    0x81,
                    0x6e
                ),
                's' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x60,
                    0x83,
                    0x46,
                    0x83,
                    0x62,
                    0x83,
                    0x4e,
                    0x83,
                    0x7d,
                    0x81,
                    0x5b,
                    0x83,
                    0x4e,
                    0x81,
                    0x6e
                )
            ),
            '&#63160;' => array(
                'e' => '&#63160;',
                'i' => '&#63813;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x72, 0x0f)
            ),
            '&#63040;' => array(
                'e' => '&#63040;',
                'i' => pack('C*', 0x81, 0x99, 0x9c, 0x63),
                's' => pack('C*', 0x81, 0x99, 0x9c, 0x63)
            ),
            '&#63044;' => array(
                'e' => '&#63044;',
                'i' => '&#63903;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4e, 0x0f)
            ),
            '&#63054;' => array(
                'e' => '&#63054;',
                'i' => '&#63904;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2f, 0x0f)
            ),
            '&#63161;' => array(
                'e' => '&#63161;',
                'i' => pack('C*', 0x92, 0xb9),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x43, 0x0f)
            ),
            '&#63404;' => array(
                'e' => '&#63404;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x74, 0x83, 0x48, 0x83, 0x8b, 0x83, 0x5f, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x74, 0x83, 0x48, 0x83, 0x8b, 0x83, 0x5f, 0x81, 0x6e)
            ),
            '&#63189;' => array(
                'e' => '&#63189;',
                'i' => '&#63893;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x21, 0x0f)
            ),
            '&#63348;' => array(
                'e' => '&#63348;',
                'i' => '&#63958;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6e, 0x0f)
            ),
            '&#63349;' => array(
                'e' => '&#63349;',
                'i' => '&#63963;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6f, 0x0f)
            ),
            '&#63092;' => array(
                'e' => '&#63092;',
                'i' => '&#63715;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x43, 0x0f)
            ),
            '&#63405;' => array(
                'e' => '&#63405;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x74, 0x83, 0x48, 0x83, 0x8b, 0x83, 0x5f, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x74, 0x83, 0x48, 0x83, 0x8b, 0x83, 0x5f, 0x81, 0x6e)
            ),
            '&#63411;' => array(
                'e' => '&#63411;',
                'i' => '&#63720;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x29, 0x0f)
            ),
            '&#63190;' => array(
                'e' => '&#63190;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x74, 0x83, 0x4c, 0x83, 0x5f, 0x83, 0x56, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x74, 0x83, 0x4c, 0x83, 0x5f, 0x83, 0x56, 0x81, 0x6e)
            ),
            '&#63385;' => array(
                'e' => '&#63385;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4a, 0x81, 0x5b, 0x83, 0x68, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x4a, 0x81, 0x5b, 0x83, 0x68, 0x81, 0x6e)
            ),
            '&#63350;' => array(
                'e' => '&#63350;',
                'i' => pack('C*', 0x81, 0xa3),
                's' => pack('C*', 0x81, 0xa3)
            ),
            '&#63351;' => array(
                'e' => '&#63351;',
                'i' => pack('C*', 0x81, 0xa5),
                's' => pack('C*', 0x81, 0xa5)
            ),
            '&#63376;' => array(
                'e' => '&#63376;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x74, 0x82, 0x72, 0x82, 0x60, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x2c, 0x0f)
            ),
            '&#63093;' => array(
                'e' => '&#63093;',
                'i' => '&#63716;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x68, 0x0f)
            ),
            '&#63361;' => array(
                'e' => '&#63361;',
                'i' => '&#63722;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x21, 0x0f)
            ),
            '&#63412;' => array(
                'e' => '&#63412;',
                'i' => '&#63697;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x65, 0x0f)
            ),
            '&#63214;' => array(
                'e' => '&#63214;',
                'i' => '&#63714;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x28, 0x0f)
            ),
            '&#63076;' => array(
                'e' => '&#63076;',
                'i' => '&#63649;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6b, 0x0f)
            ),
            '&#63124;' => array(
                'e' => '&#63124;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x74,
                    0x83,
                    0x62,
                    0x83,
                    0x67,
                    0x83,
                    0x7b,
                    0x81,
                    0x5b,
                    0x83,
                    0x8b,
                    0x81,
                    0x6e
                ),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x4b, 0x0f)
            ),
            '&#63362;' => array(
                'e' => '&#63362;',
                'i' => '&#63716;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x68, 0x0f)
            ),
            '&#63068;' => array(
                'e' => '&#63068;',
                'i' => '&#63956;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x57, 0x0f)
            ),
            '&#63042;' => array(
                'e' => '&#63042;',
                'i' => '&#63694;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6e, 0x0f)
            ),
            '&#63363;' => array(
                'e' => '&#63363;',
                'i' => '&#63716;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x68, 0x0f)
            ),
            '&#63364;' => array(
                'e' => '&#63364;',
                'i' => '&#63716;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x68, 0x0f)
            ),
            '&#63365;' => array(
                'e' => '&#63365;',
                'i' => '&#63716;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x68, 0x0f)
            ),
            '&#63366;' => array(
                'e' => '&#63366;',
                'i' => '&#63722;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x21, 0x0f)
            ),
            '&#63215;' => array(
                'e' => '&#63215;',
                'i' => '&#63702;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x33, 0x0f)
            ),
            '&#63367;' => array(
                'e' => '&#63367;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4a, 0x83, 0x8c, 0x83, 0x93, 0x83, 0x5f, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x4a, 0x83, 0x8c, 0x83, 0x93, 0x83, 0x5f, 0x81, 0x5b, 0x81, 0x6e)
            ),
            '&#63094;' => array(
                'e' => '&#63094;',
                'i' => '&#63711;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x45, 0x0f)
            ),
            '&#63077;' => array(
                'e' => '&#63077;',
                'i' => '&#63648;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x69, 0x0f)
            ),
            '&#63226;' => array(
                'e' => '&#63226;',
                'i' => '&#63863;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x23, 0x0f)
            ),
            '&#63386;' => array(
                'e' => '&#63386;',
                'i' => '&#63866;',
                's' => pack('C*', 0x81, 0x8f)
            ),
            '&#63216;' => array(
                'e' => '&#63216;',
                'i' => '&#63704;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5d, 0x0f)
            ),
            '&#63387;' => array(
                'e' => '&#63387;',
                'i' => '&#63704;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5d, 0x0f)
            ),
            '&#63108;' => array(
                'e' => '&#63108;',
                'i' => '&#63684;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x56, 0x0f)
            ),
            '&#63165;' => array(
                'e' => '&#63165;',
                'i' => '&#63976;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x24, 0x0f)
            ),
            '&#63388;' => array(
                'e' => '&#63388;',
                'i' => pack('C*', 0x81, 0x6d, 0x95, 0xef, 0x92, 0x9a, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x95, 0xef, 0x92, 0x9a, 0x81, 0x6e)
            ),
            '&#63389;' => array(
                'e' => '&#63389;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x72, 0x83, 0x66, 0x83, 0x49, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x49, 0x0f)
            ),
            '&#63191;' => array(
                'e' => '&#63191;',
                'i' => '&#63739;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x81, 0x83, 0x4b, 0x83, 0x6c, 0x81, 0x6e)
            ),
            '&#63352;' => array(
                'e' => '&#63352;',
                'i' => pack('C*', 0x84, 0xa4, 0x81, 0xa8),
                's' => pack('C*', 0x84, 0xa4, 0x81, 0xa8)
            ),
            '&#63353;' => array(
                'e' => '&#63353;',
                'i' => '&#63870;',
                's' => pack('C*', 0x81, 0xa9, 0x84, 0xa3)
            ),
            '&#63217;' => array(
                'e' => '&#63217;',
                'i' => '&#63873;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x34, 0x0f)
            ),
            '&#63218;' => array(
                'e' => '&#63218;',
                'i' => '&#63869;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5f, 0x0f)
            ),
            '&#63368;' => array(
                'e' => '&#63368;',
                'i' => '&#63716;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x68, 0x0f)
            ),
            '&#63095;' => array(
                'e' => '&#63095;',
                'i' => '&#63716;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x68, 0x0f)
            ),
            '&#63390;' => array(
                'e' => '&#63390;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x6c, 0x83, 0x57, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x6c, 0x83, 0x57, 0x81, 0x6e)
            ),
            '&#63219;' => array(
                'e' => '&#63219;',
                'i' => '&#63701;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5e, 0x0f)
            ),
            '&#63114;' => array(
                'e' => '&#63114;',
                'i' => '&#63679;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3b, 0x0f)
            ),
            '&#63391;' => array(
                'e' => '&#63391;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x74, 0x83, 0x8d, 0x83, 0x62, 0x83, 0x73, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x36, 0x0f)
            ),
            '&#63377;' => array(
                'e' => '&#63377;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4f, 0x83, 0x89, 0x83, 0x74, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6a, 0x0f)
            ),
            '&#63378;' => array(
                'e' => '&#63378;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4f, 0x83, 0x89, 0x83, 0x74, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6a, 0x0f)
            ),
            '&#63220;' => array(
                'e' => '&#63220;',
                'i' => '&#63686;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x21, 0x0f)
            ),
            '&#63392;' => array(
                'e' => '&#63392;',
                'i' => '&#63904;',
                's' => pack('C*', 0x81, 0x6d, 0x89, 0xf9, 0x92, 0x86, 0x93, 0x64, 0x93, 0x94, 0x81, 0x6e)
            ),
            '&#63369;' => array(
                'e' => '&#63369;',
                'i' => '&#63716;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x68, 0x0f)
            ),
            '&#63354;' => array(
                'e' => '&#63354;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x60,
                    0x83,
                    0x46,
                    0x83,
                    0x62,
                    0x83,
                    0x4e,
                    0x83,
                    0x7d,
                    0x81,
                    0x5b,
                    0x83,
                    0x4e,
                    0x81,
                    0x6e
                ),
                's' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x60,
                    0x83,
                    0x46,
                    0x83,
                    0x62,
                    0x83,
                    0x4e,
                    0x83,
                    0x7d,
                    0x81,
                    0x5b,
                    0x83,
                    0x4e,
                    0x81,
                    0x6e
                )
            ),
            '&#63143;' => array(
                'e' => '&#63143;',
                'i' => '&#63980;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x38, 0x0f)
            ),
            '&#63162;' => array(
                'e' => '&#63162;',
                'i' => '&#63813;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x72, 0x0f)
            ),
            '&#63393;' => array(
                'e' => '&#63393;',
                'i' => pack('C*', 0x81, 0x6d, 0x93, 0x64, 0x92, 0x72, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x93, 0x64, 0x92, 0x72, 0x81, 0x6e)
            ),
            '&#63355;' => array(
                'e' => '&#63355;',
                'i' => '&#63919;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x58, 0x83, 0x4e, 0x83, 0x8d, 0x81, 0x5b, 0x83, 0x8b, 0x81, 0x6e)
            ),
            '&#63370;' => array(
                'e' => '&#63370;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0xe6, 0x82, 0xd1, 0x82, 0xe5, 0x82, 0xa4, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x89, 0xe6, 0x82, 0xd1, 0x82, 0xe5, 0x82, 0xa4, 0x81, 0x6e)
            ),
            '&#63221;' => array(
                'e' => '&#63221;',
                'i' => '&#63869;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x64, 0x0f)
            ),
            '&#63394;' => array(
                'e' => '&#63394;',
                'i' => '&#63930;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x4f, 0x0f)
            ),
            '&#63192;' => array(
                'e' => '&#63192;',
                'i' => pack('C*', 0x81, 0xa9),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x50, 0x0f)
            ),
            '&#63193;' => array(
                'e' => '&#63193;',
                'i' => pack('C*', 0x81, 0xa8),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x51, 0x0f)
            ),
            '&#63371;' => array(
                'e' => '&#63371;',
                'i' => '&#63716;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x68, 0x0f)
            ),
            '&#63096;' => array(
                'e' => '&#63096;',
                'i' => '&#63957;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x4e, 0x83, 0x8a, 0x83, 0x62, 0x83, 0x76, 0x81, 0x6e)
            ),
            '&#63144;' => array(
                'e' => '&#63144;',
                'i' => '&#63718;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x32, 0x0f)
            ),
            '&#63222;' => array(
                'e' => '&#63222;',
                'i' => pack('C*', 0x81, 0x6d, 0x96, 0xbc, 0x8e, 0x44, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x96, 0xbc, 0x8e, 0x44, 0x81, 0x6e)
            ),
            '&#63109;' => array(
                'e' => '&#63109;',
                'i' => '&#63696;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x63, 0x0f)
            ),
            '&#63372;' => array(
                'e' => '&#63372;',
                'i' => '&#63716;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x68, 0x0f)
            ),
            '&#63115;' => array(
                'e' => '&#63115;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x67, 0x83, 0x89, 0x83, 0x62, 0x83, 0x4e, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x4f, 0x0f)
            ),
            '&#63097;' => array(
                'e' => '&#63097;',
                'i' => '&#63934;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x21, 0x0f)
            ),
            '&#63395;' => array(
                'e' => '&#63395;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x6f, 0x82, 0x63, 0x82, 0x62, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x6f, 0x82, 0x63, 0x82, 0x62, 0x81, 0x6e)
            ),
            '&#63406;' => array(
                'e' => '&#63406;',
                'i' => '&#63859;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x23, 0x0f)
            ),
            '&#63396;' => array(
                'e' => '&#63396;',
                'i' => '&#63933;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x8c, 0x83, 0x93, 0x83, 0x60, 0x81, 0x6e)
            ),
            '&#63407;' => array(
                'e' => '&#63407;',
                'i' => pack('C*', 0x81, 0x6d, 0x91, 0x97, 0x90, 0x4d, 0x82, 0x61, 0x82, 0x6e, 0x82, 0x77, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x91, 0x97, 0x90, 0x4d, 0x82, 0x61, 0x82, 0x6e, 0x82, 0x77, 0x81, 0x6e)
            ),
            '&#63408;' => array(
                'e' => '&#63408;',
                'i' => pack('C*', 0x81, 0x6d, 0x8e, 0xf3, 0x90, 0x4d, 0x82, 0x61, 0x82, 0x6e, 0x82, 0x77, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x8e, 0xf3, 0x90, 0x4d, 0x82, 0x61, 0x82, 0x6e, 0x82, 0x77, 0x81, 0x6e)
            ),
            '&#63223;' => array(
                'e' => '&#63223;',
                'i' => '&#63720;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x29, 0x0f)
            ),
            '&#63110;' => array(
                'e' => '&#63110;',
                'i' => '&#63685;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x58, 0x0f)
            ),
            '&#63373;' => array(
                'e' => '&#63373;',
                'i' => pack('C*', 0x81, 0x6d, 0x92, 0xe8, 0x8b, 0x4b, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x92, 0xe8, 0x8b, 0x4b, 0x81, 0x6e)
            ),
            '&#63098;' => array(
                'e' => '&#63098;',
                'i' => pack('C*', 0x81, 0x6d, 0x8e, 0x4f, 0x8a, 0x70, 0x92, 0xe8, 0x8b, 0x4b, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x8e, 0x4f, 0x8a, 0x70, 0x92, 0xe8, 0x8b, 0x4b, 0x81, 0x6e)
            ),
            '&#63379;' => array(
                'e' => '&#63379;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4f, 0x83, 0x89, 0x83, 0x74, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x4f, 0x83, 0x89, 0x83, 0x74, 0x81, 0x6e)
            ),
            '&#63133;' => array(
                'e' => '&#63133;',
                'i' => pack('C*', 0x81, 0x6d, 0x93, 0xf7, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x93, 0xf7, 0x81, 0x6e)
            ),
            '&#63397;' => array(
                'e' => '&#63397;',
                'i' => '&#63721;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2a, 0x0f)
            ),
            '&#63398;' => array(
                'e' => '&#63398;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x52, 0x83, 0x93, 0x83, 0x5a, 0x83, 0x93, 0x83, 0x67, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x52, 0x83, 0x93, 0x83, 0x5a, 0x83, 0x93, 0x83, 0x67, 0x81, 0x6e)
            ),
            '&#63194;' => array(
                'e' => '&#63194;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0xc6, 0x91, 0xb0, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x89, 0xc6, 0x91, 0xb0, 0x81, 0x6e)
            ),
            '&#63399;' => array(
                'e' => '&#63399;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x8a, 0x83, 0x93, 0x83, 0x4e, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x8a, 0x83, 0x93, 0x83, 0x4e, 0x81, 0x6e)
            ),
            '&#63224;' => array(
                'e' => '&#63224;',
                'i' => '&#63718;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x32, 0x0f)
            ),
            '&#63225;' => array(
                'e' => '&#63225;',
                'i' => '&#63860;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2b, 0x0f)
            ),
            '&#63078;' => array(
                'e' => '&#63078;',
                'i' => '&#63647;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6a, 0x0f, 0x1b, 0x24, 0x47, 0x69, 0x0f)
            ),
            '&#63116;' => array(
                'e' => '&#63116;',
                'i' => '&#63683;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3d, 0x0f)
            ),
            '&#63117;' => array(
                'e' => '&#63117;',
                'i' => '&#63815;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3c, 0x0f)
            ),
            '&#63137;' => array(
                'e' => '&#63137;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x54, 0x83, 0x43, 0x83, 0x52, 0x83, 0x8d, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x54, 0x83, 0x43, 0x83, 0x52, 0x83, 0x8d, 0x81, 0x6e)
            ),
            '&#63400;' => array(
                'e' => '&#63400;',
                'i' => pack('C*', 0x81, 0x6d, 0x90, 0x56, 0x95, 0xb7, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x90, 0x56, 0x95, 0xb7, 0x81, 0x6e)
            ),
            '&#63118;' => array(
                'e' => '&#63118;',
                'i' => '&#63676;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3e, 0x0f)
            ),
            '&#63401;' => array(
                'e' => '&#63401;',
                'i' => null,
                's' => null
            ),
            '&#63402;' => array(
                'e' => '&#63402;',
                'i' => null,
                's' => null
            ),
            '&#63403;' => array(
                'e' => '&#63403;',
                'i' => null,
                's' => null
            ),
            '&#63061;' => array(
                'e' => '&#63061;',
                'i' => '&#63712;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2e, 0x0f)
            ),
            '&#63062;' => array(
                'e' => '&#63062;',
                'i' => '&#63713;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x28, 0x0f)
            ),
            '&#63063;' => array(
                'e' => '&#63063;',
                'i' => '&#63740;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2a, 0x0f)
            ),
            '&#63064;' => array(
                'e' => '&#63064;',
                'i' => pack('C*', 0x81, 0x6d, 0x8e, 0xe1, 0x97, 0x74, 0x83, 0x7d, 0x81, 0x5b, 0x83, 0x4e, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x29, 0x0f)
            ),
            '&#63227;' => array(
                'e' => '&#63227;',
                'i' => '&#63879;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3c, 0x0f)
            ),
            '&#63228;' => array(
                'e' => '&#63228;',
                'i' => '&#63880;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3d, 0x0f)
            ),
            '&#63296;' => array(
                'e' => '&#63296;',
                'i' => '&#63881;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3e, 0x0f)
            ),
            '&#63297;' => array(
                'e' => '&#63297;',
                'i' => '&#63882;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3f, 0x0f)
            ),
            '&#63298;' => array(
                'e' => '&#63298;',
                'i' => '&#63883;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x40, 0x0f)
            ),
            '&#63299;' => array(
                'e' => '&#63299;',
                'i' => '&#63884;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x41, 0x0f)
            ),
            '&#63300;' => array(
                'e' => '&#63300;',
                'i' => '&#63885;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x42, 0x0f)
            ),
            '&#63301;' => array(
                'e' => '&#63301;',
                'i' => '&#63886;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x43, 0x0f)
            ),
            '&#63302;' => array(
                'e' => '&#63302;',
                'i' => '&#63887;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x44, 0x0f)
            ),
            '&#63303;' => array(
                'e' => '&#63303;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x50, 0x82, 0x4f, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x50, 0x82, 0x4f, 0x81, 0x6e)
            ),
            '&#63041;' => array(
                'e' => '&#63041;',
                'i' => '&#63652;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x63, 0x0f)
            ),
            '&#63069;' => array(
                'e' => '&#63069;',
                'i' => '&#63650;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x68, 0x0f)
            ),
            '&#63079;' => array(
                'e' => '&#63079;',
                'i' => '&#63655;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x5f, 0x0f)
            ),
            '&#63080;' => array(
                'e' => '&#63080;',
                'i' => '&#63656;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x60, 0x0f)
            ),
            '&#63081;' => array(
                'e' => '&#63081;',
                'i' => '&#63657;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x61, 0x0f)
            ),
            '&#63082;' => array(
                'e' => '&#63082;',
                'i' => '&#63658;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x62, 0x0f)
            ),
            '&#63083;' => array(
                'e' => '&#63083;',
                'i' => '&#63659;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x63, 0x0f)
            ),
            '&#63084;' => array(
                'e' => '&#63084;',
                'i' => '&#63660;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x64, 0x0f)
            ),
            '&#63085;' => array(
                'e' => '&#63085;',
                'i' => '&#63661;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x65, 0x0f)
            ),
            '&#63086;' => array(
                'e' => '&#63086;',
                'i' => '&#63662;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x66, 0x0f)
            ),
            '&#63087;' => array(
                'e' => '&#63087;',
                'i' => '&#63663;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x67, 0x0f)
            ),
            '&#63088;' => array(
                'e' => '&#63088;',
                'i' => '&#63664;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x68, 0x0f)
            ),
            '&#63089;' => array(
                'e' => '&#63089;',
                'i' => '&#63665;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x69, 0x0f)
            ),
            '&#63090;' => array(
                'e' => '&#63090;',
                'i' => '&#63666;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6a, 0x0f)
            ),
            '&#63091;' => array(
                'e' => '&#63091;',
                'i' => pack('C*', 0x81, 0x6d, 0x8e, 0xd6, 0x8e, 0x67, 0x8d, 0xc0, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6b, 0x0f)
            ),
            '&#63099;' => array(
                'e' => '&#63099;',
                'i' => '&#63689;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x74, 0x0f)
            ),
            '&#63100;' => array(
                'e' => '&#63100;',
                'i' => '&#63691;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x76, 0x0f)
            ),
            '&#63101;' => array(
                'e' => '&#63101;',
                'i' => '&#63695;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x58, 0x0f, 0x1b, 0x24, 0x45, 0x59, 0x0f)
            ),
            '&#63102;' => array(
                'e' => '&#63102;',
                'i' => '&#63693;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6f, 0x0f)
            ),
            '&#63104;' => array(
                'e' => '&#63104;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x6f, 0x83, 0x58, 0x92, 0xe2, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x70, 0x0f)
            ),
            '&#63105;' => array(
                'e' => '&#63105;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x41, 0x83, 0x93, 0x83, 0x65, 0x83, 0x69, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6b, 0x0f)
            ),
            '&#63106;' => array(
                'e' => '&#63106;',
                'i' => '&#63682;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x22, 0x0f)
            ),
            '&#63107;' => array(
                'e' => '&#63107;',
                'i' => '&#63688;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6d, 0x0f)
            ),
            '&#63374;' => array(
                'e' => '&#63374;',
                'i' => '&#63692;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5a, 0x0f)
            ),
            '&#63375;' => array(
                'e' => '&#63375;',
                'i' => pack('C*', 0x81, 0x6d, 0x92, 0x6e, 0x90, 0x7d, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x92, 0x6e, 0x90, 0x7d, 0x81, 0x6e)
            ),
            '&#63111;' => array(
                'e' => '&#63111;',
                'i' => '&#63938;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x56, 0x0f)
            ),
            '&#63112;' => array(
                'e' => '&#63112;',
                'i' => '&#63681;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x79, 0x0f)
            ),
            '&#63113;' => array(
                'e' => '&#63113;',
                'i' => '&#63678;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x55, 0x0f)
            ),
            '&#63043;' => array(
                'e' => '&#63043;',
                'i' => '&#63960;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x35, 0x0f)
            ),
            '&#63119;' => array(
                'e' => '&#63119;',
                'i' => '&#63671;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x38, 0x0f)
            ),
            '&#63120;' => array(
                'e' => '&#63120;',
                'i' => '&#63670;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x35, 0x0f)
            ),
            '&#63121;' => array(
                'e' => '&#63121;',
                'i' => '&#63927;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x58, 0x83, 0x6d, 0x83, 0x7b, 0x81, 0x6e)
            ),
            '&#63122;' => array(
                'e' => '&#63122;',
                'i' => '&#63674;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x52, 0x0f)
            ),
            '&#63045;' => array(
                'e' => '&#63045;',
                'i' => pack('C*', 0x81, 0x6d, 0x8a, 0xcf, 0x97, 0x97, 0x8e, 0xd4, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x44, 0x0f)
            ),
            '&#63125;' => array(
                'e' => '&#63125;',
                'i' => '&#63900;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x43, 0x0f)
            ),
            '&#63126;' => array(
                'e' => '&#63126;',
                'i' => '&#63984;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2b, 0x0f)
            ),
            '&#63127;' => array(
                'e' => '&#63127;',
                'i' => '&#63824;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x44, 0x0f)
            ),
            '&#63128;' => array(
                'e' => '&#63128;',
                'i' => '&#63831;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x6b, 0x0f)
            ),
            '&#63129;' => array(
                'e' => '&#63129;',
                'i' => pack('C*', 0x81, 0x6d, 0x93, 0x8c, 0x8b, 0x9e, 0x83, 0x5e, 0x83, 0x8f, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x29, 0x0f)
            ),
            '&#63046;' => array(
                'e' => '&#63046;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x56, 0x82, 0x56, 0x82, 0x56, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x53, 0x0f)
            ),
            '&#63047;' => array(
                'e' => '&#63047;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x49, 0x83, 0x81, 0x83, 0x66, 0x83, 0x67, 0x83, 0x45, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x49, 0x83, 0x81, 0x83, 0x66, 0x83, 0x67, 0x83, 0x45, 0x81, 0x6e)
            ),
            '&#63134;' => array(
                'e' => '&#63134;',
                'i' => pack('C*', 0x81, 0x6d, 0x93, 0x49, 0x92, 0x86, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x50, 0x0f)
            ),
            '&#63135;' => array(
                'e' => '&#63135;',
                'i' => '&#63724;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x51, 0x81, 0x5b, 0x83, 0x80, 0x81, 0x6e)
            ),
            '&#63136;' => array(
                'e' => '&#63136;',
                'i' => '&#63930;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x4f, 0x0f)
            ),
            '&#63138;' => array(
                'e' => '&#63138;',
                'i' => '&#63816;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x53, 0x0f)
            ),
            '&#63139;' => array(
                'e' => '&#63139;',
                'i' => '&#63981;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x50, 0x0f)
            ),
            '&#63140;' => array(
                'e' => '&#63140;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0xa8, 0x89, 0xbb, 0x82, 0xaf, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x3b, 0x0f)
            ),
            '&#63141;' => array(
                'e' => '&#63141;',
                'i' => pack('C*', 0x81, 0x6d, 0x93, 0xfa, 0x82, 0xcc, 0x8a, 0xdb, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x2b, 0x0f)
            ),
            '&#63142;' => array(
                'e' => '&#63142;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x58, 0x83, 0x43, 0x83, 0x4a, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x68, 0x0f)
            ),
            '&#63145;' => array(
                'e' => '&#63145;',
                'i' => '&#63983;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x66, 0x0f)
            ),
            '&#63146;' => array(
                'e' => '&#63146;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x74, 0x83, 0x89, 0x83, 0x43, 0x83, 0x70, 0x83, 0x93, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x67, 0x0f)
            ),
            '&#63147;' => array(
                'e' => '&#63147;',
                'i' => '&#63975;',
                's' => pack('C*', 0x81, 0x6d, 0x82, 0xb3, 0x82, 0xad, 0x82, 0xe7, 0x82, 0xf1, 0x82, 0xda, 0x5d)
            ),
            '&#63148;' => array(
                'e' => '&#63148;',
                'i' => '&#63990;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x39, 0x0f)
            ),
            '&#63149;' => array(
                'e' => '&#63149;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x43, 0x83, 0x60, 0x83, 0x53, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x67, 0x0f)
            ),
            '&#63150;' => array(
                'e' => '&#63150;',
                'i' => '&#63982;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x62, 0x0f)
            ),
            '&#63151;' => array(
                'e' => '&#63151;',
                'i' => '&#63700;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x40, 0x0f)
            ),
            '&#63048;' => array(
                'e' => '&#63048;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4e, 0x83, 0x57, 0x83, 0x89, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x74, 0x0f)
            ),
            '&#63152;' => array(
                'e' => '&#63152;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x45, 0x83, 0x54, 0x83, 0x4d, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x4c, 0x0f)
            ),
            '&#63153;' => array(
                'e' => '&#63153;',
                'i' => '&#63993;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3a, 0x0f)
            ),
            '&#63154;' => array(
                'e' => '&#63154;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x54, 0x83, 0x8b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x29, 0x0f)
            ),
            '&#63155;' => array(
                'e' => '&#63155;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4a, 0x83, 0x47, 0x83, 0x8b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x51, 0x0f)
            ),
            '&#63156;' => array(
                'e' => '&#63156;',
                'i' => '&#63814;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6f, 0x0f)
            ),
            '&#63157;' => array(
                'e' => '&#63157;',
                'i' => '&#63989;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x75, 0x0f)
            ),
            '&#63158;' => array(
                'e' => '&#63158;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x41, 0x83, 0x8a, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x41, 0x83, 0x8a, 0x81, 0x6e)
            ),
            '&#63159;' => array(
                'e' => '&#63159;',
                'i' => '&#63994;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2b, 0x0f)
            ),
            '&#63163;' => array(
                'e' => '&#63163;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x72, 0x81, 0x5b, 0x83, 0x60, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x27, 0x0f)
            ),
            '&#63164;' => array(
                'e' => '&#63164;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0xd0, 0x82, 0xdc, 0x82, 0xed, 0x82, 0xe8, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x25, 0x0f)
            ),
            '&#63049;' => array(
                'e' => '&#63049;',
                'i' => '&#63893;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x77, 0x0f)
            ),
            '&#63050;' => array(
                'e' => '&#63050;',
                'i' => '&#63894;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x79, 0x0f)
            ),
            '&#63051;' => array(
                'e' => '&#63051;',
                'i' => '&#63954;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x31, 0x0f)
            ),
            '&#63052;' => array(
                'e' => '&#63052;',
                'i' => '&#63951;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x23, 0x0f)
            ),
            '&#63053;' => array(
                'e' => '&#63053;',
                'i' => '&#63910;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5c, 0x0f)
            ),
            '&#63166;' => array(
                'e' => '&#63166;',
                'i' => '&#63905;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x54, 0x0f)
            ),
            '&#63167;' => array(
                'e' => '&#63167;',
                'i' => '&#63916;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x51, 0x0f)
            ),
            '&#63168;' => array(
                'e' => '&#63168;',
                'i' => '&#63949;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x25, 0x0f)
            ),
            '&#63055;' => array(
                'e' => '&#63055;',
                'i' => '&#63891;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x43, 0x0f)
            ),
            '&#63056;' => array(
                'e' => '&#63056;',
                'i' => '&#63892;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x47, 0x0f)
            ),
            '&#63057;' => array(
                'e' => '&#63057;',
                'i' => '&#63903;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4e, 0x0f)
            ),
            '&#63058;' => array(
                'e' => '&#63058;',
                'i' => '&#63907;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x31, 0x0f)
            ),
            '&#63059;' => array(
                'e' => '&#63059;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0x8a, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x3d, 0x0f)
            ),
            '&#63169;' => array(
                'e' => '&#63169;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x72, 0x82, 0x6e, 0x82, 0x72, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x72, 0x82, 0x6e, 0x82, 0x72, 0x81, 0x6e)
            ),
            '&#63170;' => array(
                'e' => '&#63170;',
                'i' => pack('C*', 0x81, 0x6d, 0x97, 0xcd, 0x82, 0xb1, 0x82, 0xd4, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6c, 0x0f)
            ),
            '&#63171;' => array(
                'e' => '&#63171;',
                'i' => '&#63726;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x49, 0x0f)
            ),
            '&#63172;' => array(
                'e' => '&#63172;',
                'i' => '&#63902;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x23, 0x0f)
            ),
            '&#63173;' => array(
                'e' => '&#63173;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0x46, 0x92, 0x88, 0x90, 0x6c, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2c, 0x0f)
            ),
            '&#63174;' => array(
                'e' => '&#63174;',
                'i' => '&#63652;',
                's' => pack('C*', 0x81, 0x6d, 0x82, 0xc8, 0x82, 0xe9, 0x82, 0xc6, 0x81, 0x6e)
            ),
            '&#63175;' => array(
                'e' => '&#63175;',
                'i' => '&#63737;',
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x56, 0x0f)
            ),
            '&#63176;' => array(
                'e' => '&#63176;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x41, 0x83, 0x4e, 0x83, 0x7d, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x3a, 0x0f)
            ),
            '&#63177;' => array(
                'e' => '&#63177;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0xd4, 0x8a, 0xdb, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x89, 0xd4, 0x8a, 0xdb, 0x81, 0x6e)
            ),
            '&#63178;' => array(
                'e' => '&#63178;',
                'i' => '&#63961;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x35, 0x0f)
            ),
            '&#63179;' => array(
                'e' => '&#63179;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x50, 0x82, 0x4f, 0x82, 0x4f, 0x93, 0x5f, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x50, 0x82, 0x4f, 0x82, 0x4f, 0x93, 0x5f, 0x81, 0x6e)
            ),
            '&#63180;' => array(
                'e' => '&#63180;',
                'i' => '&#63906;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2d, 0x0f)
            ),
            '&#63181;' => array(
                'e' => '&#63181;',
                'i' => '&#63917;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x50, 0x0f)
            ),
            '&#63182;' => array(
                'e' => '&#63182;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x45, 0x83, 0x93, 0x83, 0x60, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x7a, 0x0f)
            ),
            '&#63183;' => array(
                'e' => '&#63183;',
                'i' => pack('C*', 0x81, 0x6d, 0x90, 0x6c, 0x8d, 0xb7, 0x82, 0xb5, 0x8e, 0x77, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2f, 0x0f)
            ),
            '&#63184;' => array(
                'e' => '&#63184;',
                'i' => pack('C*', 0x81, 0x6d, 0x93, 0xbe, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x46, 0x0f)
            ),
            '&#63185;' => array(
                'e' => '&#63185;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x68, 0x83, 0x4e, 0x83, 0x8d, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x3c, 0x0f)
            ),
            '&#63186;' => array(
                'e' => '&#63186;',
                'i' => '&#63948;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2e, 0x0f)
            ),
            '&#63195;' => array(
                'e' => '&#63195;',
                'i' => '&#63723;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x4a, 0x0f)
            ),
            '&#63196;' => array(
                'e' => '&#63196;',
                'i' => '&#63703;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5c, 0x0f)
            ),
            '&#63197;' => array(
                'e' => '&#63197;',
                'i' => '&#63924;',
                's' => pack('C*', 0x81, 0x6d, 0x8d, 0xe0, 0x95, 0x7a, 0x81, 0x6e)
            ),
            '&#63198;' => array(
                'e' => '&#63198;',
                'i' => '&#63908;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x46, 0x0f)
            ),
            '&#63199;' => array(
                'e' => '&#63199;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4d, 0x83, 0x5e, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x61, 0x0f)
            ),
            '&#63200;' => array(
                'e' => '&#63200;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x6f, 0x83, 0x43, 0x83, 0x49, 0x83, 0x8a, 0x83, 0x93, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x6f, 0x83, 0x43, 0x83, 0x49, 0x83, 0x8a, 0x83, 0x93, 0x81, 0x6e)
            ),
            '&#63201;' => array(
                'e' => '&#63201;',
                'i' => '&#63707;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2a, 0x0f)
            ),
            '&#63202;' => array(
                'e' => '&#63202;',
                'i' => '&#63925;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x3c, 0x0f)
            ),
            '&#63203;' => array(
                'e' => '&#63203;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x73, 0x83, 0x58, 0x83, 0x67, 0x83, 0x8b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x33, 0x0f)
            ),
            '&#63204;' => array(
                'e' => '&#63204;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x47, 0x83, 0x58, 0x83, 0x65, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x3e, 0x0f)
            ),
            '&#63380;' => array(
                'e' => '&#63380;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x64, 0x82, 0x79, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x64, 0x82, 0x79, 0x81, 0x6e)
            ),
            '&#63381;' => array(
                'e' => '&#63381;',
                'i' => '&#63867;',
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x65, 0x82, 0x71, 0x82, 0x64, 0x82, 0x64, 0x81, 0x6e)
            ),
            '&#63205;' => array(
                'e' => '&#63205;',
                'i' => '&#63725;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x46, 0x0f)
            ),
            '&#63206;' => array(
                'e' => '&#63206;',
                'i' => '&#63923;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x26, 0x0f)
            ),
            '&#63207;' => array(
                'e' => '&#63207;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x74, 0x82, 0x65, 0x82, 0x6e, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2c, 0x0f)
            ),
            '&#63208;' => array(
                'e' => '&#63208;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x74, 0x82, 0x6f, 0x81, 0x49, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x33, 0x0f)
            ),
            '&#63209;' => array(
                'e' => '&#63209;',
                'i' => pack('C*', 0x81, 0x6d, 0x92, 0x8d, 0x8e, 0xcb, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5b, 0x0f)
            ),
            '&#63413;' => array(
                'e' => '&#63413;',
                'i' => '&#63653;',
                's' => pack('C*', 0x81, 0x6d, 0x96, 0xb6, 0x81, 0x6e)
            ),
            '&#63414;' => array(
                'e' => '&#63414;',
                'i' => '&#63669;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x34, 0x0f)
            ),
            '&#63415;' => array(
                'e' => '&#63415;',
                'i' => '&#63673;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x4a, 0x0f)
            ),
            '&#63416;' => array(
                'e' => '&#63416;',
                'i' => '&#63675;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x7c, 0x83, 0x50, 0x83, 0x78, 0x83, 0x8b, 0x81, 0x6e)
            ),
            '&#63417;' => array(
                'e' => '&#63417;',
                'i' => '&#63708;',
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x22, 0x0f)
            ),
            '&#63418;' => array(
                'e' => '&#63418;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0x89, 0x8c, 0x80, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x23, 0x0f)
            ),
            '&#63419;' => array(
                'e' => '&#63419;',
                'i' => '&#63710;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x43, 0x83, 0x78, 0x83, 0x93, 0x83, 0x67, 0x81, 0x6e)
            ),
            '&#63420;' => array(
                'e' => '&#63420;',
                'i' => '&#63717;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x34, 0x0f)
            ),
            '&#63421;' => array(
                'e' => '&#63421;',
                'i' => '&#63719;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x6b, 0x0f)
            ),
            '&#63422;' => array(
                'e' => '&#63422;',
                'i' => '&#63727;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2e, 0x0f)
            ),
            '&#63423;' => array(
                'e' => '&#63423;',
                'i' => '&#63728;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2d, 0x0f)
            ),
            '&#63424;' => array(
                'e' => '&#63424;',
                'i' => '&#63729;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2f, 0x0f)
            ),
            '&#63425;' => array(
                'e' => '&#63425;',
                'i' => '&#63730;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x39, 0x0f)
            ),
            '&#63426;' => array(
                'e' => '&#63426;',
                'i' => '&#63731;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x3b, 0x0f)
            ),
            '&#63427;' => array(
                'e' => '&#63427;',
                'i' => '&#63733;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x31, 0x0f)
            ),
            '&#63428;' => array(
                'e' => '&#63428;',
                'i' => '&#63734;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x32, 0x0f)
            ),
            '&#63429;' => array(
                'e' => '&#63429;',
                'i' => '&#63808;',
                's' => pack('C*', 0x81, 0x9c)
            ),
            '&#63430;' => array(
                'e' => '&#63430;',
                'i' => '&#63809;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6c, 0x0f)
            ),
            '&#63431;' => array(
                'e' => '&#63431;',
                'i' => '&#63810;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6c, 0x0f)
            ),
            '&#63432;' => array(
                'e' => '&#63432;',
                'i' => '&#63872;',
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x62, 0x82, 0x6b, 0x81, 0x6e)
            ),
            '&#63433;' => array(
                'e' => '&#63433;',
                'i' => '&#63888;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x45, 0x0f)
            ),
            '&#63434;' => array(
                'e' => '&#63434;',
                'i' => '&#63920;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6d, 0x0f)
            ),
            '&#63435;' => array(
                'e' => '&#63435;',
                'i' => '&#63897;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x27, 0x0f)
            ),
            '&#63436;' => array(
                'e' => '&#63436;',
                'i' => '&#63890;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x47, 0x0f)
            ),
            '&#63437;' => array(
                'e' => '&#63437;',
                'i' => '&#63914;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x68, 0x83, 0x93, 0x83, 0x62, 0x81, 0x6e)
            ),
            '&#63438;' => array(
                'e' => '&#63438;',
                'i' => '&#63915;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x51, 0x0f)
            ),
            '&#63439;' => array(
                'e' => '&#63439;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x82,
                    0x85,
                    0x82,
                    0x9a,
                    0x82,
                    0x90,
                    0x82,
                    0x8c,
                    0x82,
                    0x95,
                    0x82,
                    0x93,
                    0x81,
                    0x6e
                ),
                's' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x82,
                    0x85,
                    0x82,
                    0x9a,
                    0x82,
                    0x90,
                    0x82,
                    0x8c,
                    0x82,
                    0x95,
                    0x82,
                    0x93,
                    0x81,
                    0x6e
                )
            ),
            '&#63440;' => array(
                'e' => '&#63440;',
                'i' => pack('C*', 0x81, 0x6d, 0x92, 0x6e, 0x8b, 0x85, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x92, 0x6e, 0x8b, 0x85, 0x81, 0x6e)
            ),
            '&#63441;' => array(
                'e' => '&#63441;',
                'i' => '&#63985;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x60, 0x0f)
            ),
            '&#63461;' => array(
                'e' => '&#63461;',
                'i' => '&#63874;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x32, 0x0f)
            ),
            '&#63462;' => array(
                'e' => '&#63462;',
                'i' => '&#63923;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x26, 0x0f)
            ),
            '&#63463;' => array(
                'e' => '&#63463;',
                'i' => '&#63738;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x27, 0x0f)
            ),
            '&#63464;' => array(
                'e' => '&#63464;',
                'i' => '&#63931;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2c, 0x0f)
            ),
            '&#63465;' => array(
                'e' => '&#63465;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x89, 0x83, 0x57, 0x83, 0x49, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x48, 0x0f)
            ),
            '&#63466;' => array(
                'e' => '&#63466;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x6f, 0x83, 0x89, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x52, 0x0f)
            ),
            '&#63467;' => array(
                'e' => '&#63467;',
                'i' => pack('C*', 0x81, 0x6d, 0x8b, 0xb3, 0x89, 0xef, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x57, 0x0f)
            ),
            '&#63468;' => array(
                'e' => '&#63468;',
                'i' => '&#63677;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x54, 0x0f)
            ),
            '&#63469;' => array(
                'e' => '&#63469;',
                'i' => '&#63973;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5b, 0x0f)
            ),
            '&#63470;' => array(
                'e' => '&#63470;',
                'i' => '&#63899;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5e, 0x0f)
            ),
            '&#63471;' => array(
                'e' => '&#63471;',
                'i' => pack('C*', 0x81, 0x6d, 0x93, 0x56, 0x8e, 0x67, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6e, 0x0f)
            ),
            '&#63472;' => array(
                'e' => '&#63472;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x67, 0x83, 0x89, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x70, 0x0f)
            ),
            '&#63473;' => array(
                'e' => '&#63473;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4e, 0x83, 0x7d, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x71, 0x0f)
            ),
            '&#63474;' => array(
                'e' => '&#63474;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x6c, 0x83, 0x59, 0x83, 0x7e, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x73, 0x0f)
            ),
            '&#63475;' => array(
                'e' => '&#63475;',
                'i' => '&#63950;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x25, 0x0f)
            ),
            '&#63476;' => array(
                'e' => '&#63476;',
                'i' => '&#63947;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x26, 0x0f)
            ),
            '&#63477;' => array(
                'e' => '&#63477;',
                'i' => '&#63996;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x30, 0x0f)
            ),
            '&#63478;' => array(
                'e' => '&#63478;',
                'i' => '&#63944;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x28, 0x0f)
            ),
            '&#63479;' => array(
                'e' => '&#63479;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x5e, 0x83, 0x52, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2a, 0x0f)
            ),
            '&#63480;' => array(
                'e' => '&#63480;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x8d, 0x83, 0x50, 0x83, 0x62, 0x83, 0x67, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2d, 0x0f)
            ),
            '&#63481;' => array(
                'e' => '&#63481;',
                'i' => '&#63935;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2e, 0x0f)
            ),
            '&#63482;' => array(
                'e' => '&#63482;',
                'i' => '&#63902;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x31, 0x0f)
            ),
            '&#63483;' => array(
                'e' => '&#63483;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x6e, 0x83, 0x93, 0x83, 0x7d, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x36, 0x0f)
            ),
            '&#63484;' => array(
                'e' => '&#63484;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0xd4, 0x89, 0xce, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x37, 0x0f)
            ),
            '&#62272;' => array(
                'e' => '&#62272;',
                'i' => '&#63980;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x39, 0x0f)
            ),
            '&#62273;' => array(
                'e' => '&#62273;',
                'i' => '&#63715;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x3e, 0x0f)
            ),
            '&#62274;' => array(
                'e' => '&#62274;',
                'i' => pack('C*', 0x81, 0x6d, 0x95, 0xac, 0x90, 0x85, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x41, 0x0f)
            ),
            '&#62275;' => array(
                'e' => '&#62275;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4c, 0x83, 0x83, 0x83, 0x93, 0x83, 0x76, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x42, 0x0f)
            ),
            '&#62276;' => array(
                'e' => '&#62276;',
                'i' => pack('C*', 0x81, 0x6d, 0x96, 0x83, 0x90, 0x9d, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x4d, 0x0f)
            ),
            '&#62277;' => array(
                'e' => '&#62277;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x75, 0x82, 0x72, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x4e, 0x0f)
            ),
            '&#62278;' => array(
                'e' => '&#62278;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x67, 0x83, 0x8d, 0x83, 0x74, 0x83, 0x42, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x51, 0x0f)
            ),
            '&#62279;' => array(
                'e' => '&#62279;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4a, 0x83, 0x81, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x4a, 0x83, 0x81, 0x81, 0x6e)
            ),
            '&#62280;' => array(
                'e' => '&#62280;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x58, 0x83, 0x79, 0x83, 0x43, 0x83, 0x93, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x31, 0x0f)
            ),
            '&#62281;' => array(
                'e' => '&#62281;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x8d, 0x83, 0x56, 0x83, 0x41, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x32, 0x0f)
            ),
            '&#62282;' => array(
                'e' => '&#62282;',
                'i' => pack('C*', 0x81, 0x6d, 0x8d, 0x48, 0x8e, 0x96, 0x92, 0x86, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x57, 0x0f)
            ),
            '&#62283;' => array(
                'e' => '&#62283;',
                'i' => '&#63900;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5f, 0x0f)
            ),
            '&#62284;' => array(
                'e' => '&#62284;',
                'i' => pack('C*', 0x81, 0x6d, 0x8f, 0x6a, 0x93, 0xfa, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x63, 0x0f)
            ),
            '&#62285;' => array(
                'e' => '&#62285;',
                'i' => pack('C*', 0x81, 0x6d, 0x97, 0x5b, 0x8f, 0xc4, 0x82, 0xaf, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x6a, 0x0f)
            ),
            '&#62286;' => array(
                'e' => '&#62286;',
                'i' => '&#63988;',
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x43, 0x0f)
            ),
            '&#62287;' => array(
                'e' => '&#62287;',
                'i' => pack('C*', 0x81, 0x6d, 0x8a, 0x94, 0x89, 0xbf, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6a, 0x0f)
            ),
            '&#62288;' => array(
                'e' => '&#62288;',
                'i' => pack('C*', 0x81, 0x6d, 0x8c, 0x78, 0x8a, 0xaf, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x72, 0x0f)
            ),
            '&#62289;' => array(
                'e' => '&#62289;',
                'i' => '&#63686;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x73, 0x0f)
            ),
            '&#62290;' => array(
                'e' => '&#62290;',
                'i' => '&#63687;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x75, 0x0f)
            ),
            '&#62291;' => array(
                'e' => '&#62291;',
                'i' => '&#63971;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x77, 0x0f)
            ),
            '&#62292;' => array(
                'e' => '&#62292;',
                'i' => '&#63690;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x78, 0x0f)
            ),
            '&#62293;' => array(
                'e' => '&#62293;',
                'i' => '&#63682;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x22, 0x0f)
            ),
            '&#62294;' => array(
                'e' => '&#62294;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x50, 0x82, 0x57, 0x8b, 0xd6, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x27, 0x0f)
            ),
            '&#62295;' => array(
                'e' => '&#62295;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x6f, 0x83, 0x8a, 0x82, 0x52, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2b, 0x0f)
            ),
            '&#62296;' => array(
                'e' => '&#62296;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x62, 0x82, 0x6e, 0x82, 0x6e, 0x82, 0x6b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x34, 0x0f)
            ),
            '&#62297;' => array(
                'e' => '&#62297;',
                'i' => pack('C*', 0x81, 0x6d, 0x8a, 0x84, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x47, 0x0f)
            ),
            '&#62298;' => array(
                'e' => '&#62298;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x54, 0x81, 0x5b, 0x83, 0x72, 0x83, 0x58, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x48, 0x0f)
            ),
            '&#62299;' => array(
                'e' => '&#62299;',
                'i' => '&#63868;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x49, 0x0f)
            ),
            '&#62300;' => array(
                'e' => '&#62300;',
                'i' => '&#63968;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4a, 0x0f)
            ),
            '&#62301;' => array(
                'e' => '&#62301;',
                'i' => '&#63966;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4b, 0x0f)
            ),
            '&#62302;' => array(
                'e' => '&#62302;',
                'i' => pack('C*', 0x81, 0x6d, 0x8e, 0x77, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4c, 0x0f)
            ),
            '&#62303;' => array(
                'e' => '&#62303;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0x63, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4d, 0x0f)
            ),
            '&#62304;' => array(
                'e' => '&#62304;',
                'i' => pack('C*', 0x81, 0xaa),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4e, 0x0f)
            ),
            '&#62305;' => array(
                'e' => '&#62305;',
                'i' => pack('C*', 0x81, 0xab),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4f, 0x0f)
            ),
            '&#62306;' => array(
                'e' => '&#62306;',
                'i' => pack('C*', 0x81, 0x6d, 0x90, 0xe8, 0x82, 0xa2, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x5e, 0x0f)
            ),
            '&#62307;' => array(
                'e' => '&#62307;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x7d,
                    0x83,
                    0x69,
                    0x81,
                    0x5b,
                    0x83,
                    0x82,
                    0x81,
                    0x5b,
                    0x83,
                    0x68,
                    0x81,
                    0x6e
                ),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x70, 0x0f)
            ),
            '&#62308;' => array(
                'e' => '&#62308;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x50,
                    0x81,
                    0x5b,
                    0x83,
                    0x5e,
                    0x83,
                    0x43,
                    0x82,
                    0x8f,
                    0x82,
                    0x86,
                    0x82,
                    0x86,
                    0x81,
                    0x6e
                ),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x71, 0x0f)
            ),
            '&#62309;' => array(
                'e' => '&#62309;',
                'i' => '&#63722;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x21, 0x0f)
            ),
            '&#62310;' => array(
                'e' => '&#62310;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x6c, 0x83, 0x4e, 0x83, 0x5e, 0x83, 0x43, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x22, 0x0f)
            ),
            '&#62311;' => array(
                'e' => '&#62311;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x6e,
                    0x83,
                    0x43,
                    0x83,
                    0x72,
                    0x83,
                    0x58,
                    0x83,
                    0x4a,
                    0x83,
                    0x58,
                    0x81,
                    0x6e
                ),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x23, 0x0f)
            ),
            '&#62312;' => array(
                'e' => '&#62312;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0xd4, 0x91, 0xa9, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x26, 0x0f)
            ),
            '&#62313;' => array(
                'e' => '&#62313;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x54, 0x83, 0x7b, 0x83, 0x65, 0x83, 0x93, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x28, 0x0f)
            ),
            '&#62314;' => array(
                'e' => '&#62314;',
                'i' => '&#63984;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2b, 0x0f)
            ),
            '&#62315;' => array(
                'e' => '&#62315;',
                'i' => '&#63699;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2c, 0x0f)
            ),
            '&#62316;' => array(
                'e' => '&#62316;',
                'i' => pack('C*', 0x81, 0x6d, 0x8f, 0x6a, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2d, 0x0f)
            ),
            '&#62317;' => array(
                'e' => '&#62317;',
                'i' => pack('C*', 0x81, 0x6d, 0x96, 0xf2, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2f, 0x0f)
            ),
            '&#62318;' => array(
                'e' => '&#62318;',
                'i' => pack('C*', 0x81, 0x6d, 0x95, 0x97, 0x91, 0x44, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x30, 0x0f)
            ),
            '&#62319;' => array(
                'e' => '&#62319;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4e, 0x83, 0x89, 0x83, 0x62, 0x83, 0x4a, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x32, 0x0f)
            ),
            '&#62320;' => array(
                'e' => '&#62320;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x64, 0x82, 0x79, 0x83, 0x69, 0x83, 0x72, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x64, 0x82, 0x79, 0x83, 0x69, 0x83, 0x72, 0x81, 0x6e)
            ),
            '&#62321;' => array(
                'e' => '&#62321;',
                'i' => pack('C*', 0x81, 0x6d, 0x96, 0x58, 0x8e, 0x71, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x38, 0x0f)
            ),
            '&#62322;' => array(
                'e' => '&#62322;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x75, 0x81, 0x5b, 0x83, 0x63, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x3b, 0x0f)
            ),
            '&#62323;' => array(
                'e' => '&#62323;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x7d, 0x83, 0x6a, 0x83, 0x85, 0x83, 0x4c, 0x83, 0x41, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x3d, 0x0f)
            ),
            '&#62324;' => array(
                'e' => '&#62324;',
                'i' => pack('C*', 0x81, 0x6d, 0x94, 0xfc, 0x97, 0x65, 0x89, 0x40, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x3f, 0x0f)
            ),
            '&#62325;' => array(
                'e' => '&#62325;',
                'i' => pack('C*', 0x81, 0x6d, 0x8f, 0xb0, 0x89, 0xae, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x40, 0x0f)
            ),
            '&#62326;' => array(
                'e' => '&#62326;',
                'i' => pack('C*', 0x81, 0x6d, 0x92, 0x85, 0x95, 0xa8, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x41, 0x0f)
            ),
            '&#62327;' => array(
                'e' => '&#62327;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x72, 0x83, 0x4c, 0x83, 0x6a, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x42, 0x0f)
            ),
            '&#62328;' => array(
                'e' => '&#62328;',
                'i' => '&#63726;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2c, 0x0f)
            ),
            '&#62329;' => array(
                'e' => '&#62329;',
                'i' => '&#63726;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x47, 0x0f)
            ),
            '&#62330;' => array(
                'e' => '&#62330;',
                'i' => '&#63726;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4a, 0x0f)
            ),
            '&#62331;' => array(
                'e' => '&#62331;',
                'i' => '&#63726;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4b, 0x0f)
            ),
            '&#62332;' => array(
                'e' => '&#62332;',
                'i' => '&#63726;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4c, 0x0f)
            ),
            '&#62333;' => array(
                'e' => '&#62333;',
                'i' => '&#63726;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4d, 0x0f)
            ),
            '&#62334;' => array(
                'e' => '&#62334;',
                'i' => '&#63903;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4e, 0x0f)
            ),
            '&#62336;' => array(
                'e' => '&#62336;',
                'i' => '&#63672;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x33, 0x0f)
            ),
            '&#62337;' => array(
                'e' => '&#62337;',
                'i' => '&#63812;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x52, 0x0f)
            ),
            '&#62338;' => array(
                'e' => '&#62338;',
                'i' => '&#63939;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x58, 0x0f)
            ),
            '&#62339;' => array(
                'e' => '&#62339;',
                'i' => '&#63986;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x59, 0x0f)
            ),
            '&#62340;' => array(
                'e' => '&#62340;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x5c,
                    0x83,
                    0x74,
                    0x83,
                    0x67,
                    0x83,
                    0x4e,
                    0x83,
                    0x8a,
                    0x81,
                    0x5b,
                    0x83,
                    0x80,
                    0x81,
                    0x6e
                ),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x5a, 0x0f)
            ),
            '&#62341;' => array(
                'e' => '&#62341;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x7c, 0x83, 0x65, 0x83, 0x67, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x5b, 0x0f)
            ),
            '&#62342;' => array(
                'e' => '&#62342;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0xbe, 0x82, 0xf1, 0x82, 0xb2, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x5c, 0x0f)
            ),
            '&#62343;' => array(
                'e' => '&#62343;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0xb9, 0x82, 0xf1, 0x82, 0xd7, 0x82, 0xa2, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x5d, 0x0f)
            ),
            '&#62344;' => array(
                'e' => '&#62344;',
                'i' => '&#63985;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x5e, 0x0f)
            ),
            '&#62345;' => array(
                'e' => '&#62345;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x70, 0x83, 0x58, 0x83, 0x5e, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x5f, 0x0f)
            ),
            '&#62346;' => array(
                'e' => '&#62346;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4a, 0x83, 0x8c, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x61, 0x0f)
            ),
            '&#62347;' => array(
                'e' => '&#62347;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0xa8, 0x82, 0xc5, 0x82, 0xf1, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x63, 0x0f)
            ),
            '&#62348;' => array(
                'e' => '&#62348;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0xb7, 0x82, 0xb5, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x64, 0x0f)
            ),
            '&#62349;' => array(
                'e' => '&#62349;',
                'i' => '&#63978;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x65, 0x0f)
            ),
            '&#62350;' => array(
                'e' => '&#62350;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0xdd, 0x82, 0xa9, 0x82, 0xf1, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x66, 0x0f)
            ),
            '&#62351;' => array(
                'e' => '&#62351;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x67, 0x83, 0x7d, 0x83, 0x67, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x69, 0x0f)
            ),
            '&#62352;' => array(
                'e' => '&#62352;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x69, 0x83, 0x58, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x6a, 0x0f)
            ),
            '&#62353;' => array(
                'e' => '&#62353;',
                'i' => pack('C*', 0x81, 0x6d, 0x95, 0xd9, 0x93, 0x96, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x6c, 0x0f)
            ),
            '&#62354;' => array(
                'e' => '&#62354;',
                'i' => pack('C*', 0x81, 0x6d, 0x93, 0xe7, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x6d, 0x0f)
            ),
            '&#62355;' => array(
                'e' => '&#62355;',
                'i' => '&#63953;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x22, 0x0f)
            ),
            '&#62356;' => array(
                'e' => '&#62356;',
                'i' => '&#63941;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x23, 0x0f)
            ),
            '&#62357;' => array(
                'e' => '&#62357;',
                'i' => '&#63992;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x24, 0x0f)
            ),
            '&#62358;' => array(
                'e' => '&#62358;',
                'i' => '&#63952;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x26, 0x0f)
            ),
            '&#62359;' => array(
                'e' => '&#62359;',
                'i' => '&#63896;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x27, 0x0f)
            ),
            '&#62360;' => array(
                'e' => '&#62360;',
                'i' => '&#63910;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x28, 0x0f)
            ),
            '&#62361;' => array(
                'e' => '&#62361;',
                'i' => '&#63942;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2a, 0x0f)
            ),
            '&#62362;' => array(
                'e' => '&#62362;',
                'i' => '&#63996;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2b, 0x0f)
            ),
            '&#62363;' => array(
                'e' => '&#62363;',
                'i' => pack('C*', 0x81, 0x6d, 0x95, 0x97, 0x8e, 0xd7, 0x82, 0xd0, 0x82, 0xab, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2c, 0x0f)
            ),
            '&#62364;' => array(
                'e' => '&#62364;',
                'i' => pack('C*', 0x81, 0x6d, 0x94, 0x4d, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2d, 0x0f)
            ),
            '&#62365;' => array(
                'e' => '&#62365;',
                'i' => '&#63946;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2e, 0x0f)
            ),
            '&#62366;' => array(
                'e' => '&#62366;',
                'i' => '&#63897;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x30, 0x0f)
            ),
            '&#62367;' => array(
                'e' => '&#62367;',
                'i' => '&#63944;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2f, 0x0f)
            ),
            '&#62368;' => array(
                'e' => '&#62368;',
                'i' => '&#63908;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x46, 0x0f)
            ),
            '&#62369;' => array(
                'e' => '&#62369;',
                'i' => '&#63893;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x76, 0x0f)
            ),
            '&#62370;' => array(
                'e' => '&#62370;',
                'i' => pack('C*', 0x81, 0x69, 0x81, 0x84, 0x82, 0x52, 0x81, 0x83, 0x81, 0x6a),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x37, 0x0f)
            ),
            '&#62371;' => array(
                'e' => '&#62371;',
                'i' => pack('C*', 0x81, 0x69, 0x81, 0x4c, 0x82, 0x52, 0x81, 0x4d, 0x81, 0x6a),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x38, 0x0f)
            ),
            '&#62372;' => array(
                'e' => '&#62372;',
                'i' => pack('C*', 0x81, 0x6d, 0x95, 0x40, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x3a, 0x0f)
            ),
            '&#62373;' => array(
                'e' => '&#62373;',
                'i' => '&#63902;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x3c, 0x0f)
            ),
            '&#62374;' => array(
                'e' => '&#62374;',
                'i' => pack('C*', 0x81, 0x69, 0x81, 0x84, 0x90, 0x6c, 0x81, 0x83, 0x81, 0x6a),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x3d, 0x0f)
            ),
            '&#62375;' => array(
                'e' => '&#62375;',
                'i' => pack('C*', 0x81, 0x6d, 0x94, 0x8f, 0x8e, 0xe8, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x3f, 0x0f)
            ),
            '&#62376;' => array(
                'e' => '&#62376;',
                'i' => '&#63920;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x40, 0x0f)
            ),
            '&#62377;' => array(
                'e' => '&#62377;',
                'i' => '&#63909;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x41, 0x0f)
            ),
            '&#62378;' => array(
                'e' => '&#62378;',
                'i' => '&#63734;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x3e, 0x0f)
            ),
            '&#62379;' => array(
                'e' => '&#62379;',
                'i' => '&#63956;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x43, 0x0f)
            ),
            '&#62380;' => array(
                'e' => '&#62380;',
                'i' => '&#63920;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x44, 0x0f)
            ),
            '&#62381;' => array(
                'e' => '&#62381;',
                'i' => pack('C*', 0x82, 0x8d, 0x81, 0x69, 0x81, 0x51, 0x81, 0x51, 0x81, 0x6a, 0x82, 0x8d),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x46, 0x0f)
            ),
            '&#62382;' => array(
                'e' => '&#62382;',
                'i' => '&#63890;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x45, 0x0f)
            ),
            '&#62383;' => array(
                'e' => '&#62383;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x6f, 0x83, 0x6a, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x49, 0x0f)
            ),
            '&#62384;' => array(
                'e' => '&#62384;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x67,
                    0x83,
                    0x89,
                    0x83,
                    0x93,
                    0x83,
                    0x79,
                    0x83,
                    0x62,
                    0x83,
                    0x67,
                    0x81,
                    0x6e
                ),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x62, 0x0f)
            ),
            '&#62385;' => array(
                'e' => '&#62385;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x72, 0x83, 0x8a, 0x83, 0x84, 0x81, 0x5b, 0x83, 0x68, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x4c, 0x0f)
            ),
            '&#62386;' => array(
                'e' => '&#62386;',
                'i' => pack('C*', 0x81, 0x6d, 0x20, 0x90, 0x85, 0x89, 0x6a, 0x20, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x4d, 0x0f)
            ),
            '&#62387;' => array(
                'e' => '&#62387;',
                'i' => pack('C*', 0x81, 0x6d, 0x20, 0x8f, 0xc1, 0x96, 0x68, 0x8e, 0xd4, 0x20, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x50, 0x0f)
            ),
            '&#62388;' => array(
                'e' => '&#62388;',
                'i' => pack('C*', 0x81, 0x6d, 0x20, 0x8b, 0x7e, 0x8b, 0x7d, 0x8e, 0xd4, 0x20, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x51, 0x0f)
            ),
            '&#62389;' => array(
                'e' => '&#62389;',
                'i' => pack('C*', 0x81, 0x6d, 0x20, 0x83, 0x70, 0x83, 0x67, 0x83, 0x4a, 0x81, 0x5b, 0x20, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x52, 0x0f)
            ),
            '&#62390;' => array(
                'e' => '&#62390;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x20,
                    0x83,
                    0x57,
                    0x83,
                    0x46,
                    0x83,
                    0x62,
                    0x83,
                    0x67,
                    0x83,
                    0x52,
                    0x81,
                    0x5b,
                    0x83,
                    0x58,
                    0x83,
                    0x5e,
                    0x81,
                    0x5b,
                    0x20,
                    0x81,
                    0x6e
                ),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x53, 0x0f)
            ),
            '&#62391;' => array(
                'e' => '&#62391;',
                'i' => pack('C*', 0x81, 0x6d, 0x20, 0x96, 0xe5, 0x8f, 0xbc, 0x20, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x56, 0x0f)
            ),
            '&#62392;' => array(
                'e' => '&#62392;',
                'i' => pack('C*', 0x81, 0x6d, 0x20, 0x90, 0x97, 0x8d, 0xd5, 0x82, 0xe8, 0x20, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x58, 0x0f)
            ),
            '&#62393;' => array(
                'e' => '&#62393;',
                'i' => pack('C*', 0x81, 0x6d, 0x20, 0x91, 0xb2, 0x8b, 0xc6, 0x8e, 0xae, 0x20, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x59, 0x0f)
            ),
            '&#62394;' => array(
                'e' => '&#62394;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x89, 0x83, 0x93, 0x83, 0x68, 0x83, 0x5a, 0x83, 0x8b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x5a, 0x0f)
            ),
            '&#62395;' => array(
                'e' => '&#62395;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x20,
                    0x82,
                    0xb1,
                    0x82,
                    0xa2,
                    0x82,
                    0xcc,
                    0x82,
                    0xda,
                    0x82,
                    0xe8,
                    0x20,
                    0x81,
                    0x6e
                ),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x5b, 0x0f)
            ),
            '&#62396;' => array(
                'e' => '&#62396;',
                'i' => '&#63654;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x5c, 0x0f)
            ),
            '&#62397;' => array(
                'e' => '&#62397;',
                'i' => pack('C*', 0x81, 0x6d, 0x20, 0x89, 0xd4, 0x89, 0xc5, 0x20, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x20, 0x89, 0xd4, 0x89, 0xc5, 0x20, 0x81, 0x6e)
            ),
            '&#62398;' => array(
                'e' => '&#62398;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4a, 0x83, 0x4c, 0x95, 0x58, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x5f, 0x0f)
            ),
            '&#62399;' => array(
                'e' => '&#62399;',
                'i' => pack('C*', 0x81, 0x6d, 0x20, 0x90, 0xfc, 0x8d, 0x81, 0x89, 0xd4, 0x89, 0xce, 0x20, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x60, 0x0f)
            ),
            '&#62400;' => array(
                'e' => '&#62400;',
                'i' => pack('C*', 0x81, 0x6d, 0x8a, 0xaa, 0x8a, 0x4c, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x61, 0x0f)
            ),
            '&#62401;' => array(
                'e' => '&#62401;',
                'i' => pack('C*', 0x81, 0x6d, 0x20, 0x95, 0x97, 0x97, 0xe9, 0x20, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x62, 0x0f)
            ),
            '&#62402;' => array(
                'e' => '&#62402;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x20,
                    0x83,
                    0x6e,
                    0x83,
                    0x8d,
                    0x83,
                    0x45,
                    0x83,
                    0x42,
                    0x83,
                    0x93,
                    0x20,
                    0x81,
                    0x6e
                ),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x65, 0x0f)
            ),
            '&#62403;' => array(
                'e' => '&#62403;',
                'i' => pack('C*', 0x81, 0x6d, 0x20, 0x82, 0xa8, 0x8c, 0x8e, 0x8c, 0xa9, 0x20, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x66, 0x0f)
            ),
            '&#62404;' => array(
                'e' => '&#62404;',
                'i' => pack('C*', 0x81, 0x6d, 0x20, 0x83, 0x54, 0x83, 0x93, 0x83, 0x5e, 0x20, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x68, 0x0f)
            ),
            '&#62405;' => array(
                'e' => '&#62405;',
                'i' => '&#63831;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x6b, 0x0f)
            ),
            '&#62406;' => array(
                'e' => '&#62406;',
                'i' => pack('C*', 0x81, 0x6d, 0x93, 0xf8, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x6c, 0x0f)
            ),
            '&#62407;' => array(
                'e' => '&#62407;',
                'i' => '&#63690;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x78, 0x0f, 0x1b, 0x24, 0x51, 0x21, 0x0f)
            ),
            '&#62408;' => array(
                'e' => '&#62408;',
                'i' => '&#63647;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x69, 0x0f)
            ),
            '&#62409;' => array(
                'e' => '&#62409;',
                'i' => '&#63709;',
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x23, 0x0f)
            ),
            '&#62410;' => array(
                'e' => '&#62410;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x66, 0x83, 0x70, 0x81, 0x5b, 0x83, 0x67, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x24, 0x0f)
            ),
            '&#62411;' => array(
                'e' => '&#62411;',
                'i' => pack('C*', 0x81, 0x6d, 0x8f, 0xe9, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x28, 0x0f)
            ),
            '&#62412;' => array(
                'e' => '&#62412;',
                'i' => pack('C*', 0x81, 0x6d, 0x8f, 0xe9, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x26, 0x0f)
            ),
            '&#62413;' => array(
                'e' => '&#62413;',
                'i' => pack('C*', 0x81, 0x6d, 0x8d, 0x48, 0x8f, 0xea, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x28, 0x0f)
            ),
            '&#62414;' => array(
                'e' => '&#62414;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x74, 0x83, 0x89, 0x83, 0x93, 0x83, 0x58, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x2d, 0x0f)
            ),
            '&#63442;' => array(
                'e' => '&#63442;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x49,
                    0x81,
                    0x5b,
                    0x83,
                    0x76,
                    0x83,
                    0x93,
                    0x83,
                    0x45,
                    0x83,
                    0x46,
                    0x81,
                    0x5b,
                    0x83,
                    0x75,
                    0x81,
                    0x6e
                ),
                's' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x49,
                    0x81,
                    0x5b,
                    0x83,
                    0x76,
                    0x83,
                    0x93,
                    0x83,
                    0x45,
                    0x83,
                    0x46,
                    0x81,
                    0x5b,
                    0x83,
                    0x75,
                    0x81,
                    0x6e
                )
            ),
            '&#63443;' => array(
                'e' => '&#63443;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4a, 0x83, 0x4d, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x64, 0x0f)
            ),
            '&#63444;' => array(
                'e' => '&#63444;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x60, 0x82, 0x61, 0x82, 0x62, 0x82, 0x63, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x60, 0x82, 0x61, 0x82, 0x62, 0x82, 0x63, 0x81, 0x6e)
            ),
            '&#63445;' => array(
                'e' => '&#63445;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x81, 0x82, 0x82, 0x82, 0x83, 0x82, 0x84, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x81, 0x82, 0x82, 0x82, 0x83, 0x82, 0x84, 0x81, 0x6e)
            ),
            '&#63446;' => array(
                'e' => '&#63446;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x50, 0x82, 0x51, 0x82, 0x52, 0x82, 0x53, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x50, 0x82, 0x51, 0x82, 0x52, 0x82, 0x53, 0x81, 0x6e)
            ),
            '&#63447;' => array(
                'e' => '&#63447;',
                'i' => pack('C*', 0x81, 0x6d, 0x8b, 0x4c, 0x8d, 0x86, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x8b, 0x4c, 0x8d, 0x86, 0x81, 0x6e)
            ),
            '&#63448;' => array(
                'e' => '&#63448;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0xc2, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x89, 0xc2, 0x81, 0x6e)
            ),
            '&#63449;' => array(
                'e' => '&#63449;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x60,
                    0x83,
                    0x46,
                    0x83,
                    0x62,
                    0x83,
                    0x4e,
                    0x83,
                    0x7d,
                    0x81,
                    0x5b,
                    0x83,
                    0x4e,
                    0x81,
                    0x6e
                ),
                's' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x60,
                    0x83,
                    0x46,
                    0x83,
                    0x62,
                    0x83,
                    0x4e,
                    0x83,
                    0x7d,
                    0x81,
                    0x5b,
                    0x83,
                    0x4e,
                    0x81,
                    0x6e
                )
            ),
            '&#63450;' => array(
                'e' => '&#63450;',
                'i' => '&#63826;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x79, 0x83, 0x93, 0x81, 0x6e)
            ),
            '&#63451;' => array(
                'e' => '&#63451;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x89,
                    0x83,
                    0x57,
                    0x83,
                    0x49,
                    0x83,
                    0x7b,
                    0x83,
                    0x5e,
                    0x83,
                    0x93,
                    0x81,
                    0x6e
                ),
                's' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x89,
                    0x83,
                    0x57,
                    0x83,
                    0x49,
                    0x83,
                    0x7b,
                    0x83,
                    0x5e,
                    0x83,
                    0x93,
                    0x81,
                    0x6e
                )
            ),
            '&#63452;' => array(
                'e' => '&#63452;',
                'i' => '&#63873;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x34, 0x0f)
            ),
            '&#63453;' => array(
                'e' => '&#63453;',
                'i' => pack('C*', 0x81, 0x6d, 0x81, 0xa9, 0x82, 0x61, 0x82, 0x60, 0x82, 0x62, 0x82, 0x6a, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x55, 0x0f)
            ),
            '&#63454;' => array(
                'e' => '&#63454;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x75,
                    0x83,
                    0x62,
                    0x83,
                    0x4e,
                    0x83,
                    0x7d,
                    0x81,
                    0x5b,
                    0x83,
                    0x4e,
                    0x81,
                    0x6e
                ),
                's' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x75,
                    0x83,
                    0x62,
                    0x83,
                    0x4e,
                    0x83,
                    0x7d,
                    0x81,
                    0x5b,
                    0x83,
                    0x4e,
                    0x81,
                    0x6e
                )
            ),
            '&#63455;' => array(
                'e' => '&#63455;',
                'i' => '&#63858;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x24, 0x0f)
            ),
            '&#63456;' => array(
                'e' => '&#63456;',
                'i' => '&#63684;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x56, 0x0f)
            ),
            '&#63457;' => array(
                'e' => '&#63457;',
                'i' => '&#63686;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x21, 0x0f)
            ),
            '&#63458;' => array(
                'e' => '&#63458;',
                'i' => '&#63722;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x21, 0x0f)
            ),
            '&#63459;' => array(
                'e' => '&#63459;',
                'i' => '&#63869;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x64, 0x0f)
            ),
            '&#63460;' => array(
                'e' => '&#63460;',
                'i' => '&#63962;',
                's' => pack('C*', 0x81, 0xaa, 0x81, 0xab)
            ),
            '&#62415;' => array(
                'e' => '&#62415;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x68, 0x83, 0x43, 0x83, 0x63, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x2e, 0x0f)
            ),
            '&#62416;' => array(
                'e' => '&#62416;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x43, 0x83, 0x5e, 0x83, 0x8a, 0x83, 0x41, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x2f, 0x0f)
            ),
            '&#62417;' => array(
                'e' => '&#62417;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x43, 0x83, 0x4d, 0x83, 0x8a, 0x83, 0x58, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x30, 0x0f)
            ),
            '&#62418;' => array(
                'e' => '&#62418;',
                'i' => pack('C*', 0x81, 0x6d, 0x92, 0x86, 0x8d, 0x91, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x33, 0x0f)
            ),
            '&#62419;' => array(
                'e' => '&#62419;',
                'i' => pack('C*', 0x81, 0x6d, 0x8a, 0xd8, 0x8d, 0x91, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x34, 0x0f)
            ),
            '&#62420;' => array(
                'e' => '&#62420;',
                'i' => pack('C*', 0x81, 0x6d, 0x94, 0x92, 0x90, 0x6c, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x35, 0x0f)
            ),
            '&#62421;' => array(
                'e' => '&#62421;',
                'i' => pack('C*', 0x81, 0x6d, 0x92, 0x86, 0x8d, 0x91, 0x90, 0x6c, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x36, 0x0f)
            ),
            '&#62422;' => array(
                'e' => '&#62422;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x43, 0x83, 0x93, 0x83, 0x68, 0x90, 0x6c, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x37, 0x0f)
            ),
            '&#62423;' => array(
                'e' => '&#62423;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0xa8, 0x82, 0xb6, 0x82, 0xa2, 0x82, 0xb3, 0x82, 0xf1, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x38, 0x0f)
            ),
            '&#62424;' => array(
                'e' => '&#62424;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0xa8, 0x82, 0xce, 0x82, 0xa0, 0x82, 0xb3, 0x82, 0xf1, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x39, 0x0f)
            ),
            '&#62425;' => array(
                'e' => '&#62425;',
                'i' => pack('C*', 0x81, 0x6d, 0x90, 0xd4, 0x82, 0xbf, 0x82, 0xe1, 0x82, 0xf1, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x3a, 0x0f)
            ),
            '&#62426;' => array(
                'e' => '&#62426;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x8d,
                    0x48,
                    0x8e,
                    0x96,
                    0x8c,
                    0xbb,
                    0x8f,
                    0xea,
                    0x82,
                    0xcc,
                    0x90,
                    0x6c,
                    0x81,
                    0x6e
                ),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x3b, 0x0f)
            ),
            '&#62427;' => array(
                'e' => '&#62427;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0xa8, 0x95, 0x50, 0x97, 0x6c, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x3c, 0x0f)
            ),
            '&#62428;' => array(
                'e' => '&#62428;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x43, 0x83, 0x8b, 0x83, 0x4a, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x40, 0x0f)
            ),
            '&#62429;' => array(
                'e' => '&#62429;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x5f, 0x83, 0x93, 0x83, 0x58, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x3f, 0x0f)
            ),
            '&#62430;' => array(
                'e' => '&#62430;',
                'i' => '&#63990;',
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x42, 0x0f)
            ),
            '&#62431;' => array(
                'e' => '&#62431;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x51, 0x83, 0x57, 0x83, 0x51, 0x83, 0x57, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x45, 0x0f)
            ),
            '&#62432;' => array(
                'e' => '&#62432;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x5d, 0x83, 0x45, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x46, 0x0f)
            ),
            '&#62433;' => array(
                'e' => '&#62433;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x52, 0x83, 0x41, 0x83, 0x89, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x47, 0x0f)
            ),
            '&#62434;' => array(
                'e' => '&#62434;',
                'i' => pack('C*', 0x81, 0x6d, 0x8b, 0x8d, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x4b, 0x0f)
            ),
            '&#62435;' => array(
                'e' => '&#62435;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x77, 0x83, 0x72, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x4d, 0x0f)
            ),
            '&#62436;' => array(
                'e' => '&#62436;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x6a, 0x83, 0x8f, 0x83, 0x67, 0x83, 0x8a, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x4e, 0x0f)
            ),
            '&#62437;' => array(
                'e' => '&#62437;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x43, 0x83, 0x6d, 0x83, 0x56, 0x83, 0x56, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x4f, 0x0f)
            ),
            '&#62438;' => array(
                'e' => '&#62438;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x89, 0x83, 0x4e, 0x83, 0x5f, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x50, 0x0f)
            ),
            '&#62439;' => array(
                'e' => '&#62439;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x60, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x52, 0x0f)
            ),
            '&#62440;' => array(
                'e' => '&#62440;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x61, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x53, 0x0f)
            ),
            '&#62441;' => array(
                'e' => '&#62441;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x6e, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x55, 0x0f)
            ),
            '&#62442;' => array(
                'e' => '&#62442;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x60, 0x82, 0x61, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x54, 0x0f)
            ),
            '&#62443;' => array(
                'e' => '&#62443;',
                'i' => '&#63737;',
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x56, 0x0f)
            ),
            '&#62444;' => array(
                'e' => '&#62444;',
                'i' => '&#63738;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x27, 0x0f)
            ),
            '&#62445;' => array(
                'e' => '&#62445;',
                'i' => '&#63875;',
                's' => pack('C*', 0x81, 0x6d, 0x8a, 0xf8, 0x81, 0x6e)
            ),
            '&#62446;' => array(
                'e' => '&#62446;',
                'i' => '&#63898;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x56, 0x0f)
            ),
            '&#62447;' => array(
                'e' => '&#62447;',
                'i' => '&#63909;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x59, 0x0f)
            ),
            '&#62448;' => array(
                'e' => '&#62448;',
                'i' => '&#63912;',
                's' => pack('C*', 0x81, 0x49, 0x81, 0x48)
            ),
            '&#62449;' => array(
                'e' => '&#62449;',
                'i' => '&#63913;',
                's' => pack('C*', 0x81, 0x49, 0x81, 0x49)
            ),
            '&#62450;' => array(
                'e' => '&#62450;',
                'i' => '&#63919;',
                's' => pack('C*', 0x81, 0x60)
            ),
            '&#62451;' => array(
                'e' => '&#62451;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x81, 0x83, 0x8d, 0x83, 0x93, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x81, 0x83, 0x8d, 0x83, 0x93, 0x81, 0x6e)
            ),
            '&#62452;' => array(
                'e' => '&#62452;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x70,
                    0x83,
                    0x43,
                    0x83,
                    0x69,
                    0x83,
                    0x62,
                    0x83,
                    0x76,
                    0x83,
                    0x8b,
                    0x81,
                    0x6e
                ),
                's' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x70,
                    0x83,
                    0x43,
                    0x83,
                    0x69,
                    0x83,
                    0x62,
                    0x83,
                    0x76,
                    0x83,
                    0x8b,
                    0x81,
                    0x6e
                )
            ),
            '&#62453;' => array(
                'e' => '&#62453;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x75, 0x83, 0x68, 0x83, 0x45, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x75, 0x83, 0x68, 0x83, 0x45, 0x81, 0x6e)
            ),
            '&#62454;' => array(
                'e' => '&#62454;',
                'i' => '&#63977;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x6f, 0x83, 0x69, 0x83, 0x69, 0x81, 0x6e)
            ),
            '&#62455;' => array(
                'e' => '&#62455;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x82,
                    0xc6,
                    0x82,
                    0xa4,
                    0x82,
                    0xe0,
                    0x82,
                    0xeb,
                    0x82,
                    0xb1,
                    0x82,
                    0xb5,
                    0x81,
                    0x6e
                ),
                's' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x82,
                    0xc6,
                    0x82,
                    0xa4,
                    0x82,
                    0xe0,
                    0x82,
                    0xeb,
                    0x82,
                    0xb1,
                    0x82,
                    0xb5,
                    0x81,
                    0x6e
                )
            ),
            '&#62456;' => array(
                'e' => '&#62456;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4c, 0x83, 0x6d, 0x83, 0x52, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x4c, 0x83, 0x6d, 0x83, 0x52, 0x81, 0x6e)
            ),
            '&#62457;' => array(
                'e' => '&#62457;',
                'i' => pack('C*', 0x81, 0x6d, 0x8c, 0x49, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x8c, 0x49, 0x81, 0x6e)
            ),
            '&#62458;' => array(
                'e' => '&#62458;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x82, 0x83, 0x82, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x82, 0x83, 0x82, 0x81, 0x6e)
            ),
            '&#62459;' => array(
                'e' => '&#62459;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0xe2, 0x82, 0xab, 0x82, 0xa2, 0x82, 0xe0, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0xe2, 0x82, 0xab, 0x82, 0xa2, 0x82, 0xe0, 0x81, 0x6e)
            ),
            '&#62460;' => array(
                'e' => '&#62460;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x73, 0x83, 0x55, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x73, 0x83, 0x55, 0x81, 0x6e)
            ),
            '&#62528;' => array(
                'e' => '&#62528;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x5e, 0x81, 0x5b, 0x83, 0x4c, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x5e, 0x81, 0x5b, 0x83, 0x4c, 0x81, 0x5b, 0x81, 0x6e)
            ),
            '&#62529;' => array(
                'e' => '&#62529;',
                'i' => pack('C*', 0x81, 0x75, 0x8e, 0xb5, 0x97, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x75, 0x8e, 0xb5, 0x97, 0x5b, 0x81, 0x6e)
            ),
            '&#62530;' => array(
                'e' => '&#62530;',
                'i' => '&#63698;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x64, 0x0f)
            ),
            '&#62531;' => array(
                'e' => '&#62531;',
                'i' => pack('C*', 0x81, 0x6d, 0x92, 0x43, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x92, 0x43, 0x81, 0x6e)
            ),
            '&#62532;' => array(
                'e' => '&#62532;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x73, 0x83, 0x41, 0x83, 0x6d, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x73, 0x83, 0x41, 0x83, 0x6d, 0x81, 0x6e)
            ),
            '&#62533;' => array(
                'e' => '&#62533;',
                'i' => '&#63927;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x37, 0x0f)
            ),
            '&#62534;' => array(
                'e' => '&#62534;',
                'i' => '&#63990;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x39, 0x0f)
            ),
            '&#62535;' => array(
                'e' => '&#62535;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x7b, 0x81, 0x5b, 0x83, 0x8a, 0x83, 0x93, 0x83, 0x4f, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x7b, 0x81, 0x5b, 0x83, 0x8a, 0x83, 0x93, 0x83, 0x4f, 0x81, 0x6e)
            ),
            '&#62536;' => array(
                'e' => '&#62536;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0xc8, 0x82, 0xdc, 0x82, 0xcd, 0x82, 0xb0, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0xc8, 0x82, 0xdc, 0x82, 0xcd, 0x82, 0xb0, 0x81, 0x6e)
            ),
            '&#62537;' => array(
                'e' => '&#62537;',
                'i' => pack('C*', 0x81, 0x6d, 0x93, 0x56, 0x8b, 0xe7, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x93, 0x56, 0x8b, 0xe7, 0x81, 0x6e)
            ),
            '&#62538;' => array(
                'e' => '&#62538;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x70, 0x83, 0x93, 0x83, 0x5f, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x70, 0x83, 0x93, 0x83, 0x5f, 0x81, 0x6e)
            ),
            '&#62539;' => array(
                'e' => '&#62539;',
                'i' => '&#63949;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x29, 0x0f)
            ),
            '&#62540;' => array(
                'e' => '&#62540;',
                'i' => '&#63994;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2b, 0x0f)
            ),
            '&#62541;' => array(
                'e' => '&#62541;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0xd4, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x25, 0x0f)
            ),
            '&#62542;' => array(
                'e' => '&#62542;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x41,
                    0x83,
                    0x43,
                    0x83,
                    0x58,
                    0x83,
                    0x4e,
                    0x83,
                    0x8a,
                    0x81,
                    0x5b,
                    0x83,
                    0x80,
                    0x81,
                    0x6e
                ),
                's' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x83,
                    0x41,
                    0x83,
                    0x43,
                    0x83,
                    0x58,
                    0x83,
                    0x4e,
                    0x83,
                    0x8a,
                    0x81,
                    0x5b,
                    0x83,
                    0x80,
                    0x81,
                    0x6e
                )
            ),
            '&#62543;' => array(
                'e' => '&#62543;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x68, 0x81, 0x5b, 0x83, 0x69, 0x83, 0x62, 0x83, 0x63, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x68, 0x81, 0x5b, 0x83, 0x69, 0x83, 0x62, 0x83, 0x63, 0x81, 0x6e)
            ),
            '&#62544;' => array(
                'e' => '&#62544;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4e, 0x83, 0x62, 0x83, 0x4c, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x4e, 0x83, 0x62, 0x83, 0x4c, 0x81, 0x5b, 0x81, 0x6e)
            ),
            '&#62545;' => array(
                'e' => '&#62545;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x60, 0x83, 0x87, 0x83, 0x52, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x60, 0x83, 0x87, 0x83, 0x52, 0x81, 0x6e)
            ),
            '&#62546;' => array(
                'e' => '&#62546;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4c, 0x83, 0x83, 0x83, 0x93, 0x83, 0x66, 0x83, 0x42, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x4c, 0x83, 0x83, 0x83, 0x93, 0x83, 0x66, 0x83, 0x42, 0x81, 0x6e)
            ),
            '&#62547;' => array(
                'e' => '&#62547;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4c, 0x83, 0x83, 0x83, 0x93, 0x83, 0x66, 0x83, 0x42, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x4c, 0x83, 0x83, 0x83, 0x93, 0x83, 0x66, 0x83, 0x42, 0x81, 0x6e)
            ),
            '&#62548;' => array(
                'e' => '&#62548;',
                'i' => pack('C*', 0x28, 0x2f, 0x81, 0x51, 0x81, 0x5f, 0x81, 0x6a),
                's' => pack('C*', 0x28, 0x2f, 0x81, 0x51, 0x81, 0x5f, 0x81, 0x6a)
            ),
            '&#62549;' => array(
                'e' => '&#62549;',
                'i' => pack('C*', 0x81, 0x69, 0x81, 0x45, 0x81, 0x7e, 0x81, 0x45, 0x81, 0x6a),
                's' => pack('C*', 0x81, 0x69, 0x81, 0x45, 0x81, 0x7e, 0x81, 0x45, 0x81, 0x6a)
            ),
            '&#62550;' => array(
                'e' => '&#62550;',
                'i' => pack('C*', 0x81, 0x62, 0x81, 0x69, 0x81, 0x45, 0x81, 0x7e, 0x81, 0x45, 0x81, 0x6a, 0x81, 0x62),
                's' => pack('C*', 0x81, 0x62, 0x81, 0x69, 0x81, 0x45, 0x81, 0x7e, 0x81, 0x45, 0x81, 0x6a, 0x81, 0x62)
            ),
            '&#62551;' => array(
                'e' => '&#62551;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0xce, 0x8e, 0x52, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x89, 0xce, 0x8e, 0x52, 0x81, 0x6e)
            ),
            '&#62552;' => array(
                'e' => '&#62552;',
                'i' => '&#63726;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x42, 0x0f)
            ),
            '&#62553;' => array(
                'e' => '&#62553;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x60, 0x82, 0x61, 0x82, 0x62, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x60, 0x82, 0x61, 0x82, 0x62, 0x81, 0x6e)
            ),
            '&#62554;' => array(
                'e' => '&#62554;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x76, 0x83, 0x8a, 0x83, 0x93, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x76, 0x83, 0x8a, 0x83, 0x93, 0x81, 0x6e)
            ),
            '&#62555;' => array(
                'e' => '&#62555;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x7e, 0x83, 0x63, 0x83, 0x6f, 0x83, 0x60, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x7e, 0x83, 0x63, 0x83, 0x6f, 0x83, 0x60, 0x81, 0x6e)
            ),
            '&#62556;' => array(
                'e' => '&#62556;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0xc4, 0x82, 0xf1, 0x82, 0xc6, 0x82, 0xa4, 0x92, 0x8e, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0xc4, 0x82, 0xf1, 0x82, 0xc6, 0x82, 0xa4, 0x92, 0x8e, 0x81, 0x6e)
            ),
            '&#62557;' => array(
                'e' => '&#62557;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x6e, 0x83, 0x60, 0x83, 0x7e, 0x83, 0x63, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x6e, 0x83, 0x60, 0x83, 0x7e, 0x83, 0x63, 0x81, 0x6e)
            ),
            '&#62558;' => array(
                'e' => '&#62558;',
                'i' => '&#63978;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x65, 0x0f)
            ),
            '&#62559;' => array(
                'e' => '&#62559;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x94,
                    0xf2,
                    0x82,
                    0xf1,
                    0x82,
                    0xc5,
                    0x82,
                    0xa2,
                    0x82,
                    0xad,
                    0x82,
                    0xa8,
                    0x8b,
                    0xe0,
                    0x81,
                    0x6e
                ),
                's' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x94,
                    0xf2,
                    0x82,
                    0xf1,
                    0x82,
                    0xc5,
                    0x82,
                    0xa2,
                    0x82,
                    0xad,
                    0x82,
                    0xa8,
                    0x8b,
                    0xe0,
                    0x81,
                    0x6e
                )
            ),
            '&#62560;' => array(
                'e' => '&#62560;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x4e, 0x83, 0x89, 0x83, 0x4e, 0x83, 0x89, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x27, 0x0f)
            ),
            '&#62561;' => array(
                'e' => '&#62561;',
                'i' => '&#63945;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x36, 0x0f)
            ),
            '&#62562;' => array(
                'e' => '&#62562;',
                'i' => '&#63945;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x36, 0x0f)
            ),
            '&#62563;' => array(
                'e' => '&#62563;',
                'i' => '&#63831;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x6b, 0x0f)
            ),
            '&#62564;' => array(
                'e' => '&#62564;',
                'i' => pack('C*', 0x81, 0x69, 0x81, 0x4c, 0x84, 0x48, 0x81, 0x4d, 0x81, 0x6a),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x38, 0x0f)
            ),
            '&#62565;' => array(
                'e' => '&#62565;',
                'i' => '&#63892;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x77, 0x0f)
            ),
            '&#62566;' => array(
                'e' => '&#62566;',
                'i' => '&#63859;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x23, 0x0f)
            ),
            '&#62567;' => array(
                'e' => '&#62567;',
                'i' => '&#63951;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x32, 0x0f)
            ),
            '&#62568;' => array(
                'e' => '&#62568;',
                'i' => '&#63951;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x32, 0x0f)
            ),
            '&#62569;' => array(
                'e' => '&#62569;',
                'i' => '&#63947;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x26, 0x0f)
            ),
            '&#62570;' => array(
                'e' => '&#62570;',
                'i' => '&#63896;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x23, 0x0f)
            ),
            '&#62571;' => array(
                'e' => '&#62571;',
                'i' => '&#63896;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x23, 0x0f)
            ),
            '&#62572;' => array(
                'e' => '&#62572;',
                'i' => '&#63955;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x31, 0x0f)
            ),
            '&#62573;' => array(
                'e' => '&#62573;',
                'i' => '&#63955;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x31, 0x0f)
            ),
            '&#62574;' => array(
                'e' => '&#62574;',
                'i' => '&#63992;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x24, 0x0f)
            ),
            '&#62575;' => array(
                'e' => '&#62575;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x68, 0x83, 0x8c, 0x83, 0x58, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x39, 0x0f)
            ),
            '&#62576;' => array(
                'e' => '&#62576;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x82, 0x83, 0x41, 0x83, 0x43, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x82, 0x83, 0x41, 0x83, 0x43, 0x81, 0x6e)
            ),
            '&#62577;' => array(
                'e' => '&#62577;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0x77, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x59, 0x0f)
            ),
            '&#62578;' => array(
                'e' => '&#62578;',
                'i' => pack('C*', 0x81, 0x6d, 0x89, 0xd4, 0x8e, 0x44, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x89, 0xd4, 0x8e, 0x44, 0x81, 0x6e)
            ),
            '&#62579;' => array(
                'e' => '&#62579;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x57, 0x83, 0x87, 0x81, 0x5b, 0x83, 0x4a, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x57, 0x83, 0x87, 0x81, 0x5b, 0x83, 0x4a, 0x81, 0x5b, 0x81, 0x6e)
            ),
            '&#62580;' => array(
                'e' => '&#62580;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x47, 0x83, 0x72, 0x83, 0x74, 0x83, 0x89, 0x83, 0x43, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x47, 0x83, 0x72, 0x83, 0x74, 0x83, 0x89, 0x83, 0x43, 0x81, 0x6e)
            ),
            '&#62581;' => array(
                'e' => '&#62581;',
                'i' => '&#63863;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x23, 0x0f)
            ),
            '&#62582;' => array(
                'e' => '&#62582;',
                'i' => '&#63960;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x21, 0x0f)
            ),
            '&#62583;' => array(
                'e' => '&#62583;',
                'i' => pack('C*', 0x81, 0x6d, 0x83, 0x70, 0x83, 0x67, 0x83, 0x4a, 0x81, 0x5b, 0x81, 0x6e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x52, 0x0f)
            ),
            '&#62584;' => array(
                'e' => '&#62584;',
                'i' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x82,
                    0x64,
                    0x82,
                    0x79,
                    0x83,
                    0x80,
                    0x81,
                    0x5b,
                    0x83,
                    0x72,
                    0x81,
                    0x5b,
                    0x81,
                    0x6e
                ),
                's' => pack(
                    'C*',
                    0x81,
                    0x6d,
                    0x82,
                    0x64,
                    0x82,
                    0x79,
                    0x83,
                    0x80,
                    0x81,
                    0x5b,
                    0x83,
                    0x72,
                    0x81,
                    0x5b,
                    0x81,
                    0x6e
                )
            ),
            '&#62585;' => array(
                'e' => '&#62585;',
                'i' => '&#63890;',
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x47, 0x0f)
            ),
            '&#62586;' => array(
                'e' => '&#62586;',
                'i' => '&#63988;',
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x43, 0x0f)
            ),
            '&#62587;' => array(
                'e' => '&#62587;',
                'i' => '&#63926;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x57, 0x81, 0x5b, 0x83, 0x93, 0x83, 0x59, 0x81, 0x6e)
            ),
            '&#62588;' => array(
                'e' => '&#62588;',
                'i' => '&#63932;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x23, 0x0f, 0x1b, 0x24, 0x47, 0x42, 0x0f)
            ),
            '&#62589;' => array(
                'e' => '&#62589;',
                'i' => '&#63962;',
                's' => pack('C*', 0x81, 0xaa, 0x81, 0xab)
            ),
            '&#62590;' => array(
                'e' => '&#62590;',
                'i' => '&#63969;',
                's' => pack('C*', 0x81, 0xcc)
            ),
            '&#62592;' => array(
                'e' => '&#62592;',
                'i' => '&#63970;',
                's' => pack('C*', 0x81, 0xaa, 0x81, 0xab)
            ),
            '&#62593;' => array(
                'e' => '&#62593;',
                'i' => '&#63972;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x5e, 0x0f)
            ),
            '&#62594;' => array(
                'e' => '&#62594;',
                'i' => '&#63979;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x30, 0x0f)
            ),
            '&#62595;' => array(
                'e' => '&#62595;',
                'i' => '&#63987;',
                's' => pack('C*', 0x81, 0x6d, 0x83, 0x4a, 0x83, 0x5e, 0x83, 0x63, 0x83, 0x80, 0x83, 0x8a, 0x81, 0x6e)
            ),
            '&#62596;' => array(
                'e' => '&#62596;',
                'i' => '&#63992;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x24, 0x0f)
            ),
            '&#62597;' => array(
                'e' => '&#62597;',
                'i' => '&#63992;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x25, 0x0f)
            ),
            '&#62598;' => array(
                'e' => '&#62598;',
                'i' => pack('C*', 0x81, 0x6d, 0x82, 0x62, 0x83, 0x81, 0x81, 0x5b, 0x83, 0x8b, 0x81, 0x6e),
                's' => pack('C*', 0x81, 0x6d, 0x82, 0x62, 0x83, 0x81, 0x81, 0x5b, 0x83, 0x8b, 0x81, 0x6e)
            ),
            '&#62599;' => array(
                'e' => '&#62599;',
                'i' => '&#63974;',
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x30, 0x0f)
            ),
            '&#62600;' => array(
                'e' => '&#62600;',
                'i' => '&#63732;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x30, 0x0f)
            ),
            '&#62601;' => array(
                'e' => '&#62601;',
                'i' => '&#63877;',
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x30, 0x0f)
            ),
            '&#62602;' => array(
                'e' => '&#62602;',
                'i' => pack('C*', 0x81, 0x69, 0x81, 0x4f, 0x81, 0x7c, 0x81, 0x4f, 0x81, 0x6a, 0x81, 0x5e),
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x32, 0x0f)
            ),
            '&#62603;' => array(
                'e' => '&#62603;',
                'i' => pack('C*', 0x81, 0x5f, 0x81, 0x69, 0x5e, 0x82, 0x8f, 0x5e, 0x81, 0x6a, 0x81, 0x5e),
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x47, 0x0f)
            ),
            '&#62604;' => array(
                'e' => '&#62604;',
                'i' => '&#63896;',
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x23, 0x0f)
            ),
            '&#62605;' => array(
                'e' => '&#62605;',
                'i' => '&#63894;',
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x47, 0x0f)
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x21, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x21, 0x0f),
                'i' => '&#63893;',
                'e' => '&#63189;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x22, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x22, 0x0f),
                'i' => '&#63893;',
                'e' => '&#63187;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x23, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x23, 0x0f),
                'i' => '&#63902;',
                'e' => '&#63172;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x24, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x24, 0x0f),
                'i' => '&#63893;',
                'e' => '&#63189;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x25, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x25, 0x0f),
                'i' => '&#63893;',
                'e' => '&#63187;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x26, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x26, 0x0f),
                'i' => '&#63923;',
                'e' => '&#63462;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x27, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x27, 0x0f),
                'i' => '&#63738;',
                'e' => '&#63463;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x28, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x28, 0x0f),
                'i' => '&#63714;',
                'e' => '&#63214;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x29, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x29, 0x0f),
                'i' => '&#63720;',
                'e' => '&#63411;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x2a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2a, 0x0f),
                'i' => '&#63721;',
                'e' => '&#63397;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x2b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2b, 0x0f),
                'i' => '&#63860;',
                'e' => '&#63225;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x2c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2c, 0x0f),
                'i' => '&#63931;',
                'e' => '&#63464;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x2d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2d, 0x0f),
                'i' => '&#63906;',
                'e' => '&#63180;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x2e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2e, 0x0f),
                'i' => '&#63948;',
                'e' => '&#63186;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x2f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x2f, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63183;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x30, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x30, 0x0f),
                'i' => '&#63732;',
                'e' => '&#62600;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x31, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x31, 0x0f),
                'i' => '&#63733;',
                'e' => '&#63427;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x32, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x32, 0x0f),
                'i' => '&#63734;',
                'e' => '&#63428;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x33, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x33, 0x0f),
                'i' => '&#63672;',
                'e' => '&#62336;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x34, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x34, 0x0f),
                'i' => '&#63669;',
                'e' => '&#63414;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x35, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x35, 0x0f),
                'i' => '&#63670;',
                'e' => '&#63120;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x36, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x36, 0x0f),
                'i' => '&#63668;',
                'e' => '&#63123;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x37, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x37, 0x0f),
                'i' => '&#63927;',
                'e' => '&#62533;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x38, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x38, 0x0f),
                'i' => '&#63671;',
                'e' => '&#63119;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x39, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x39, 0x0f),
                'i' => '&#63990;',
                'e' => '&#63090;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x3a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3a, 0x0f),
                'i' => '&#63993;',
                'e' => '&#63153;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x3b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3b, 0x0f),
                'i' => '&#63679;',
                'e' => '&#63114;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x3c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3c, 0x0f),
                'i' => '&#63815;',
                'e' => '&#63117;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x3d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3d, 0x0f),
                'i' => '&#63683;',
                'e' => '&#63116;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x3e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3e, 0x0f),
                'i' => '&#63676;',
                'e' => '&#63118;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x3f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x3f, 0x0f),
                'i' => '&#63678;',
                'e' => '&#63113;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x40, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x40, 0x0f),
                'i' => pack('C*', 0x5b, 0x3f, 0x5d),
                'e' => '&#63067;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x41, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x41, 0x0f),
                'i' => '&#63911;',
                'e' => '&#63066;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x42, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x42, 0x0f),
                'i' => '&#63889;',
                'e' => '&#63410;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x43, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x43, 0x0f),
                'i' => '&#63891;',
                'e' => '&#63055;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x44, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x44, 0x0f),
                'i' => '&#63838;',
                'e' => '&#63409;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x45, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x45, 0x0f),
                'i' => '&#63838;',
                'e' => '&#63409;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x46, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x46, 0x0f),
                'i' => '&#63838;',
                'e' => '&#63409;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x47, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x47, 0x0f),
                'i' => '&#63838;',
                'e' => '&#63409;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x48, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x48, 0x0f),
                'i' => '&#63838;',
                'e' => '&#63409;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x49, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x49, 0x0f),
                'i' => '&#63838;',
                'e' => '&#63409;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x4a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x4a, 0x0f),
                'i' => '&#63838;',
                'e' => '&#63409;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x4b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x4b, 0x0f),
                'i' => '&#63838;',
                'e' => '&#63409;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x4c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x4c, 0x0f),
                'i' => '&#63838;',
                'e' => '&#63409;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x4d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x4d, 0x0f),
                'i' => '&#63838;',
                'e' => '&#63409;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x4e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x4e, 0x0f),
                'i' => '&#63838;',
                'e' => '&#63409;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x4f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x4f, 0x0f),
                'i' => '&#63838;',
                'e' => '&#63409;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x50, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x50, 0x0f),
                'i' => '&#63981;',
                'e' => '&#63139;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x51, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x51, 0x0f),
                'i' => '&#63935;',
                'e' => '&#63481;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x52, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x52, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63466;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x53, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x53, 0x0f),
                'i' => '&#63816;',
                'e' => '&#63138;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x54, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x54, 0x0f),
                'i' => '&#63936;',
                'e' => '&#63213;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x55, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x55, 0x0f),
                'i' => '&#63936;',
                'e' => '&#63213;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x56, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x56, 0x0f),
                'i' => '&#63684;',
                'e' => '&#63108;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x57, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x57, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63467;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x58, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x58, 0x0f),
                'i' => '&#63685;',
                'e' => '&#63110;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x59, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x59, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62577;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x5a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5a, 0x0f),
                'i' => '&#63692;',
                'e' => '&#63374;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x5b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5b, 0x0f),
                'i' => '&#63973;',
                'e' => '&#63469;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x5c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5c, 0x0f),
                'i' => '&#63703;',
                'e' => '&#63196;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x5d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5d, 0x0f),
                'i' => '&#63704;',
                'e' => '&#63216;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x5e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5e, 0x0f),
                'i' => '&#63899;',
                'e' => '&#63470;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x5f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x5f, 0x0f),
                'i' => '&#63869;',
                'e' => '&#63218;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x60, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x60, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x61, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x61, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63199;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x62, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x62, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62384;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x63, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x63, 0x0f),
                'i' => '&#63696;',
                'e' => '&#63109;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x64, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x64, 0x0f),
                'i' => '&#63698;',
                'e' => '&#63131;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x65, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x65, 0x0f),
                'i' => '&#63697;',
                'e' => '&#63412;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x66, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x66, 0x0f),
                'i' => '&#63983;',
                'e' => '&#63145;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x67, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x67, 0x0f),
                'i' => '&#63699;',
                'e' => '&#63132;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x68, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x68, 0x0f),
                'i' => '&#63650;',
                'e' => '&#63069;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x69, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x69, 0x0f),
                'i' => '&#63648;',
                'e' => '&#63077;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x6a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6a, 0x0f),
                'i' => '&#63647;',
                'e' => '&#63072;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x6b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6b, 0x0f),
                'i' => '&#63649;',
                'e' => '&#63076;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x6c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6c, 0x0f),
                'i' => '&#63811;',
                'e' => '&#63070;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x6d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6d, 0x0f),
                'i' => '&#63647;',
                'e' => '&#62408;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x6e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6e, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63471;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x6f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x6f, 0x0f),
                'i' => '&#63814;',
                'e' => '&#63156;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x70, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x70, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63472;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x71, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x71, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63473;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x72, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x72, 0x0f),
                'i' => '&#63813;',
                'e' => '&#63162;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x73, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x73, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63474;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x74, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x74, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63048;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x75, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x75, 0x0f),
                'i' => '&#63989;',
                'e' => '&#63157;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x76, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x76, 0x0f),
                'i' => '&#63893;',
                'e' => '&#62369;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x77, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x77, 0x0f),
                'i' => '&#63893;',
                'e' => '&#63049;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x78, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x78, 0x0f),
                'i' => '&#63895;',
                'e' => '&#62359;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x79, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x79, 0x0f),
                'i' => '&#63894;',
                'e' => '&#63050;'
            ),
            pack('C*', 0x1b, 0x24, 0x47, 0x7a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x47, 0x7a, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63182;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x21, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x21, 0x0f),
                'i' => '&#63686;',
                'e' => '&#63220;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x22, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x22, 0x0f),
                'i' => '&#63686;',
                'e' => '&#63220;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x23, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x23, 0x0f),
                'i' => '&#63859;',
                'e' => '&#62566;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x24, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x24, 0x0f),
                'i' => '&#63858;',
                'e' => '&#63455;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x25, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x25, 0x0f),
                'i' => '&#63949;',
                'e' => '&#63168;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x26, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x26, 0x0f),
                'i' => '&#63947;',
                'e' => '&#63476;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x27, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x27, 0x0f),
                'i' => '&#63996;',
                'e' => '&#63477;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x28, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x28, 0x0f),
                'i' => '&#63944;',
                'e' => '&#63478;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x29, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x29, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63154;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x2a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2a, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63479;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x2b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2b, 0x0f),
                'i' => '&#63994;',
                'e' => '&#63159;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x2c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2c, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63173;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x2d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2d, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63480;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x2e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2e, 0x0f),
                'i' => '&#63935;',
                'e' => '&#63481;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x2f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x2f, 0x0f),
                'i' => '&#63904;',
                'e' => '&#63054;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x30, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x30, 0x0f),
                'i' => '&#63974;',
                'e' => '&#63212;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x31, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x31, 0x0f),
                'i' => '&#63902;',
                'e' => '&#63482;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x32, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x32, 0x0f),
                'i' => '&#63718;',
                'e' => '&#63144;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x33, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x33, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63203;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x34, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x34, 0x0f),
                'i' => '&#63873;',
                'e' => '&#63217;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x35, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x35, 0x0f),
                'i' => '&#63960;',
                'e' => '&#63043;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x36, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x36, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63483;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x37, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x37, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63484;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x38, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x38, 0x0f),
                'i' => '&#63980;',
                'e' => '&#63143;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x39, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x39, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62272;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x3a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x3a, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63176;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x3b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x3b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63140;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x3c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x3c, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63185;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x3d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x3d, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63059;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x3e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x3e, 0x0f),
                'i' => '&#63715;',
                'e' => '&#62273;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x3f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x3f, 0x0f),
                'i' => '&#63830;',
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x40, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x40, 0x0f),
                'i' => '&#63700;',
                'e' => '&#63151;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x41, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x41, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62274;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x42, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x42, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62275;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x43, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x43, 0x0f),
                'i' => '&#63900;',
                'e' => '&#63125;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x44, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x44, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63045;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x45, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x45, 0x0f),
                'i' => '&#63711;',
                'e' => '&#63094;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x46, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x46, 0x0f),
                'i' => '&#63725;',
                'e' => '&#63205;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x47, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x47, 0x0f),
                'i' => '&#63725;',
                'e' => '&#63205;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x48, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x48, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63465;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x49, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x49, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63389;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x4a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x4a, 0x0f),
                'i' => '&#63723;',
                'e' => '&#63195;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x4b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x4b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63173;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x4c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x4c, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x4d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x4d, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62276;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x4e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x4e, 0x0f),
                'i' => pack('C*', 0x5b, 0x56, 0x53, 0x5d),
                'e' => '&#62277;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x4f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x4f, 0x0f),
                'i' => '&#63930;',
                'e' => '&#63136;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x50, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x50, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63134;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x51, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x51, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62278;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x52, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x52, 0x0f),
                'i' => '&#63674;',
                'e' => '&#63122;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x53, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x53, 0x0f),
                'i' => pack('C*', 0x5b, 0x37, 0x37, 0x37, 0x5d),
                'e' => '&#63046;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x54, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x54, 0x0f),
                'i' => '&#63993;',
                'e' => '&#63153;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x55, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x55, 0x0f),
                'i' => '&#63815;',
                'e' => '&#63117;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x56, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x56, 0x0f),
                'i' => '&#63938;',
                'e' => '&#63111;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x57, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x57, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62282;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x58, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x58, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0x89, 0x5d),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x59, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x59, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0x8a, 0x5d),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x5a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5a, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62425;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x5b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63209;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x5c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5c, 0x0f),
                'i' => '&#63910;',
                'e' => '&#63053;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x5d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5d, 0x0f),
                'i' => '&#63651;',
                'e' => '&#63071;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x5e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5e, 0x0f),
                'i' => '&#63701;',
                'e' => '&#63219;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x5f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x5f, 0x0f),
                'i' => '&#63900;',
                'e' => '&#62283;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x60, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x60, 0x0f),
                'i' => '&#63695;',
                'e' => '&#63101;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x61, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x61, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63210;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x62, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x62, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63210;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x63, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x63, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62284;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x64, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x64, 0x0f),
                'i' => '&#63869;',
                'e' => '&#63221;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x65, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x65, 0x0f),
                'i' => '&#63869;',
                'e' => '&#63221;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x66, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x66, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62285;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x67, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x67, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63146;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x68, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x68, 0x0f),
                'i' => '&#63716;',
                'e' => '&#63095;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x69, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x69, 0x0f),
                'i' => pack('C*', 0x5b, 0x24, 0x5c, 0x5d),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x6a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6a, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62287;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x6b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63105;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x6c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6c, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63170;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x6d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6d, 0x0f),
                'i' => '&#63688;',
                'e' => '&#63107;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x6e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6e, 0x0f),
                'i' => '&#63694;',
                'e' => '&#63042;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x6f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x6f, 0x0f),
                'i' => '&#63693;',
                'e' => '&#63102;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x70, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x70, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63104;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x71, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x71, 0x0f),
                'i' => '&#63695;',
                'e' => '&#63101;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x72, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x72, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62288;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x73, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x73, 0x0f),
                'i' => '&#63686;',
                'e' => '&#62289;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x74, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x74, 0x0f),
                'i' => '&#63689;',
                'e' => '&#63099;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x75, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x75, 0x0f),
                'i' => '&#63687;',
                'e' => '&#62290;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x76, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x76, 0x0f),
                'i' => '&#63691;',
                'e' => '&#63100;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x77, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x77, 0x0f),
                'i' => '&#63971;',
                'e' => '&#62291;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x78, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x78, 0x0f),
                'i' => '&#63690;',
                'e' => '&#62292;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x79, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x79, 0x0f),
                'i' => '&#63681;',
                'e' => '&#63112;'
            ),
            pack('C*', 0x1b, 0x24, 0x45, 0x7a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x45, 0x7a, 0x0f),
                'i' => '&#63679;',
                'e' => '&#63114;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x21, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x21, 0x0f),
                'i' => '&#63960;',
                'e' => '&#62582;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x22, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x22, 0x0f),
                'i' => '&#63682;',
                'e' => '&#62293;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x23, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x23, 0x0f),
                'i' => pack('C*', 0x5b, 0xba, 0xba, 0x5d),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x24, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x24, 0x0f),
                'i' => '&#63901;',
                'e' => '&#63410;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x25, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x25, 0x0f),
                'i' => '&#63901;',
                'e' => '&#63057;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x26, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x26, 0x0f),
                'i' => '&#63901;',
                'e' => '&#63322;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x27, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x27, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62294;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x28, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x28, 0x0f),
                'i' => '&#63713;',
                'e' => '&#63062;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x29, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x29, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63064;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x2a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2a, 0x0f),
                'i' => '&#63740;',
                'e' => '&#63063;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x2b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62295;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x2c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2c, 0x0f),
                'i' => '&#63726;',
                'e' => '&#62328;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x2d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2d, 0x0f),
                'i' => '&#63728;',
                'e' => '&#63423;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x2e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2e, 0x0f),
                'i' => '&#63727;',
                'e' => '&#63422;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x2f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x2f, 0x0f),
                'i' => '&#63729;',
                'e' => '&#63424;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x30, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x30, 0x0f),
                'i' => '&#63877;',
                'e' => '&#62601;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x31, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x31, 0x0f),
                'i' => '&#63876;',
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x32, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x32, 0x0f),
                'i' => '&#63874;',
                'e' => '&#63461;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x33, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x33, 0x0f),
                'i' => pack('C*', 0x5b, 0x55, 0x50, 0x5d),
                'e' => '&#63208;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x34, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x34, 0x0f),
                'i' => pack('C*', 0x5b, 0x43, 0x4f, 0x4f, 0x4c, 0x5d),
                'e' => '&#62296;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x35, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x35, 0x0f),
                'i' => pack('C*', 0x5b, 0x97, 0x4c, 0x5d),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x36, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x36, 0x0f),
                'i' => pack('C*', 0x5b, 0x96, 0xb3, 0x5d),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x37, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x37, 0x0f),
                'i' => pack('C*', 0x5b, 0x8c, 0x8e, 0x5d),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x38, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x38, 0x0f),
                'i' => pack('C*', 0x5b, 0x90, 0x5c, 0x5d),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x39, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x39, 0x0f),
                'i' => '&#63808;',
                'e' => '&#63334;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x3a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3a, 0x0f),
                'i' => '&#63808;',
                'e' => '&#63335;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x3b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3b, 0x0f),
                'i' => '&#63808;',
                'e' => '&#63335;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x3c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3c, 0x0f),
                'i' => '&#63879;',
                'e' => '&#63227;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x3d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3d, 0x0f),
                'i' => '&#63880;',
                'e' => '&#63228;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x3e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3e, 0x0f),
                'i' => '&#63881;',
                'e' => '&#63296;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x3f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x3f, 0x0f),
                'i' => '&#63882;',
                'e' => '&#63297;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x40, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x40, 0x0f),
                'i' => '&#63883;',
                'e' => '&#63298;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x41, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x41, 0x0f),
                'i' => '&#63884;',
                'e' => '&#63299;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x42, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x42, 0x0f),
                'i' => '&#63885;',
                'e' => '&#63300;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x43, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x43, 0x0f),
                'i' => '&#63886;',
                'e' => '&#63301;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x44, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x44, 0x0f),
                'i' => '&#63887;',
                'e' => '&#63302;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x45, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x45, 0x0f),
                'i' => '&#63888;',
                'e' => '&#63433;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x46, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x46, 0x0f),
                'i' => pack('C*', 0x5b, 0x93, 0xbe, 0x5d),
                'e' => '&#63184;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x47, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x47, 0x0f),
                'i' => pack('C*', 0x5b, 0x8a, 0x84, 0x5d),
                'e' => '&#62297;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x48, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x48, 0x0f),
                'i' => pack('C*', 0x5b, 0x83, 0x54, 0x5d),
                'e' => '&#62298;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x49, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x49, 0x0f),
                'i' => '&#63868;',
                'e' => '&#62299;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x4a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4a, 0x0f),
                'i' => '&#63968;',
                'e' => '&#62300;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x4b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4b, 0x0f),
                'i' => '&#63966;',
                'e' => '&#62301;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x4c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4c, 0x0f),
                'i' => pack('C*', 0x5b, 0x8e, 0x77, 0x5d),
                'e' => '&#62302;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x4d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4d, 0x0f),
                'i' => pack('C*', 0x5b, 0x89, 0x63, 0x5d),
                'e' => '&#62303;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x4e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4e, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0xaa, 0x5d),
                'e' => '&#62304;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x4f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x4f, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0xab, 0x5d),
                'e' => '&#62305;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x50, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x50, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0xa9, 0x5d),
                'e' => '&#63192;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x51, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x51, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0xa8, 0x5d),
                'e' => '&#63193;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x52, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x52, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0xaa, 0x5d),
                'e' => '&#63323;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x53, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x53, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0xab, 0x5d),
                'e' => '&#63324;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x54, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x54, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0xa8, 0x5d),
                'e' => '&#63342;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x55, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x55, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0xa9, 0x5d),
                'e' => '&#63343;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x56, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x56, 0x0f),
                'i' => '&#63705;',
                'e' => '&#63345;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x57, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x57, 0x0f),
                'i' => '&#63736;',
                'e' => '&#63336;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x58, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x58, 0x0f),
                'i' => '&#63735;',
                'e' => '&#63337;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x59, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x59, 0x0f),
                'i' => '&#63817;',
                'e' => '&#63346;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x5a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x5a, 0x0f),
                'i' => pack('C*', 0x5b, 0x3e, 0x5d),
                'e' => '&#63306;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x5b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x5b, 0x0f),
                'i' => pack('C*', 0x5b, 0x3c, 0x5d),
                'e' => '&#63305;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x5c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x5c, 0x0f),
                'i' => pack('C*', 0x5b, 0x3e, 0x3e, 0x5d),
                'e' => '&#63308;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x5d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x5d, 0x0f),
                'i' => pack('C*', 0x5b, 0x3c, 0x3c, 0x5d),
                'e' => '&#63307;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x5e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x5e, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62306;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x5f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x5f, 0x0f),
                'i' => '&#63655;',
                'e' => '&#63079;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x60, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x60, 0x0f),
                'i' => '&#63656;',
                'e' => '&#63080;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x61, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x61, 0x0f),
                'i' => '&#63657;',
                'e' => '&#63081;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x62, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x62, 0x0f),
                'i' => '&#63658;',
                'e' => '&#63082;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x63, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x63, 0x0f),
                'i' => '&#63659;',
                'e' => '&#63083;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x64, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x64, 0x0f),
                'i' => '&#63660;',
                'e' => '&#63084;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x65, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x65, 0x0f),
                'i' => '&#63661;',
                'e' => '&#63085;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x66, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x66, 0x0f),
                'i' => '&#63662;',
                'e' => '&#63086;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x67, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x67, 0x0f),
                'i' => '&#63663;',
                'e' => '&#63087;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x68, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x68, 0x0f),
                'i' => '&#63664;',
                'e' => '&#63088;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x69, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x69, 0x0f),
                'i' => '&#63665;',
                'e' => '&#63089;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x6a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6a, 0x0f),
                'i' => '&#63666;',
                'e' => '&#63090;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x6b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63091;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x6c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6c, 0x0f),
                'i' => pack('C*', 0x5b, 0x54, 0x4f, 0x50, 0x5d),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x6d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6d, 0x0f),
                'i' => '&#63920;',
                'e' => '&#63434;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x6e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6e, 0x0f),
                'i' => '&#63958;',
                'e' => '&#63348;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x6f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x6f, 0x0f),
                'i' => '&#63963;',
                'e' => '&#63349;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x70, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x70, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62307;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x71, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x71, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62308;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x72, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x72, 0x0f),
                'i' => '&#63964;',
                'e' => '&#63065;'
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x73, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x73, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x74, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x74, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x75, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x75, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x76, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x76, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x77, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x77, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x78, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x78, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x79, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x79, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x46, 0x7a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x46, 0x7a, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x21, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x21, 0x0f),
                'i' => '&#63722;',
                'e' => '&#62309;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x22, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x22, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62310;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x23, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x23, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62311;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x24, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x24, 0x0f),
                'i' => '&#63976;',
                'e' => '&#63165;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x25, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x25, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63164;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x26, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x26, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62312;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x27, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x27, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63163;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x28, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x28, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62313;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x29, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x29, 0x0f),
                'i' => pack('C*', 0x5b, 0x57, 0x43, 0x5d),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x2a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2a, 0x0f),
                'i' => '&#63707;',
                'e' => '&#63201;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x2b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2b, 0x0f),
                'i' => '&#63984;',
                'e' => '&#62314;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x2c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2c, 0x0f),
                'i' => '&#63699;',
                'e' => '&#62315;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x2d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2d, 0x0f),
                'i' => pack('C*', 0x5b, 0x8f, 0x6a, 0x5d),
                'e' => '&#62316;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x2e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2e, 0x0f),
                'i' => '&#63712;',
                'e' => '&#63061;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x2f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x2f, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62317;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x30, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x30, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62318;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x31, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x31, 0x0f),
                'i' => '&#63907;',
                'e' => '&#63058;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x32, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x32, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62319;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x33, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x33, 0x0f),
                'i' => '&#63702;',
                'e' => '&#63215;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x34, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x34, 0x0f),
                'i' => '&#63717;',
                'e' => '&#63420;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x35, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x35, 0x0f),
                'i' => '&#63961;',
                'e' => '&#63178;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x36, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x36, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63391;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x37, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x37, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63210;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x38, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x38, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62321;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x39, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x39, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62575;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x3a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x3a, 0x0f),
                'i' => '&#63701;',
                'e' => '&#63219;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x3b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x3b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62322;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x3c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x3c, 0x0f),
                'i' => '&#63925;',
                'e' => '&#63202;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x3d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x3d, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62323;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x3e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x3e, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63204;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x3f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x3f, 0x0f),
                'i' => '&#63702;',
                'e' => '&#62324;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x40, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x40, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62325;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x41, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x41, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62326;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x42, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x42, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62327;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x43, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x43, 0x0f),
                'i' => '&#63715;',
                'e' => '&#63092;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x44, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x44, 0x0f),
                'i' => '&#63824;',
                'e' => '&#63127;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x45, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x45, 0x0f),
                'i' => '&#63928;',
                'e' => '&#63211;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x46, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x46, 0x0f),
                'i' => '&#63908;',
                'e' => '&#63198;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x47, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x47, 0x0f),
                'i' => '&#63889;',
                'e' => '&#62329;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x48, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x48, 0x0f),
                'i' => '&#63890;',
                'e' => '&#62585;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x49, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x49, 0x0f),
                'i' => '&#63889;',
                'e' => '&#63171;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x4a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4a, 0x0f),
                'i' => '&#63889;',
                'e' => '&#62330;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x4b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4b, 0x0f),
                'i' => '&#63889;',
                'e' => '&#62331;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x4c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4c, 0x0f),
                'i' => '&#63889;',
                'e' => '&#62332;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x4d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4d, 0x0f),
                'i' => '&#63889;',
                'e' => '&#62333;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x4e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4e, 0x0f),
                'i' => '&#63903;',
                'e' => '&#62334;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x4f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x4f, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0x99, 0x5d),
                'e' => '&#63075;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x50, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x50, 0x0f),
                'i' => '&#63917;',
                'e' => '&#63181;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x51, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x51, 0x0f),
                'i' => '&#63915;',
                'e' => '&#63438;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x52, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x52, 0x0f),
                'i' => '&#63812;',
                'e' => '&#62337;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x53, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x53, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0x7e, 0x5d),
                'e' => '&#63340;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x54, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x54, 0x0f),
                'i' => '&#63905;',
                'e' => '&#63166;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x55, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x55, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0x99, 0x5d),
                'e' => '&#63075;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x56, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x56, 0x0f),
                'i' => pack('C*', 0x5b, 0x3f, 0x5d),
                'e' => '&#63067;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x57, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x57, 0x0f),
                'i' => '&#63911;',
                'e' => '&#63066;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x58, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x58, 0x0f),
                'i' => '&#63939;',
                'e' => '&#62338;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x59, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x59, 0x0f),
                'i' => '&#63986;',
                'e' => '&#62339;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x5a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x5a, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62340;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x5b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x5b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62341;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x5c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x5c, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62342;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x5d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x5d, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62343;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x5e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x5e, 0x0f),
                'i' => '&#63985;',
                'e' => '&#62344;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x5f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x5f, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62345;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x60, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x60, 0x0f),
                'i' => '&#63985;',
                'e' => '&#63441;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x61, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x61, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62346;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x62, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x62, 0x0f),
                'i' => '&#63982;',
                'e' => '&#63150;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x63, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x63, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62347;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x64, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x64, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62348;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x65, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x65, 0x0f),
                'i' => '&#63978;',
                'e' => '&#62349;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x66, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x66, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62350;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x67, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x67, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63149;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x68, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x68, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63142;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x69, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x69, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62351;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x6a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x6a, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62352;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x6b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x6b, 0x0f),
                'i' => '&#63719;',
                'e' => '&#63421;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x6c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x6c, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62353;'
            ),
            pack('C*', 0x1b, 0x24, 0x4f, 0x6d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x4f, 0x6d, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62354;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x21, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x21, 0x0f),
                'i' => '&#63944;',
                'e' => '&#63478;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x22, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x22, 0x0f),
                'i' => '&#63953;',
                'e' => '&#62355;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x23, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x23, 0x0f),
                'i' => '&#63941;',
                'e' => '&#62356;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x24, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x24, 0x0f),
                'i' => '&#63992;',
                'e' => '&#62357;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x25, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x25, 0x0f),
                'i' => '&#63950;',
                'e' => '&#63475;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x26, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x26, 0x0f),
                'i' => '&#63952;',
                'e' => '&#62358;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x27, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x27, 0x0f),
                'i' => '&#63896;',
                'e' => '&#62359;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x28, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x28, 0x0f),
                'i' => '&#63910;',
                'e' => '&#62360;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x29, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x29, 0x0f),
                'i' => '&#63949;',
                'e' => '&#63168;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x2a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2a, 0x0f),
                'i' => '&#63942;',
                'e' => '&#62361;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x2b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2b, 0x0f),
                'i' => '&#63996;',
                'e' => '&#62362;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x2c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2c, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62363;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x2d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2d, 0x0f),
                'i' => '&#63951;',
                'e' => '&#62364;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x2e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2e, 0x0f),
                'i' => '&#63946;',
                'e' => '&#62365;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x2f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x2f, 0x0f),
                'i' => '&#63944;',
                'e' => '&#62362;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x30, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x30, 0x0f),
                'i' => '&#63897;',
                'e' => '&#62366;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x31, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x31, 0x0f),
                'i' => '&#63954;',
                'e' => '&#63051;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x32, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x32, 0x0f),
                'i' => '&#63954;',
                'e' => '&#62568;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x33, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x33, 0x0f),
                'i' => '&#63955;',
                'e' => '&#62573;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x34, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x34, 0x0f),
                'i' => '&#63893;',
                'e' => '&#62369;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x35, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x35, 0x0f),
                'i' => '&#63893;',
                'e' => '&#63049;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x36, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x36, 0x0f),
                'i' => '&#63945;',
                'e' => '&#62561;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x37, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x37, 0x0f),
                'i' => '&#63947;',
                'e' => '&#62370;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x38, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x38, 0x0f),
                'i' => '&#63947;',
                'e' => '&#62371;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x39, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x39, 0x0f),
                'i' => '&#63730;',
                'e' => '&#63425;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x3a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x3a, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62372;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x3b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x3b, 0x0f),
                'i' => '&#63731;',
                'e' => '&#63426;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x3c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x3c, 0x0f),
                'i' => '&#63902;',
                'e' => '&#62373;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x3d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x3d, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62374;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x3e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x3e, 0x0f),
                'i' => '&#63734;',
                'e' => '&#62378;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x3f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x3f, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62375;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x40, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x40, 0x0f),
                'i' => '&#63920;',
                'e' => '&#62376;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x41, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x41, 0x0f),
                'i' => '&#63909;',
                'e' => '&#62377;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x42, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x42, 0x0f),
                'i' => '&#63734;',
                'e' => '&#62378;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x43, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x43, 0x0f),
                'i' => '&#63956;',
                'e' => '&#62379;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x44, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x44, 0x0f),
                'i' => '&#63920;',
                'e' => '&#62380;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x45, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x45, 0x0f),
                'i' => '&#63890;',
                'e' => '&#62382;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x46, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x46, 0x0f),
                'i' => pack('C*', 0x5b, 0x6d, 0x28, 0x5f, 0x5f, 0x29, 0x6d, 0x5d),
                'e' => '&#62381;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x47, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x47, 0x0f),
                'i' => pack('C*', 0x5b, 0x81, 0x5f, 0x28, 0x5e, 0x6f, 0x5e, 0x29, 0x81, 0x5e, 0x5d),
                'e' => '&#62603;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x48, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x48, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x49, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x49, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62383;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x4a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x4a, 0x0f),
                'i' => '&#63673;',
                'e' => '&#63415;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x4b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x4b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63124;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x4c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x4c, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62385;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x4d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x4d, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62386;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x4e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x4e, 0x0f),
                'i' => '&#63680;',
                'e' => '&#63114;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x4f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x4f, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63115;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x50, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x50, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62387;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x51, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x51, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62388;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x52, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x52, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62389;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x53, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x53, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62390;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x54, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x54, 0x0f),
                'i' => '&#63677;',
                'e' => '&#63468;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x55, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x55, 0x0f),
                'i' => '&#63678;',
                'e' => '&#63113;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x56, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x56, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62391;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x57, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x57, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62552;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x58, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x58, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62392;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x59, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x59, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62393;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x5a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x5a, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62394;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x5b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x5b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62395;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x5c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x5c, 0x0f),
                'i' => '&#63654;',
                'e' => '&#62396;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x5d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x5d, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63467;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x5e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x5e, 0x0f),
                'i' => '&#63972;',
                'e' => '&#62593;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x5f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x5f, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62398;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x60, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x60, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62399;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x61, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x61, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62400;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x62, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x62, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62401;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x63, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x63, 0x0f),
                'i' => '&#63652;',
                'e' => '&#63041;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x64, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x64, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x65, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x65, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62402;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x66, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x66, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62403;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x67, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x67, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62272;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x68, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x68, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62404;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x69, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x69, 0x0f),
                'i' => '&#63647;',
                'e' => '&#62408;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x6a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x6a, 0x0f),
                'i' => '&#63647;',
                'e' => '&#62285;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x6b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x6b, 0x0f),
                'i' => '&#63831;',
                'e' => '&#62405;'
            ),
            pack('C*', 0x1b, 0x24, 0x50, 0x6c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x50, 0x6c, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62406;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x21, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x21, 0x0f),
                'i' => '&#63690;',
                'e' => '&#62407;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x22, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x22, 0x0f),
                'i' => '&#63708;',
                'e' => '&#63417;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x23, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x23, 0x0f),
                'i' => '&#63709;',
                'e' => '&#62409;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x24, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x24, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62410;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x25, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x25, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62411;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x26, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x26, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62412;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x27, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x27, 0x0f),
                'i' => '&#63704;',
                'e' => '&#63216;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x28, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x28, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62413;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x29, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x29, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63129;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x2a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x2a, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x2b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x2b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63141;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x2c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x2c, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63376;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x2d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x2d, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62414;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x2e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x2e, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62415;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x2f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x2f, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62416;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x30, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x30, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62417;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x31, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x31, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62280;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x32, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x32, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62281;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x33, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x33, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62418;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x34, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x34, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62419;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x35, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x35, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62420;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x36, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x36, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62421;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x37, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x37, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62422;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x38, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x38, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62423;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x39, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x39, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62424;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x3a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x3a, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62425;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x3b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x3b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62426;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x3c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x3c, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62427;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x3d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x3d, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x3e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x3e, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x3f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x3f, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62429;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x40, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x40, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62428;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x41, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x41, 0x0f),
                'i' => '&#63988;',
                'e' => '&#63161;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x42, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x42, 0x0f),
                'i' => '&#63990;',
                'e' => '&#62430;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x43, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x43, 0x0f),
                'i' => '&#63988;',
                'e' => '&#63161;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x44, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x44, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x45, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x45, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62431;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x46, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x46, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62432;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x47, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x47, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62433;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x48, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x48, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63154;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x49, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x49, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63079;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x4a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x4a, 0x0f),
                'i' => '&#63813;',
                'e' => '&#63162;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x4b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x4b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62434;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x4c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x4c, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63152;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x4d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x4d, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62435;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x4e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x4e, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62436;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x4f, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x4f, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62437;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x50, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x50, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#62438;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x51, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x51, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => '&#63155;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x52, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x52, 0x0f),
                'i' => pack('C*', 0x5b, 0x41, 0x5d),
                'e' => '&#62439;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x53, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x53, 0x0f),
                'i' => pack('C*', 0x5b, 0x42, 0x5d),
                'e' => '&#62440;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x54, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x54, 0x0f),
                'i' => pack('C*', 0x5b, 0x41, 0x42, 0x5d),
                'e' => '&#62442;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x55, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x55, 0x0f),
                'i' => pack('C*', 0x5b, 0x4f, 0x5d),
                'e' => '&#62441;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x56, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x56, 0x0f),
                'i' => '&#63737;',
                'e' => '&#62443;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x57, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x57, 0x0f),
                'i' => '&#63959;',
                'e' => '&#63338;'
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x58, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x58, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x51, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x51, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x5a, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x5a, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x5b, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x5b, 0x0f),
                'i' => pack('C*', 0x81, 0xac),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x5c, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x5c, 0x0f),
                'i' => pack('C*', 0x5b, 0x76),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x5d, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x5d, 0x0f),
                'i' => pack('C*', 0x6f, 0x64, 0x61),
                'e' => null
            ),
            pack('C*', 0x1b, 0x24, 0x51, 0x5e, 0x0f) => array(
                's' => pack('C*', 0x1b, 0x24, 0x51, 0x5e, 0x0f),
                'i' => pack('C*', 0x66, 0x6f, 0x6e, 0x65, 0x5d),
                'e' => null
            )
        );
        return $table;
    }

    /**
     * EZ絵文字番号エンティティ変換テーブル
     *
     * @return array
     * @static
     */
    public static function getAuEmojiNum2EntytyArray()
    {
        $table = array(
            1 => '&#63065;',
            2 => '&#63066;',
            3 => '&#63067;',
            4 => '&#63304;',
            5 => '&#63305;',
            6 => '&#63306;',
            7 => '&#63307;',
            8 => '&#63308;',
            9 => '&#63309;',
            10 => '&#63310;',
            11 => '&#63311;',
            12 => '&#63130;',
            13 => '&#63210;',
            14 => '&#63382;',
            15 => '&#63070;',
            16 => '&#63071;',
            17 => '&#63312;',
            18 => '&#63313;',
            19 => '&#63314;',
            20 => '&#63315;',
            21 => '&#63316;',
            22 => '&#63317;',
            23 => '&#63318;',
            24 => '&#63319;',
            25 => '&#63383;',
            26 => '&#63320;',
            27 => '&#63321;',
            28 => '&#63322;',
            29 => '&#63323;',
            30 => '&#63324;',
            31 => '&#63325;',
            32 => '&#63326;',
            33 => '&#63327;',
            34 => '&#63328;',
            35 => '&#63329;',
            36 => '&#63330;',
            37 => '&#63331;',
            38 => '&#63332;',
            39 => '&#63333;',
            40 => '&#63334;',
            41 => '&#63335;',
            42 => '&#63336;',
            43 => '&#63337;',
            44 => '&#63072;',
            45 => '&#63123;',
            46 => '&#63409;',
            47 => '&#63073;',
            48 => '&#63211;',
            49 => '&#63356;',
            50 => '&#63187;',
            51 => '&#63410;',
            52 => '&#63131;',
            53 => '&#63212;',
            54 => '&#63338;',
            55 => '&#63339;',
            56 => '&#63357;',
            57 => '&#63384;',
            58 => '&#63060;',
            59 => '&#63358;',
            60 => '&#63074;',
            61 => '&#63340;',
            62 => '&#63341;',
            63 => '&#63342;',
            64 => '&#63343;',
            65 => '&#63132;',
            66 => '&#63344;',
            67 => '&#63360;',
            68 => '&#63188;',
            69 => '&#63075;',
            70 => '&#63345;',
            71 => '&#63346;',
            72 => '&#63213;',
            73 => '&#63347;',
            74 => '&#63160;',
            75 => '&#63040;',
            76 => '&#63044;',
            77 => '&#63054;',
            78 => '&#63161;',
            79 => '&#63404;',
            80 => '&#63189;',
            81 => '&#63348;',
            82 => '&#63349;',
            83 => '&#63092;',
            84 => '&#63405;',
            85 => '&#63411;',
            86 => '&#63190;',
            87 => '&#63385;',
            88 => '&#63350;',
            89 => '&#63351;',
            90 => '&#63376;',
            91 => '&#63093;',
            92 => '&#63361;',
            93 => '&#63412;',
            94 => '&#63214;',
            95 => '&#63076;',
            96 => '&#63124;',
            97 => '&#63362;',
            98 => '&#63068;',
            99 => '&#63042;',
            100 => '&#63363;',
            101 => '&#63364;',
            102 => '&#63365;',
            103 => '&#63366;',
            104 => '&#63215;',
            105 => '&#63367;',
            106 => '&#63094;',
            107 => '&#63077;',
            108 => '&#63226;',
            109 => '&#63386;',
            110 => '&#63216;',
            111 => '&#63387;',
            112 => '&#63108;',
            113 => '&#63165;',
            114 => '&#63388;',
            115 => '&#63389;',
            116 => '&#63191;',
            117 => '&#63352;',
            118 => '&#63353;',
            119 => '&#63217;',
            120 => '&#63218;',
            121 => '&#63368;',
            122 => '&#63095;',
            123 => '&#63390;',
            124 => '&#63219;',
            125 => '&#63114;',
            126 => '&#63391;',
            127 => '&#63377;',
            128 => '&#63378;',
            129 => '&#63220;',
            130 => '&#63392;',
            131 => '&#63369;',
            132 => '&#63354;',
            133 => '&#63143;',
            134 => '&#63162;',
            135 => '&#63393;',
            136 => '&#63355;',
            137 => '&#63370;',
            138 => '&#63221;',
            139 => '&#63394;',
            140 => '&#63192;',
            141 => '&#63193;',
            142 => '&#63371;',
            143 => '&#63096;',
            144 => '&#63144;',
            145 => '&#63222;',
            146 => '&#63109;',
            147 => '&#63372;',
            148 => '&#63115;',
            149 => '&#63097;',
            150 => '&#63395;',
            151 => '&#63406;',
            152 => '&#63396;',
            153 => '&#63407;',
            154 => '&#63408;',
            155 => '&#63223;',
            156 => '&#63110;',
            157 => '&#63373;',
            158 => '&#63098;',
            159 => '&#63379;',
            160 => '&#63133;',
            161 => '&#63397;',
            162 => '&#63398;',
            163 => '&#63194;',
            164 => '&#63399;',
            165 => '&#63224;',
            166 => '&#63225;',
            167 => '&#63078;',
            168 => '&#63116;',
            169 => '&#63117;',
            170 => '&#63137;',
            171 => '&#63400;',
            172 => '&#63118;',
            173 => '&#63401;',
            174 => '&#63402;',
            175 => '&#63403;',
            176 => '&#63061;',
            177 => '&#63062;',
            178 => '&#63063;',
            179 => '&#63064;',
            180 => '&#63227;',
            181 => '&#63228;',
            182 => '&#63296;',
            183 => '&#63297;',
            184 => '&#63298;',
            185 => '&#63299;',
            186 => '&#63300;',
            187 => '&#63301;',
            188 => '&#63302;',
            189 => '&#63303;',
            190 => '&#63041;',
            191 => '&#63069;',
            192 => '&#63079;',
            193 => '&#63080;',
            194 => '&#63081;',
            195 => '&#63082;',
            196 => '&#63083;',
            197 => '&#63084;',
            198 => '&#63085;',
            199 => '&#63086;',
            200 => '&#63087;',
            201 => '&#63088;',
            202 => '&#63089;',
            203 => '&#63090;',
            204 => '&#63091;',
            205 => '&#63099;',
            206 => '&#63100;',
            207 => '&#63101;',
            208 => '&#63102;',
            209 => '&#63104;',
            210 => '&#63105;',
            211 => '&#63106;',
            212 => '&#63107;',
            213 => '&#63374;',
            214 => '&#63375;',
            215 => '&#63111;',
            216 => '&#63112;',
            217 => '&#63113;',
            218 => '&#63043;',
            219 => '&#63119;',
            220 => '&#63120;',
            221 => '&#63121;',
            222 => '&#63122;',
            223 => '&#63045;',
            224 => '&#63125;',
            225 => '&#63126;',
            226 => '&#63127;',
            227 => '&#63128;',
            228 => '&#63129;',
            229 => '&#63046;',
            230 => '&#63047;',
            231 => '&#63134;',
            232 => '&#63135;',
            233 => '&#63136;',
            234 => '&#63138;',
            235 => '&#63139;',
            236 => '&#63140;',
            237 => '&#63141;',
            238 => '&#63142;',
            239 => '&#63145;',
            240 => '&#63146;',
            241 => '&#63147;',
            242 => '&#63148;',
            243 => '&#63149;',
            244 => '&#63150;',
            245 => '&#63151;',
            246 => '&#63048;',
            247 => '&#63152;',
            248 => '&#63153;',
            249 => '&#63154;',
            250 => '&#63155;',
            251 => '&#63156;',
            252 => '&#63157;',
            253 => '&#63158;',
            254 => '&#63159;',
            255 => '&#63163;',
            256 => '&#63164;',
            257 => '&#63049;',
            258 => '&#63050;',
            259 => '&#63051;',
            260 => '&#63052;',
            261 => '&#63053;',
            262 => '&#63166;',
            263 => '&#63167;',
            264 => '&#63168;',
            265 => '&#63055;',
            266 => '&#63056;',
            267 => '&#63057;',
            268 => '&#63058;',
            269 => '&#63059;',
            270 => '&#63169;',
            271 => '&#63170;',
            272 => '&#63171;',
            273 => '&#63172;',
            274 => '&#63173;',
            275 => '&#63174;',
            276 => '&#63175;',
            277 => '&#63176;',
            278 => '&#63177;',
            279 => '&#63178;',
            280 => '&#63179;',
            281 => '&#63180;',
            282 => '&#63181;',
            283 => '&#63182;',
            284 => '&#63183;',
            285 => '&#63184;',
            286 => '&#63185;',
            287 => '&#63186;',
            288 => '&#63195;',
            289 => '&#63196;',
            290 => '&#63197;',
            291 => '&#63198;',
            292 => '&#63199;',
            293 => '&#63200;',
            294 => '&#63201;',
            295 => '&#63202;',
            296 => '&#63203;',
            297 => '&#63204;',
            298 => '&#63380;',
            299 => '&#63381;',
            300 => '&#63205;',
            301 => '&#63206;',
            302 => '&#63207;',
            303 => '&#63208;',
            304 => '&#63209;',
            305 => '&#63413;',
            306 => '&#63414;',
            307 => '&#63415;',
            308 => '&#63416;',
            309 => '&#63417;',
            310 => '&#63418;',
            311 => '&#63419;',
            312 => '&#63420;',
            313 => '&#63421;',
            314 => '&#63422;',
            315 => '&#63423;',
            316 => '&#63424;',
            317 => '&#63425;',
            318 => '&#63426;',
            319 => '&#63427;',
            320 => '&#63428;',
            321 => '&#63429;',
            322 => '&#63430;',
            323 => '&#63431;',
            324 => '&#63432;',
            325 => '&#63433;',
            326 => '&#63434;',
            327 => '&#63435;',
            328 => '&#63436;',
            329 => '&#63437;',
            330 => '&#63438;',
            331 => '&#63439;',
            332 => '&#63440;',
            333 => '&#63441;',
            334 => '&#63461;',
            335 => '&#63462;',
            336 => '&#63463;',
            337 => '&#63464;',
            338 => '&#63465;',
            339 => '&#63466;',
            340 => '&#63467;',
            341 => '&#63468;',
            342 => '&#63469;',
            343 => '&#63470;',
            344 => '&#63471;',
            345 => '&#63472;',
            346 => '&#63473;',
            347 => '&#63474;',
            348 => '&#63475;',
            349 => '&#63476;',
            350 => '&#63477;',
            351 => '&#63478;',
            352 => '&#63479;',
            353 => '&#63480;',
            354 => '&#63481;',
            355 => '&#63482;',
            356 => '&#63483;',
            357 => '&#63484;',
            358 => '&#62272;',
            359 => '&#62273;',
            360 => '&#62274;',
            361 => '&#62275;',
            362 => '&#62276;',
            363 => '&#62277;',
            364 => '&#62278;',
            365 => '&#62279;',
            366 => '&#62280;',
            367 => '&#62281;',
            368 => '&#62282;',
            369 => '&#62283;',
            370 => '&#62284;',
            371 => '&#62285;',
            372 => '&#62286;',
            373 => '&#62287;',
            374 => '&#62288;',
            375 => '&#62289;',
            376 => '&#62290;',
            377 => '&#62291;',
            378 => '&#62292;',
            379 => '&#62293;',
            380 => '&#62294;',
            381 => '&#62295;',
            382 => '&#62296;',
            383 => '&#62297;',
            384 => '&#62298;',
            385 => '&#62299;',
            386 => '&#62300;',
            387 => '&#62301;',
            388 => '&#62302;',
            389 => '&#62303;',
            390 => '&#62304;',
            391 => '&#62305;',
            392 => '&#62306;',
            393 => '&#62307;',
            394 => '&#62308;',
            395 => '&#62309;',
            396 => '&#62310;',
            397 => '&#62311;',
            398 => '&#62312;',
            399 => '&#62313;',
            400 => '&#62314;',
            401 => '&#62315;',
            402 => '&#62316;',
            403 => '&#62317;',
            404 => '&#62318;',
            405 => '&#62319;',
            406 => '&#62320;',
            407 => '&#62321;',
            408 => '&#62322;',
            409 => '&#62323;',
            410 => '&#62324;',
            411 => '&#62325;',
            412 => '&#62326;',
            413 => '&#62327;',
            414 => '&#62328;',
            415 => '&#62329;',
            416 => '&#62330;',
            417 => '&#62331;',
            418 => '&#62332;',
            419 => '&#62333;',
            420 => '&#62334;',
            421 => '&#62336;',
            422 => '&#62337;',
            423 => '&#62338;',
            424 => '&#62339;',
            425 => '&#62340;',
            426 => '&#62341;',
            427 => '&#62342;',
            428 => '&#62343;',
            429 => '&#62344;',
            430 => '&#62345;',
            431 => '&#62346;',
            432 => '&#62347;',
            433 => '&#62348;',
            434 => '&#62349;',
            435 => '&#62350;',
            436 => '&#62351;',
            437 => '&#62352;',
            438 => '&#62353;',
            439 => '&#62354;',
            440 => '&#62355;',
            441 => '&#62356;',
            442 => '&#62357;',
            443 => '&#62358;',
            444 => '&#62359;',
            445 => '&#62360;',
            446 => '&#62361;',
            447 => '&#62362;',
            448 => '&#62363;',
            449 => '&#62364;',
            450 => '&#62365;',
            451 => '&#62366;',
            452 => '&#62367;',
            453 => '&#62368;',
            454 => '&#62369;',
            455 => '&#62370;',
            456 => '&#62371;',
            457 => '&#62372;',
            458 => '&#62373;',
            459 => '&#62374;',
            460 => '&#62375;',
            461 => '&#62376;',
            462 => '&#62377;',
            463 => '&#62378;',
            464 => '&#62379;',
            465 => '&#62380;',
            466 => '&#62381;',
            467 => '&#62382;',
            468 => '&#62383;',
            469 => '&#62384;',
            470 => '&#62385;',
            471 => '&#62386;',
            472 => '&#62387;',
            473 => '&#62388;',
            474 => '&#62389;',
            475 => '&#62390;',
            476 => '&#62391;',
            477 => '&#62392;',
            478 => '&#62393;',
            479 => '&#62394;',
            480 => '&#62395;',
            481 => '&#62396;',
            482 => '&#62397;',
            483 => '&#62398;',
            484 => '&#62399;',
            485 => '&#62400;',
            486 => '&#62401;',
            487 => '&#62402;',
            488 => '&#62403;',
            489 => '&#62404;',
            490 => '&#62405;',
            491 => '&#62406;',
            492 => '&#62407;',
            493 => '&#62408;',
            494 => '&#62409;',
            495 => '&#62410;',
            496 => '&#62411;',
            497 => '&#62412;',
            498 => '&#62413;',
            499 => '&#62414;',
            500 => '&#63442;',
            501 => '&#63443;',
            502 => '&#63444;',
            503 => '&#63445;',
            504 => '&#63446;',
            505 => '&#63447;',
            506 => '&#63448;',
            507 => '&#63449;',
            508 => '&#63450;',
            509 => '&#63451;',
            510 => '&#63452;',
            511 => '&#63453;',
            512 => '&#63454;',
            513 => '&#63455;',
            514 => '&#63456;',
            515 => '&#63457;',
            516 => '&#63458;',
            517 => '&#63459;',
            518 => '&#63460;',
            700 => '&#62415;',
            701 => '&#62416;',
            702 => '&#62417;',
            703 => '&#62418;',
            704 => '&#62419;',
            705 => '&#62420;',
            706 => '&#62421;',
            707 => '&#62422;',
            708 => '&#62423;',
            709 => '&#62424;',
            710 => '&#62425;',
            711 => '&#62426;',
            712 => '&#62427;',
            713 => '&#62428;',
            714 => '&#62429;',
            715 => '&#62430;',
            716 => '&#62431;',
            717 => '&#62432;',
            718 => '&#62433;',
            719 => '&#62434;',
            720 => '&#62435;',
            721 => '&#62436;',
            722 => '&#62437;',
            723 => '&#62438;',
            724 => '&#62439;',
            725 => '&#62440;',
            726 => '&#62441;',
            727 => '&#62442;',
            728 => '&#62443;',
            729 => '&#62444;',
            730 => '&#62445;',
            731 => '&#62446;',
            732 => '&#62447;',
            733 => '&#62448;',
            734 => '&#62449;',
            735 => '&#62450;',
            736 => '&#62451;',
            737 => '&#62452;',
            738 => '&#62453;',
            739 => '&#62454;',
            740 => '&#62455;',
            741 => '&#62456;',
            742 => '&#62457;',
            743 => '&#62458;',
            744 => '&#62459;',
            745 => '&#62460;',
            746 => '&#62528;',
            747 => '&#62529;',
            748 => '&#62530;',
            749 => '&#62531;',
            750 => '&#62532;',
            751 => '&#62533;',
            752 => '&#62534;',
            753 => '&#62535;',
            754 => '&#62536;',
            755 => '&#62537;',
            756 => '&#62538;',
            757 => '&#62539;',
            758 => '&#62540;',
            759 => '&#62541;',
            760 => '&#62542;',
            761 => '&#62543;',
            762 => '&#62544;',
            763 => '&#62545;',
            764 => '&#62546;',
            765 => '&#62547;',
            766 => '&#62548;',
            767 => '&#62549;',
            768 => '&#62550;',
            769 => '&#62551;',
            770 => '&#62552;',
            771 => '&#62553;',
            772 => '&#62554;',
            773 => '&#62555;',
            774 => '&#62556;',
            775 => '&#62557;',
            776 => '&#62558;',
            777 => '&#62559;',
            778 => '&#62560;',
            779 => '&#62561;',
            780 => '&#62562;',
            781 => '&#62563;',
            782 => '&#62564;',
            783 => '&#62565;',
            784 => '&#62566;',
            785 => '&#62567;',
            786 => '&#62568;',
            787 => '&#62569;',
            788 => '&#62570;',
            789 => '&#62571;',
            790 => '&#62572;',
            791 => '&#62573;',
            792 => '&#62574;',
            793 => '&#62575;',
            794 => '&#62576;',
            795 => '&#62577;',
            796 => '&#62578;',
            797 => '&#62579;',
            798 => '&#62580;',
            799 => '&#62581;',
            800 => '&#62582;',
            801 => '&#62583;',
            802 => '&#62584;',
            803 => '&#62585;',
            804 => '&#62586;',
            805 => '&#62587;',
            806 => '&#62588;',
            807 => '&#62589;',
            808 => '&#62590;',
            809 => '&#62592;',
            810 => '&#62593;',
            811 => '&#62594;',
            812 => '&#62595;',
            813 => '&#62596;',
            814 => '&#62597;',
            815 => '&#62598;',
            816 => '&#62599;',
            817 => '&#62600;',
            818 => '&#62601;',
            819 => '&#62602;',
            820 => '&#62603;',
            821 => '&#62604;',
            822 => '&#62605;'
        );
        return $table;
    }

    /**
     * EZ絵文字エンティティ絵文字番号変換テーブル
     *
     * @return array
     * @static
     */
    public static function getAuEntity2NumArray()
    {
        $table = array(
            '&#63065;' => 1,
            '&#63066;' => 2,
            '&#63067;' => 3,
            '&#63304;' => 4,
            '&#63305;' => 5,
            '&#63306;' => 6,
            '&#63307;' => 7,
            '&#63308;' => 8,
            '&#63309;' => 9,
            '&#63310;' => 10,
            '&#63311;' => 11,
            '&#63130;' => 12,
            '&#63210;' => 13,
            '&#63382;' => 14,
            '&#63070;' => 15,
            '&#63071;' => 16,
            '&#63312;' => 17,
            '&#63313;' => 18,
            '&#63314;' => 19,
            '&#63315;' => 20,
            '&#63316;' => 21,
            '&#63317;' => 22,
            '&#63318;' => 23,
            '&#63319;' => 24,
            '&#63383;' => 25,
            '&#63320;' => 26,
            '&#63321;' => 27,
            '&#63322;' => 28,
            '&#63323;' => 29,
            '&#63324;' => 30,
            '&#63325;' => 31,
            '&#63326;' => 32,
            '&#63327;' => 33,
            '&#63328;' => 34,
            '&#63329;' => 35,
            '&#63330;' => 36,
            '&#63331;' => 37,
            '&#63332;' => 38,
            '&#63333;' => 39,
            '&#63334;' => 40,
            '&#63335;' => 41,
            '&#63336;' => 42,
            '&#63337;' => 43,
            '&#63072;' => 44,
            '&#63123;' => 45,
            '&#63409;' => 46,
            '&#63073;' => 47,
            '&#63211;' => 48,
            '&#63356;' => 49,
            '&#63187;' => 50,
            '&#63410;' => 51,
            '&#63131;' => 52,
            '&#63212;' => 53,
            '&#63338;' => 54,
            '&#63339;' => 55,
            '&#63357;' => 56,
            '&#63384;' => 57,
            '&#63060;' => 58,
            '&#63358;' => 59,
            '&#63074;' => 60,
            '&#63340;' => 61,
            '&#63341;' => 62,
            '&#63342;' => 63,
            '&#63343;' => 64,
            '&#63132;' => 65,
            '&#63344;' => 66,
            '&#63360;' => 67,
            '&#63188;' => 68,
            '&#63075;' => 69,
            '&#63345;' => 70,
            '&#63346;' => 71,
            '&#63213;' => 72,
            '&#63347;' => 73,
            '&#63160;' => 74,
            '&#63040;' => 75,
            '&#63044;' => 76,
            '&#63054;' => 77,
            '&#63161;' => 78,
            '&#63404;' => 79,
            '&#63189;' => 80,
            '&#63348;' => 81,
            '&#63349;' => 82,
            '&#63092;' => 83,
            '&#63405;' => 84,
            '&#63411;' => 85,
            '&#63190;' => 86,
            '&#63385;' => 87,
            '&#63350;' => 88,
            '&#63351;' => 89,
            '&#63376;' => 90,
            '&#63093;' => 91,
            '&#63361;' => 92,
            '&#63412;' => 93,
            '&#63214;' => 94,
            '&#63076;' => 95,
            '&#63124;' => 96,
            '&#63362;' => 97,
            '&#63068;' => 98,
            '&#63042;' => 99,
            '&#63363;' => 100,
            '&#63364;' => 101,
            '&#63365;' => 102,
            '&#63366;' => 103,
            '&#63215;' => 104,
            '&#63367;' => 105,
            '&#63094;' => 106,
            '&#63077;' => 107,
            '&#63226;' => 108,
            '&#63386;' => 109,
            '&#63216;' => 110,
            '&#63387;' => 111,
            '&#63108;' => 112,
            '&#63165;' => 113,
            '&#63388;' => 114,
            '&#63389;' => 115,
            '&#63191;' => 116,
            '&#63352;' => 117,
            '&#63353;' => 118,
            '&#63217;' => 119,
            '&#63218;' => 120,
            '&#63368;' => 121,
            '&#63095;' => 122,
            '&#63390;' => 123,
            '&#63219;' => 124,
            '&#63114;' => 125,
            '&#63391;' => 126,
            '&#63377;' => 127,
            '&#63378;' => 128,
            '&#63220;' => 129,
            '&#63392;' => 130,
            '&#63369;' => 131,
            '&#63354;' => 132,
            '&#63143;' => 133,
            '&#63162;' => 134,
            '&#63393;' => 135,
            '&#63355;' => 136,
            '&#63370;' => 137,
            '&#63221;' => 138,
            '&#63394;' => 139,
            '&#63192;' => 140,
            '&#63193;' => 141,
            '&#63371;' => 142,
            '&#63096;' => 143,
            '&#63144;' => 144,
            '&#63222;' => 145,
            '&#63109;' => 146,
            '&#63372;' => 147,
            '&#63115;' => 148,
            '&#63097;' => 149,
            '&#63395;' => 150,
            '&#63406;' => 151,
            '&#63396;' => 152,
            '&#63407;' => 153,
            '&#63408;' => 154,
            '&#63223;' => 155,
            '&#63110;' => 156,
            '&#63373;' => 157,
            '&#63098;' => 158,
            '&#63379;' => 159,
            '&#63133;' => 160,
            '&#63397;' => 161,
            '&#63398;' => 162,
            '&#63194;' => 163,
            '&#63399;' => 164,
            '&#63224;' => 165,
            '&#63225;' => 166,
            '&#63078;' => 167,
            '&#63116;' => 168,
            '&#63117;' => 169,
            '&#63137;' => 170,
            '&#63400;' => 171,
            '&#63118;' => 172,
            '&#63401;' => 173,
            '&#63402;' => 174,
            '&#63403;' => 175,
            '&#63061;' => 176,
            '&#63062;' => 177,
            '&#63063;' => 178,
            '&#63064;' => 179,
            '&#63227;' => 180,
            '&#63228;' => 181,
            '&#63296;' => 182,
            '&#63297;' => 183,
            '&#63298;' => 184,
            '&#63299;' => 185,
            '&#63300;' => 186,
            '&#63301;' => 187,
            '&#63302;' => 188,
            '&#63303;' => 189,
            '&#63041;' => 190,
            '&#63069;' => 191,
            '&#63079;' => 192,
            '&#63080;' => 193,
            '&#63081;' => 194,
            '&#63082;' => 195,
            '&#63083;' => 196,
            '&#63084;' => 197,
            '&#63085;' => 198,
            '&#63086;' => 199,
            '&#63087;' => 200,
            '&#63088;' => 201,
            '&#63089;' => 202,
            '&#63090;' => 203,
            '&#63091;' => 204,
            '&#63099;' => 205,
            '&#63100;' => 206,
            '&#63101;' => 207,
            '&#63102;' => 208,
            '&#63104;' => 209,
            '&#63105;' => 210,
            '&#63106;' => 211,
            '&#63107;' => 212,
            '&#63374;' => 213,
            '&#63375;' => 214,
            '&#63111;' => 215,
            '&#63112;' => 216,
            '&#63113;' => 217,
            '&#63043;' => 218,
            '&#63119;' => 219,
            '&#63120;' => 220,
            '&#63121;' => 221,
            '&#63122;' => 222,
            '&#63045;' => 223,
            '&#63125;' => 224,
            '&#63126;' => 225,
            '&#63127;' => 226,
            '&#63128;' => 227,
            '&#63129;' => 228,
            '&#63046;' => 229,
            '&#63047;' => 230,
            '&#63134;' => 231,
            '&#63135;' => 232,
            '&#63136;' => 233,
            '&#63138;' => 234,
            '&#63139;' => 235,
            '&#63140;' => 236,
            '&#63141;' => 237,
            '&#63142;' => 238,
            '&#63145;' => 239,
            '&#63146;' => 240,
            '&#63147;' => 241,
            '&#63148;' => 242,
            '&#63149;' => 243,
            '&#63150;' => 244,
            '&#63151;' => 245,
            '&#63048;' => 246,
            '&#63152;' => 247,
            '&#63153;' => 248,
            '&#63154;' => 249,
            '&#63155;' => 250,
            '&#63156;' => 251,
            '&#63157;' => 252,
            '&#63158;' => 253,
            '&#63159;' => 254,
            '&#63163;' => 255,
            '&#63164;' => 256,
            '&#63049;' => 257,
            '&#63050;' => 258,
            '&#63051;' => 259,
            '&#63052;' => 260,
            '&#63053;' => 261,
            '&#63166;' => 262,
            '&#63167;' => 263,
            '&#63168;' => 264,
            '&#63055;' => 265,
            '&#63056;' => 266,
            '&#63057;' => 267,
            '&#63058;' => 268,
            '&#63059;' => 269,
            '&#63169;' => 270,
            '&#63170;' => 271,
            '&#63171;' => 272,
            '&#63172;' => 273,
            '&#63173;' => 274,
            '&#63174;' => 275,
            '&#63175;' => 276,
            '&#63176;' => 277,
            '&#63177;' => 278,
            '&#63178;' => 279,
            '&#63179;' => 280,
            '&#63180;' => 281,
            '&#63181;' => 282,
            '&#63182;' => 283,
            '&#63183;' => 284,
            '&#63184;' => 285,
            '&#63185;' => 286,
            '&#63186;' => 287,
            '&#63195;' => 288,
            '&#63196;' => 289,
            '&#63197;' => 290,
            '&#63198;' => 291,
            '&#63199;' => 292,
            '&#63200;' => 293,
            '&#63201;' => 294,
            '&#63202;' => 295,
            '&#63203;' => 296,
            '&#63204;' => 297,
            '&#63380;' => 298,
            '&#63381;' => 299,
            '&#63205;' => 300,
            '&#63206;' => 301,
            '&#63207;' => 302,
            '&#63208;' => 303,
            '&#63209;' => 304,
            '&#63413;' => 305,
            '&#63414;' => 306,
            '&#63415;' => 307,
            '&#63416;' => 308,
            '&#63417;' => 309,
            '&#63418;' => 310,
            '&#63419;' => 311,
            '&#63420;' => 312,
            '&#63421;' => 313,
            '&#63422;' => 314,
            '&#63423;' => 315,
            '&#63424;' => 316,
            '&#63425;' => 317,
            '&#63426;' => 318,
            '&#63427;' => 319,
            '&#63428;' => 320,
            '&#63429;' => 321,
            '&#63430;' => 322,
            '&#63431;' => 323,
            '&#63432;' => 324,
            '&#63433;' => 325,
            '&#63434;' => 326,
            '&#63435;' => 327,
            '&#63436;' => 328,
            '&#63437;' => 329,
            '&#63438;' => 330,
            '&#63439;' => 331,
            '&#63440;' => 332,
            '&#63441;' => 333,
            '&#63461;' => 334,
            '&#63462;' => 335,
            '&#63463;' => 336,
            '&#63464;' => 337,
            '&#63465;' => 338,
            '&#63466;' => 339,
            '&#63467;' => 340,
            '&#63468;' => 341,
            '&#63469;' => 342,
            '&#63470;' => 343,
            '&#63471;' => 344,
            '&#63472;' => 345,
            '&#63473;' => 346,
            '&#63474;' => 347,
            '&#63475;' => 348,
            '&#63476;' => 349,
            '&#63477;' => 350,
            '&#63478;' => 351,
            '&#63479;' => 352,
            '&#63480;' => 353,
            '&#63481;' => 354,
            '&#63482;' => 355,
            '&#63483;' => 356,
            '&#63484;' => 357,
            '&#62272;' => 358,
            '&#62273;' => 359,
            '&#62274;' => 360,
            '&#62275;' => 361,
            '&#62276;' => 362,
            '&#62277;' => 363,
            '&#62278;' => 364,
            '&#62279;' => 365,
            '&#62280;' => 366,
            '&#62281;' => 367,
            '&#62282;' => 368,
            '&#62283;' => 369,
            '&#62284;' => 370,
            '&#62285;' => 371,
            '&#62286;' => 372,
            '&#62287;' => 373,
            '&#62288;' => 374,
            '&#62289;' => 375,
            '&#62290;' => 376,
            '&#62291;' => 377,
            '&#62292;' => 378,
            '&#62293;' => 379,
            '&#62294;' => 380,
            '&#62295;' => 381,
            '&#62296;' => 382,
            '&#62297;' => 383,
            '&#62298;' => 384,
            '&#62299;' => 385,
            '&#62300;' => 386,
            '&#62301;' => 387,
            '&#62302;' => 388,
            '&#62303;' => 389,
            '&#62304;' => 390,
            '&#62305;' => 391,
            '&#62306;' => 392,
            '&#62307;' => 393,
            '&#62308;' => 394,
            '&#62309;' => 395,
            '&#62310;' => 396,
            '&#62311;' => 397,
            '&#62312;' => 398,
            '&#62313;' => 399,
            '&#62314;' => 400,
            '&#62315;' => 401,
            '&#62316;' => 402,
            '&#62317;' => 403,
            '&#62318;' => 404,
            '&#62319;' => 405,
            '&#62320;' => 406,
            '&#62321;' => 407,
            '&#62322;' => 408,
            '&#62323;' => 409,
            '&#62324;' => 410,
            '&#62325;' => 411,
            '&#62326;' => 412,
            '&#62327;' => 413,
            '&#62328;' => 414,
            '&#62329;' => 415,
            '&#62330;' => 416,
            '&#62331;' => 417,
            '&#62332;' => 418,
            '&#62333;' => 419,
            '&#62334;' => 420,
            '&#62336;' => 421,
            '&#62337;' => 422,
            '&#62338;' => 423,
            '&#62339;' => 424,
            '&#62340;' => 425,
            '&#62341;' => 426,
            '&#62342;' => 427,
            '&#62343;' => 428,
            '&#62344;' => 429,
            '&#62345;' => 430,
            '&#62346;' => 431,
            '&#62347;' => 432,
            '&#62348;' => 433,
            '&#62349;' => 434,
            '&#62350;' => 435,
            '&#62351;' => 436,
            '&#62352;' => 437,
            '&#62353;' => 438,
            '&#62354;' => 439,
            '&#62355;' => 440,
            '&#62356;' => 441,
            '&#62357;' => 442,
            '&#62358;' => 443,
            '&#62359;' => 444,
            '&#62360;' => 445,
            '&#62361;' => 446,
            '&#62362;' => 447,
            '&#62363;' => 448,
            '&#62364;' => 449,
            '&#62365;' => 450,
            '&#62366;' => 451,
            '&#62367;' => 452,
            '&#62368;' => 453,
            '&#62369;' => 454,
            '&#62370;' => 455,
            '&#62371;' => 456,
            '&#62372;' => 457,
            '&#62373;' => 458,
            '&#62374;' => 459,
            '&#62375;' => 460,
            '&#62376;' => 461,
            '&#62377;' => 462,
            '&#62378;' => 463,
            '&#62379;' => 464,
            '&#62380;' => 465,
            '&#62381;' => 466,
            '&#62382;' => 467,
            '&#62383;' => 468,
            '&#62384;' => 469,
            '&#62385;' => 470,
            '&#62386;' => 471,
            '&#62387;' => 472,
            '&#62388;' => 473,
            '&#62389;' => 474,
            '&#62390;' => 475,
            '&#62391;' => 476,
            '&#62392;' => 477,
            '&#62393;' => 478,
            '&#62394;' => 479,
            '&#62395;' => 480,
            '&#62396;' => 481,
            '&#62397;' => 482,
            '&#62398;' => 483,
            '&#62399;' => 484,
            '&#62400;' => 485,
            '&#62401;' => 486,
            '&#62402;' => 487,
            '&#62403;' => 488,
            '&#62404;' => 489,
            '&#62405;' => 490,
            '&#62406;' => 491,
            '&#62407;' => 492,
            '&#62408;' => 493,
            '&#62409;' => 494,
            '&#62410;' => 495,
            '&#62411;' => 496,
            '&#62412;' => 497,
            '&#62413;' => 498,
            '&#62414;' => 499,
            '&#63442;' => 500,
            '&#63443;' => 501,
            '&#63444;' => 502,
            '&#63445;' => 503,
            '&#63446;' => 504,
            '&#63447;' => 505,
            '&#63448;' => 506,
            '&#63449;' => 507,
            '&#63450;' => 508,
            '&#63451;' => 509,
            '&#63452;' => 510,
            '&#63453;' => 511,
            '&#63454;' => 512,
            '&#63455;' => 513,
            '&#63456;' => 514,
            '&#63457;' => 515,
            '&#63458;' => 516,
            '&#63459;' => 517,
            '&#63460;' => 518,
            '&#62415;' => 700,
            '&#62416;' => 701,
            '&#62417;' => 702,
            '&#62418;' => 703,
            '&#62419;' => 704,
            '&#62420;' => 705,
            '&#62421;' => 706,
            '&#62422;' => 707,
            '&#62423;' => 708,
            '&#62424;' => 709,
            '&#62425;' => 710,
            '&#62426;' => 711,
            '&#62427;' => 712,
            '&#62428;' => 713,
            '&#62429;' => 714,
            '&#62430;' => 715,
            '&#62431;' => 716,
            '&#62432;' => 717,
            '&#62433;' => 718,
            '&#62434;' => 719,
            '&#62435;' => 720,
            '&#62436;' => 721,
            '&#62437;' => 722,
            '&#62438;' => 723,
            '&#62439;' => 724,
            '&#62440;' => 725,
            '&#62441;' => 726,
            '&#62442;' => 727,
            '&#62443;' => 728,
            '&#62444;' => 729,
            '&#62445;' => 730,
            '&#62446;' => 731,
            '&#62447;' => 732,
            '&#62448;' => 733,
            '&#62449;' => 734,
            '&#62450;' => 735,
            '&#62451;' => 736,
            '&#62452;' => 737,
            '&#62453;' => 738,
            '&#62454;' => 739,
            '&#62455;' => 740,
            '&#62456;' => 741,
            '&#62457;' => 742,
            '&#62458;' => 743,
            '&#62459;' => 744,
            '&#62460;' => 745,
            '&#62528;' => 746,
            '&#62529;' => 747,
            '&#62530;' => 748,
            '&#62531;' => 749,
            '&#62532;' => 750,
            '&#62533;' => 751,
            '&#62534;' => 752,
            '&#62535;' => 753,
            '&#62536;' => 754,
            '&#62537;' => 755,
            '&#62538;' => 756,
            '&#62539;' => 757,
            '&#62540;' => 758,
            '&#62541;' => 759,
            '&#62542;' => 760,
            '&#62543;' => 761,
            '&#62544;' => 762,
            '&#62545;' => 763,
            '&#62546;' => 764,
            '&#62547;' => 765,
            '&#62548;' => 766,
            '&#62549;' => 767,
            '&#62550;' => 768,
            '&#62551;' => 769,
            '&#62552;' => 770,
            '&#62553;' => 771,
            '&#62554;' => 772,
            '&#62555;' => 773,
            '&#62556;' => 774,
            '&#62557;' => 775,
            '&#62558;' => 776,
            '&#62559;' => 777,
            '&#62560;' => 778,
            '&#62561;' => 779,
            '&#62562;' => 780,
            '&#62563;' => 781,
            '&#62564;' => 782,
            '&#62565;' => 783,
            '&#62566;' => 784,
            '&#62567;' => 785,
            '&#62568;' => 786,
            '&#62569;' => 787,
            '&#62570;' => 788,
            '&#62571;' => 789,
            '&#62572;' => 790,
            '&#62573;' => 791,
            '&#62574;' => 792,
            '&#62575;' => 793,
            '&#62576;' => 794,
            '&#62577;' => 795,
            '&#62578;' => 796,
            '&#62579;' => 797,
            '&#62580;' => 798,
            '&#62581;' => 799,
            '&#62582;' => 800,
            '&#62583;' => 801,
            '&#62584;' => 802,
            '&#62585;' => 803,
            '&#62586;' => 804,
            '&#62587;' => 805,
            '&#62588;' => 806,
            '&#62589;' => 807,
            '&#62590;' => 808,
            '&#62592;' => 809,
            '&#62593;' => 810,
            '&#62594;' => 811,
            '&#62595;' => 812,
            '&#62596;' => 813,
            '&#62597;' => 814,
            '&#62598;' => 815,
            '&#62599;' => 816,
            '&#62600;' => 817,
            '&#62601;' => 818,
            '&#62602;' => 819,
            '&#62603;' => 820,
            '&#62604;' => 821,
            '&#62605;' => 822
        );
        return $table;
    }
}
